10 '**************************************************************************
20 'Chapter 10, Example 1: 1 dimensional Schrodinger Eq. for HO potential
30 'COMPUTATIONAL PHYSICS: An Introduction
40 'by R. C. Verma, P. K. Ahluwalia and K. C. Sharma
50 '   * * * *    
60 '**************************************************************************
70 '
80 GOSUB 10000                                                    'show header
90 '
100 DEF FNPOT (X) = .5 * X * X                      'define potential function
110 '
120 FLAG = 0                                           'set flag for later use
130 '
140 GOSUB 1000                   'introduce the problem & get input parameters
150 GOSUB 2000               'solve the Schrodinger Eq. using half-step method
155 '
160 PRINT : BEEP: PRINT "press any key to continue"
165 IF INKEY$ = "" THEN 165
170 CLS
180 IF PSIOLD * PSI >= 0 THEN 210
190 LOCATE 12, 15: BEEP: PRINT "Energy lies between "; EOLD; " & "; EGUESS
200 '
210 BEEP: LOCATE 14, 10: INPUT "do you want to try another guess Y/N "; ANS$
215  IF LEFT$(ANS$, 1) = "N" OR LEFT$(ANS$, 1) = "n" THEN 280
220  IF LEFT$(ANS$, 1) = "y" OR LEFT$(ANS$, 1) = "Y" THEN 240
225  LOCATE 14, 1: PRINT SPACE$(75): GOTO 210
230 '
240 FLAG = 1                                                      'change flag
250 EOLD = EGUESS: PSIOLD = PSI
260 CLS : GOTO 140
270 '
280 CLS : LOCATE 12, 15: BEEP: PRINT " Thank you !"
290 END
300 '
1000 '************************************************************************
1010 'subroutine to introduce the problem & to get input variables
1020 'input variables:  none
1030 'output variables: XMIN, XMAX, PAR$, EGUESS, H, PSI, PSI1, term%
1040 'global variables: none
1050 'local variables:  none
1060 'test cases: XMAX=5: PAR$=ODD, EGUESS=0.45
1070 '************************************************************************
1080 '
1090 IF FLAG <> 0 THEN 1320
1100 '
1110 LOCATE 2, 10: PRINT "This program solves the Schrodinger equation"
1120 LOCATE 3, 15: PRINT "for potential V(x) =a x ^ 2 in one dimension"
1130 LOCATE 4, 10: PRINT "----------------------------------------------------"
1140 LOCATE 6, 10: PRINT "Please give the following parameters:"
1150 '
1160 XMIN = 0                                          'initial position fixed
1170 LOCATE 8, 10: PRINT "We fix initial position: Xmin = 0"
1180 LOCATE 10, 5: INPUT "Give final position: Xmax ="; XMAX: PRINT
1190  IF XMAX > 0 THEN 1220
1200  BEEP: LOCATE 10, 5: PRINT SPACE$(70): GOTO 1180
1210 '
1220 LOCATE 12, 5: INPUT "Give step size h  < 1 = "; H
1230  IF H > 0 AND H < 1 THEN 1260
1240  BEEP: LOCATE 12, 5: PRINT SPACE$(78): GOTO 1220
1250 '
1260  TERM% = (XMAX - XMIN) / (20 * H)
1270                                 'defined for selective printing of result
1280 LOCATE 14, 5: INPUT " Choose parity of state desired odd/even = "; PAR$
1290  IF PAR$ = "odd" OR PAR$ = "ODD" OR PAR$ = "even" OR PAR$ = "EVEN" THEN 1320
1300  BEEP: LOCATE 14, 5: PRINT SPACE$(70): GOTO 1280
1310 '
1320  LOCATE 22, 1: PRINT SPACE$(75)
1330  LOCATE 22, 1: INPUT "Give guess for energy eigenvalue "; EGUESS
1340 '
1350  IF PAR$ = "odd" OR PAR$ = "ODD" THEN PSI = 1: PSI1 = 0'initial conditions
1360  IF PAR$ = "even" OR PAR$ = "EVEN" THEN PSI = 0: PSI1 = 1'for odd/even state
1370 '
1380 RETURN
1390 '
2000 '************************************************************************
2010 'subroutine to solve the Schrodinger equation using energy guess
2020 'input variables: EGUESS, XMIN, XMAX, FNPOT(), H
2030 'output variables: X, POT, PSI
2040 'global variables: none
2050 'local variables: PSI1, PSI2
2060 '************************************************************************
2070 '
2080 X = XMIN: E = EGUESS: COUNT = 0                     'initialize variables
2090 '
2100 WHILE X <= XMAX
2110   POT = FNPOT(X)                         'find potential at this position
2120   PSI2 = 2 * (FNPOT(X + H / 2) - E) * (PSI + .5 * PSI1 * H)
2130                                      'find second derivative at half step
2140   PSI = PSI + (PSI1 + .5 * PSI2 * H) * H   'new wavefunction half stepped
2150   PSI1 = PSI1 + PSI2 * H            'new first derivative of wavefunction
2160    IF COUNT MOD TERM% <> 0 THEN 2180              'for selective printing
2170   PRINT USING "x = ##.###, v(x)=###.####  psi = #.#####^^^^"; X; POT; PSI
2180   X = X + H                                                 'new position
2190   COUNT = COUNT + 1                                    'increment counter
2200 WEND
2210 '
2220 RETURN
2230 '
10000 '***********************************************************************
10010 'subroutine to display header screen
10020 'input variables:  none
10030 'output variables: none
10040 'global variables: none
10050 'local variables:  none
10060 '***********************************************************************
10070 CLS : KEY OFF                                           'clear screen
10080 SCREEN 2, 0, 0: WIDTH 80                               'change screen
10090  '
10100 LINE (1, 1)-(638, 40), , B                                'draw boxes
10110 LINE (5, 3)-(633, 37), , B
10120 LINE (70, 50)-(570, 140), , B
10130 LINE (75, 53)-(565, 137), , B
10140                                           'print title, authors, etc.
10150 LOCATE 3, 7
10160 PRINT " C O M P U T A T I O N A L   P H Y S I C S :";
10170 PRINT " (An  Introduction)"
10180 LOCATE 9, 40
10190 PRINT "by";
10200 LOCATE 11, 20: PRINT "R.C. Verma, P.K. Ahluwalia & K.C. Sharma"
10210 LOCATE 13, 34
10220 PRINT "   * * * *"
10230 '
10240 BEEP
10250 LOCATE 15, 35: PRINT "* * * * * *"
10260 LOCATE 22, 25: PRINT "press any key to continue"
10270 LOCATE 23, 25: PRINT "-------------------------"
10280 '
10290 IF INKEY$ = "" THEN 10290                            'hold the screen
10300 CLS                                                     'clear screen
10310 RETURN
10320 '

