10 '**************************************************************************
20 'Chapter 10, Example 2: Solution of Schrodinger Equation for
30 '                       Harmonic Oscillator using Runge-Kutta method
40 'COMPUTATIONAL PHYSICS: An Introduction
50 'by R. C. Verma, P. K. Ahluwalia and K. C. Sharma
60 '   * * * *   
70 DEF FNPOT (X) = .5 * X * X    'define potential function
80 '**************************************************************************
90 '
100 GOSUB 10000                                                   'show header
110 '
120 FLAG = 0                                                         'set flag
130 '
140 GOSUB 1000                   'introduce the problem & get input parameters
150 '
160 IF FLAG <> 0 THEN 190                              'jump if not first time
170 '
180 GOSUB 4000                                       'draw axes and set scales
190 GOSUB 2000                                     'find and plot wavefunction
200 FLAG = 1                                          'set flag after graphics
210 '
220 BEEP: LOCATE 23, 1: PRINT SPACE$(75)
230  LOCATE 23, 1: INPUT "Do you want to try another guess Y/N "; ANS$
240  IF LEFT$(ANS$, 1) = "y" OR LEFT$(ANS$, 1) = "Y" THEN 260
245  IF LEFT$(ANS$, 1) = "N" OR LEFT$(ANS$, 1) = "n" THEN 290
250  LOCATE 23, 1: BEEP: PRINT SPACE$(75): GOTO 230
255 '
260  EOLD = EGUESS: PSIOLD = PSI                    'remember the last results
270  GOTO 140                                        'another input for energy
280 '
290 CLS : LOCATE 12, 15: BEEP: PRINT " Thank you !"
300 END
310 '
1000 '************************************************************************
1010 'subroutine to introduce the problem & to get input variables
1020 'input variables:  none
1030 'output variables: XMIN, XMAX, PSIMIN, PSIMAX,
1040 '                  PAR$, EGUESS, H, PSI, PSI1, term%
1050 'global variables: none
1060 'local variables:  none
1070 'test cases: XMAX=5: PAR$=ODD, EGUESS=0.45
1080 '************************************************************************
1090 '
1100 IF FLAG <> 0 THEN 1340                               'for selective input
1110 '
1120 LOCATE 1, 5: PRINT "This program solves Schrodinger equation"
1130 LOCATE 2, 10: PRINT " for 1-dim. simple harmonic potential "
1140 LOCATE 4, 15: PRINT " V(x) =1/2 x^2 "
1150 LOCATE 5, 5: PRINT " (units: h/2*pi = 1, m = 1, k = 1) "
1160 LOCATE 7, 10: PRINT "Please give the following parameters:"
1170 '
1180 XMIN = 0                                          'initial position fixed
1190 LOCATE 9, 10: PRINT "We fix initial position: Xmin = 0"
1200 LOCATE 11, 5: INPUT "Give final position: Xmax ="; XMAX
1210  IF XMAX > 0 THEN 1240
1220  BEEP: LOCATE 11, 5: PRINT SPACE$(70): GOTO 1200
1230 '
1240 LOCATE 13, 5: INPUT "Give step size h  < 1 "; H
1250  IF H > 0 AND H < 1 THEN 1280
1260  BEEP: LOCATE 13, 5: PRINT SPACE$(70): GOTO 1240
1270 '
1280  TERM% = (XMAX - XMIN) / (20 * H)
1290                                 'defined for selective printing of result
1300 LOCATE 15, 5: INPUT " Choose parity of state desired odd/even = "; PAR$
1310  IF PAR$ = "odd" OR PAR$ = "ODD" OR PAR$ = "even" OR PAR$ = "EVEN" THEN 1340
1320  BEEP: LOCATE 15, 5: PRINT SPACE$(70): GOTO 1300
1330 '
1340  LOCATE 23, 1: PRINT SPACE$(75)
1350  LOCATE 23, 1: INPUT "Give guess for energy eigenvalue "; EGUESS
1360 '
1370  IF PAR$ = "odd" OR PAR$ = "ODD" THEN PSI = 1: PSI1 = 0'initial conditions
1380  IF PAR$ = "even" OR PAR$ = "EVEN" THEN PSI = 0: PSI1 = 1'for odd/even state
1390 '
1400 PSIMIN = -2: PSIMAX = 2               'range of wavefunction for graphics
1410 '
1420 RETURN
1430 '
2000 '************************************************************************
2010 'subroutine to do numerical work
2020 'input variables: XMIN, EGUESS, PSI1, FNPOT(), H, PSIOLD
2030 'output variables: X, PSI,
2040 'global variables: none
2050 'local variables: K1, K2, K3, K4, P1, P2, P3, P4, PSI1
2060 '************************************************************************
2070 '
2080 X = XMIN: E = EGUESS:                       'initialize position & energy
2090 '
2100 WHILE X <= XMAX AND ABS(PSI) <= PSIMAX             'start loop over steps
2110    GOSUB 3000                                      'draw the wavefunction
2120   K1 = PSI1                                   'find Runge-Kutta variables
2130    P1 = 2 * (FNPOT(X) - E) * PSI
2140   K2 = PSI1 + .5 * P1 * H
2150    P2 = 2 * (FNPOT(X + H / 2) - E) * (PSI + .5 * K1 * H)
2160   K3 = PSI1 + .5 * P2 * H
2170    P3 = 2 * (FNPOT(X + H / 2) - E) * (PSI + .5 * K2 * H)
2180   K4 = PSI1 + .5 * P3 * H
2190    P4 = 2 * (FNPOT(X + H) - E) * (PSI + K3 * H)
2200 '
2210    PSI = PSI + (K1 + 2 * K2 + 2 * K3 + K4) * H / 6      'new wavefunction
2220    PSI1 = PSI1 + (P1 + 2 * P2 + 2 * P3 + P4) * H / 6'new first derivative of wavefn.
2230    X = X + H                                                'new position
2240 WEND
2250 '
2260 IF PSIOLD * PSI >= 0 THEN 2310             'check if new wavefn. diverges
2270           'in a direction opposite to the old one to find energy interval
2280 LOCATE 2, 2: PRINT SPACE$(75)
2290 LOCATE 2, 12
2300 PRINT USING "Energy lies between +##.#### and +##.#### "; EOLD; EGUESS
2310 '
2320 RETURN
2330 '
3000 '************************************************************************
3010 'subroutine to display wavefunction
3020 'input variables: X, XMIN, PSI, KCOEFF1, KCOEFF2, KCOEFF3, KCOEFF4
3030 'output variables: none
3040 'global variables: none
3050 'local variables: PX%, PY%
3060 '************************************************************************
3070                                            'perform scale transformations
3080 PX% = KCOEFF1 * X + KCOEFF2                            'for x-coordinates
3090 PY% = KCOEFF3 * PSI + KCOEFF4                           'for wavefunction
3100 '
3110 IF X = XMIN THEN PSET (PX%, PY%) ELSE LINE -(PX%, PY%) 'draw wavefunction
3120 '
3130 RETURN
3140 '
4000 '************************************************************************
4010 'subroutine to draw box for graph & find scales
4020 'input variables: XMIN, XMAX, PSIMIN, PSIMAX
4030 'output variables: KCOEFF1, KCOEFF2, KCOEFF3 ,KCOEFF4,
4040 'global variables: none
4050 'local variables: YMIN, YMAX, ROWTIT%, COLTIT%
4060 '             DIV%, XSCALE, YSCALE, PXSCALE, PYSCALE
4070 '             PMX%(J%), PMY%(J%), ROWX%, COLX%, ROWY%, COLY%
4080 '             XTITLE$, YTITLE$, SUBTITLE$, PXMIN%, PXMAX%, PYMIN%, PYMAX%
4090 '************************************************************************
4100 CLS
4110 LOCATE 1, 5: PRINT "Quantum Harmonic Oscillator in 1 dimension"
4120 '
4130 PXMIN% = 136: PXMAX% = 392: PYMIN% = 32: PYMAX% = 152
4140                                                     'choose screen region
4150 SUBTITLE$ = "Wavefunction vs. x."
4160 XTITLE$ = "x": YTITLE$ = "psi"
4170 YMIN = PSIMIN: YMAX = PSIMAX                  'assign limits for graphics
4180 '
4190 DIV% = 4                     'set number of divisions (<5) on x- & y-axes
4200                                                  'find scale coefficients
4210 KCOEFF1 = (PXMAX% - PXMIN%) / (XMAX - XMIN)
4220 KCOEFF2 = (PXMAX% * XMIN - PXMIN% * XMAX) / (XMIN - XMAX)
4230 KCOEFF3 = (PYMAX% - PYMIN%) / (YMIN - YMAX)
4240 KCOEFF4 = (PYMAX% * YMAX - PYMIN% * YMIN) / (YMAX - YMIN)
4250 '
4260 ROWTIT% = PYMIN% / 8 + 1: COLTIT% = PXMAX% / 8 + 2
4270 LOCATE ROWTIT%, COLTIT%: PRINT SUBTITLE$               'print graph-title
4280 '
4290 LINE (PXMIN%, PYMIN%)-(PXMAX%, PYMAX%), , B           'draw box for graph
4300 LINE (PXMIN%, KCOEFF4)-(PXMAX%, KCOEFF4)                     'draw x-axis
4310 LINE (KCOEFF2, PYMIN%)-(KCOEFF2, PYMAX%)                     'draw y-axis
4320 '
4330 XSCALE = (XMAX - XMIN) / DIV%                       'find scale for marks
4340 YSCALE = (YMAX - YMIN) / DIV%
4350 '
4360 PXSCALE = (PXMAX% - PXMIN%) / DIV%            'find pixel scale for marks
4370 PYSCALE = (PYMAX% - PYMIN%) / DIV%
4380 '
4390 ROWX% = PYMAX% / 8 + 2              'find row for marks along x-direction
4400 COLY% = PXMIN% / 8 - 10          'find column for marks along y-direction
4410 '
4420 FOR J% = 0 TO DIV%                          'start loop to mark divisions
4430   PMX(J%) = PXMIN% + PXSCALE * J%                    'find places to mark
4440   PMY(J%) = PYMIN% + PYSCALE * J%
4450    LINE (PMX(J%), PYMAX%)-(PMX(J%), PYMAX% - 5)               'draw marks
4460    LINE (PXMIN%, PMY(J%))-(PXMIN% + 5, PMY(J%))
4470 '
4480   IF J% / 2 <> INT(J% / 2) THEN 4720            'jump for alternate marks
4490 '
4500   COLX% = PMX(J%) / 8 - 3         'find column for each mark along x-axis
4510   ROWY% = PMY(J%) / 8 + 1            'find row for each mark along y-axis
4520 '
4530   LOCATE ROWX%, COLX%
4540    IF XMAX > 999.99 OR XMIN < -999.99 THEN 4580
4550    IF XMAX < .01 AND XMIN > -.01 THEN 4580              'format selection
4560   PRINT USING "+###.##"; XMIN + XSCALE * J%      'print x-scale divisions
4570   GOTO 4600
4580   PRINT USING "+#.##^^^^"; XMIN + XSCALE * J%    'print x-scale divisions
4590 '
4600   LOCATE ROWY%, COLY%
4610    IF YMAX > 999.99 OR YMIN < -999.99 THEN 4650
4620    IF YMAX < .01 AND YMIN > -.01 THEN 4650              'format selection
4630   PRINT USING "+###.##"; YMAX - YSCALE * J%      'print y-scale divisions
4640   GOTO 4670
4650   PRINT USING "+#.##^^^^"; YMAX - YSCALE * J%    'print y-scale divisions
4660 '
4670   IF J% <> 2 THEN 4720              'print names of variables along x & y
4680    LOCATE ROWX%, COLTIT% + 3: PRINT XTITLE$; " ->"
4690    LOCATE ROWY% - 1, COLY% - 1: PRINT "^"
4700    LOCATE ROWY%, COLY% - 1: PRINT "|"
4710    LOCATE ROWY% + 1, COLY% - 1: PRINT YTITLE$
4720 NEXT J%
4730 '
4740 RETURN
4750 '
10000 '***********************************************************************
10010 ' subroutine to display header screen
10020 'input variables:  none
10030 'output variables: none
10040 'global variables: none
10050 'local variables:  none
10060 '***********************************************************************
10070 CLS : KEY OFF                                           'clear screen
10080 SCREEN 2, 0, 0: WIDTH 80                               'change screen
10090  '
10100 LINE (1, 1)-(638, 40), , B                                'draw boxes
10110 LINE (5, 3)-(633, 37), , B
10120 LINE (70, 50)-(570, 140), , B
10130 LINE (75, 53)-(565, 137), , B
10140                                           'print title, authors, etc.
10150 LOCATE 3, 7
10160 PRINT " C O M P U T A T I O N A L   P H Y S I C S :";
10170 PRINT " (An  Introduction)"
10180 LOCATE 9, 40
10190 PRINT "by";
10200 LOCATE 11, 20: PRINT "R.C. Verma, P.K. Ahluwalia & K.C. Sharma"
10210 LOCATE 13, 34
10220 PRINT "   * * * *    "
10230 '
10240 BEEP
10250 LOCATE 15, 35: PRINT "* * * * * *"
10260 LOCATE 22, 25: PRINT "press any key to continue"
10270 LOCATE 23, 25: PRINT "-------------------------"
10280 '
10290 IF INKEY$ = "" THEN 10290                            'hold the screen
10300 CLS                                                     'clear screen
10310 RETURN
10320 '

