10 '**************************************************************************
20 'Chapter 10, Example 3: Solution of 3-dim. Schrodinger Eq. (H.O. Potential)
30 '                       using Numerov method
40 'COMPUTATIONAL PHYSICS: An Introduction
50 'by R. C. Verma, P. K. Ahluwalia and K. C. Sharma
60 'Copyright 1995
70 '**************************************************************************
80 '
90 GOSUB 10000                                                    'show header
100 '
110 DEF FNPOT (X) = .5 * (X / ALFA) ^ 2            'h.o potential in new units
120 '
130 GOSUB 2000                       'introduce problem & get input parameters
140 '
150 DIM X(NPOINTS%), POT(NPOINTS%), WVF(NPOINTS%)      'dimension these arrays
160 '                                         
170 GOSUB 3000                                                 'find potential
180 GOSUB 5000                                 'find scales and draw potential
190 '
200 LOCATE 23, 1: BEEP: PRINT "press any key to continue"
210 IF INKEY$ = "" THEN 210
220 '
230 GOSUB 1000                              'subroutine to handle main program
240 '
250 LOCATE 23, 1: BEEP: PRINT "press any key to continue"
260 IF INKEY$ = "" THEN 260
270 '
280 CLS : LOCATE 12, 15: BEEP: PRINT " Thank you !"
290 '
300 END
310 '
1000 '************************************************************************
1010 'subroutine to handle the main program
1020 'input  variables:  TOLR, DEN, DESAFE
1030 'output variables:  E
1040 'global variables:  NPOINTS%
1050 'local variables: EIGENRGY, N%, SECAN%, BEGIN%, F, FBEGIN, FOLD, OLDENRGY
1060 '************************************************************************
1070 '
1080 EIGENRGY = -1                                        'minimum value for E
1090 '
1100 FOR N% = 1 TO 10                             'loop over radial quantum no.
1110   LOCATE 1, 50: PRINT "n = "; N%
1120   SECAN% = 0: BEGIN% = -1: F = TOLR         'set values to start E search
1130 '
1140   WHILE ABS(F) >= TOLR
1150     GOSUB 4000                                     'solve Schrodinger eq.
1160     GOSUB 6000                                'draw axis for wavefunction
1170     GOSUB 7000                                         'draw wavefunction
1180 '
1190     IF BEGIN% THEN FBEGIN = F                               'save F value
1200     IF F * FBEGIN < 0 THEN SECAN% = -1          'then when F changes sign
1210     IF F = FOLD GOTO 1230                           'secant search method
1220     IF SECAN% THEN DEN = -F * (E - OLDENRGY) / (F - FOLD)
1230     OLDENRGY = E: FOLD = F                             'save these values
1240     E = E + DEN                                          'increase Energy
1250     BEGIN% = 0                                      'no longer first time
1260   WEND
1270 '
1280  LOCATE 22, 20
1290  IF ABS(E) < 1000 AND ESCALE = 1 THEN PRINT USING "E=###.###"; E: GOTO 1310
1300  PRINT USING "Energy = #.##### x #.####^^^^"; E / ESCALE; ESCALE
1310  LOCATE 2, 10
1320  PRINT USING "nodes = ##,   Tolerance = #.####^^^^"; NODE%; F
1330  LOCATE 23, 15: BEEP
1340 '
1350   PRINT "Bound state found.  Press any key to continue";
1360   IF INKEY$ = "" THEN 1360
1370 '
1380   LOCATE 22, 1: PRINT SPACE$(79)
1390   LOCATE 2, 10: PRINT SPACE$(79)
1400   LOCATE 23, 1: PRINT SPACE$(79)
1410 '
1420   DEN = DESAFE
1430   EIGENRGY = OLDENRGY                                    'save eigenvalue
1440   E = OLDENRGY + DEN                                 'increase E to begin
1450 NEXT N%                                                        'next case
1460 '
1470 RETURN
1480 '
2000 '************************************************************************
2010 'subroutine to introduce the problem & to get input variables
2020 'input variables: none
2030 'output variables: XMIN, XMAX, XMAX2, VMIN, VMAX, ESCALE, TOLR, NPOINTS%
2040 '                  MASS, ALFA, GAMMA, E, DEN, DESAFE
2050 'global variables: none
2060 'local variables:  GAMMA2
2070 '************************************************************************
2080 '
2090 LOCATE 1, 5: PRINT "This program solves radial Schrodinger equation for"
2100 LOCATE 2, 5: PRINT "harmonic Oscillator potential  POT(r)= 1/2 r^2"
2110 LOCATE 3, 5: PRINT "with  the boundary conditions that the u(r)=rR(r) "
2120 LOCATE 4, 5: PRINT "function is zero at the extremes, r = 0 & r = r max."
2130 LOCATE 5, 5: PRINT "----------------------------------------------------"
2140 '
2150 LOCATE 7, 5: PRINT "Please give the following parameters:"
2160 '
2170 LOCATE 9, 5: INPUT "r max = "; XMAX2
2180  IF XMAX2 > 0 THEN 2200
2190  BEEP: LOCATE 9, 5: PRINT SPACE$(70): GOTO 2170
2200 LOCATE 11, 5: INPUT "reduce mass  = "; MASS
2210  IF MASS > 0 THEN 2230
2220  BEEP: LOCATE 11, 5: PRINT SPACE$(70): GOTO 2200
2230 LOCATE 13, 5: INPUT " alfa = "; ALFA
2240  IF ALFA > 0 THEN 2270
2250  BEEP: LOCATE 13, 5: PRINT SPACE$(70): GOTO 2230
2260 '
2270 GAMMA2 = 2 * MASS / ALFA ^ 2: GAMMA = SQR(GAMMA2)          'for later use
2280 '
2290 LOCATE 15, 5: INPUT "Give guess energy = "; E
2300  IF E <> 0 THEN 2320
2310  BEEP: LOCATE 15, 5: PRINT SPACE$(70): GOTO 2290
2320 LOCATE 17, 5: INPUT "increment in eigen-energy search  DE = "; DEN
2330  IF DEN > 0 THEN 2360
2340  BEEP: LOCATE 17, 5: PRINT SPACE$(70): GOTO 2320
2350 '
2360 DESAFE = DEN
2370 '
2380 XMIN = 0: XMAX = XMAX2: VMIN = 0: VMAX = FNPOT(XMAX2)
2390                                         'fix max, min limits for graphics
2400 ESCALE = 1                                                   'scale fixed
2410 NPOINTS% = 200                                     'no. mesh points fixed
2420 TOLR = .00005                                  'matching tolerance chosen
2430 '
2440 RETURN
2450 '
3000 '************************************************************************
3010 'subroutine to get the potential
3020 'input  variables:   XMIN2, XMAX2, ESCALE
3030 'output variables:   DELX, XMIN, x(i%), POT(I%), E, DEN,
3040 'global variables:   NPOINTS%
3050 'local  variables:   I%, XMIN2, DELX2
3060 '************************************************************************
3070 '
3080  XMIN2 = 9.999999E-15                'avoid singularity for Coulomb case
3090   DELX2 = (XMAX2 - XMIN2) / NPOINTS%
3100 '
3110  XMIN = XMIN2: DELX = DELX2
3120 '
3130  FOR I% = 0 TO NPOINTS%
3140    X(I%) = XMIN + I% * DELX
3150    POT(I%) = FNPOT(XMIN + DELX * I%)   'find potential at all mesh points
3160  NEXT I%
3170 '
3180  E = E * ESCALE: DEN = DEN * ESCALE                       'rescale energy
3190 '
3200 RETURN
3210 '
4000 '************************************************************************
4010 'subroutine to solve Schrodinger equation
4020 'input  variables: DELX, GAMMA, POT(I%), E,
4030 'output variables: F, MATCHP%, NMATCHP%, NODE%, WVF(I%), WVFMAX, WVFMIN
4040 'global variables: NPOINTS%
4050 'local  variables: C, I%, K%, VEC, VECM1, VECP1, NORM, WVFMMTCH, WVFPMTCH
4060 '************************************************************************
4070 C = (DELX * DELX / 12) * GAMMA * GAMMA                   'constant factor
4080 MATCHP% = 0                                         'clear matching point
4090 '
4100 WVF(0) = 0                  'begin with left hand side boundary condition
4110 WVF(1) = 9.999999E-10
4120 '
4130 VECM1 = C * (E - POT(0))                              'initial K*K values
4140 VEC = C * (E - POT(1))
4150 '
4160 FOR I% = 1 TO NPOINTS% - 1                       'apply Numerov algorithm
4170   VECP1 = C * (E - POT(I% + 1))
4180   IF (VEC * VECP1 <= 0 AND VEC > 0) THEN MATCHP% = I%: GOTO 4280
4190                                                           'matching point
4200   WVF(I% + 1) = (WVF(I%) * (2 - 10 * VEC) - WVF(I% - 1) * (1 + VECM1)) / (1 + VECP1)
4210   IF ABS(WVF(I% + 1)) < (1E+10) GOTO 4250         'if WVF grows too large
4220   FOR K% = 1 TO I% + 1                       'rescale all previous points
4230     WVF(K%) = WVF(K%) * 9.999999E-06
4240   NEXT K%
4250   VECM1 = VEC: VEC = VECP1                            'roll values of K*K
4260 NEXT I%
4270 '
4280 IF MATCHP% = 0 THEN MATCHP% = NPOINTS% - 10 'if there is no turning point
4290                                            'match wave functions near rhs
4300 WVFMMTCH = WVF(MATCHP%)                     'save value for normalization
4310 '
4320 WVF(NPOINTS%) = 0                     'right hand side boundary condition
4330 WVF(NPOINTS% - 1) = 9.999999E-10
4340 '
4350 VECP1 = C * (E - POT(NPOINTS%))                       'initial K*K values
4360 VEC = C * (E - POT(NPOINTS% - 1))
4370 '
4380 FOR I% = NPOINTS% - 1 TO MATCHP% + 1 STEP -1     'apply Numerov algorithm
4390   VECM1 = C * (E - POT(I% - 1))
4400   WVF(I% - 1) = (WVF(I%) * (2 - 10 * VEC) - WVF(I% + 1) * (1 + VECP1)) / (1 + VECM1)
4410   IF ABS(WVF(I% - 1)) < (1E+10) GOTO 4450         'if WVF grows too large
4420   FOR K% = NPOINTS% - 1 TO I% - 1 STEP -1    'rescale all previous points
4430     WVF(K%) = WVF(K%) * 9.999999E-06
4440   NEXT K%
4450   VECP1 = VEC: VEC = VECM1                            'roll values of K*K
4460 NEXT I%
4470 '
4480 VECM1 = C * (E - POT(MATCHP% - 1)) 'finds value needed for log derivative
4490 WVFPMTCH = (WVF(MATCHP%) * (2 - 10 * VEC) - WVF(MATCHP% + 1) * (1 + VECP1)) / (1 + VECM1)
4500 '
4510 NORM = WVF(MATCHP%) / WVFMMTCH                'norm WVF left to WVF right
4520 '
4530 FOR I% = 1 TO MATCHP% - 1                        'keeping signs unchanged
4540    WVF(I%) = WVF(I%) * ABS(NORM)
4550 NEXT I%
4560                                  'change sign of WVF right if NORM is -ve
4570 IF NORM > 0 THEN GOTO 4650
4580 '
4590   FOR I% = MATCHP% TO NPOINTS%
4600      WVF(I%) = -WVF(I%)
4610   NEXT I%
4620 '
4630   WVFPMTCH = -WVFPMTCH
4640 '
4650 WVFMAX = ABS(WVF(1))                              'find maximum WVF value
4660 '
4670 FOR I% = 1 TO NPOINTS%
4680   IF ABS(WVF(I%)) > WVFMAX THEN WVFMAX = ABS(WVF(I%))
4690 NEXT I%
4700 '
4710 WVFMIN = -WVFMAX                                 'set limits for graphics
4720 '
4730 F = (WVF(MATCHP% - 1) - WVFPMTCH) / WVFMAX   'evaluate matching condition
4740 '
4750 NODE% = 0                                                'initialize node
4760                                                               'count mode
4770 FOR I% = 2 TO NPOINTS% - 1
4780  IF SGN(WVF(I%)) * SGN(WVF(I% - 1)) < 0 THEN NODE% = NODE% + 1
4790 NEXT I%
4800 '
4810 RETURN
4820 '
5000 '************************************************************************
5010 'subroutine to draw axes for the potential
5020 'Input variables: VMIN, VMAX
5030 'output variables: TITLE$, XTITLE$, YTITLE$
5040 '                  YMIN, YMAX, PXMIN%, PXMAX%, PYMIN%, PYMAX%
5050 'local variables:
5060 'global variables: none
5070 '************************************************************************
5080 CLS
5090 '
5100 TITLE$ = "- - - Harmonic Oscillator Potential - - -"
5110 '                                                            
5120 PXMIN% = 136: PXMAX% = 488: PYMIN% = 24: PYMAX% = 144
5130                                                     'choose screen region
5150 XTITLE$ = "r": YTITLE$ = "V(r)"
5160           'assign limits for graphics subroutine
5170 YMIN = VMIN: YMAX = VMAX
5180 '
5190  GOSUB 8000                                    'find scales and draw axes
5200  GOSUB 9000                                               'draw potential
5210 '
5220 RETURN
5230 '
6000 '************************************************************************
6010 'subroutine to draw axes for the wavefunction
6020 'Input variables: WVFMIN, WVFMAX
6030 'output variables: TITLE$, XTITLE$, YTITLE$
6040 '                  YMIN, YMAX, PXMIN%, PXMAX%, PYMIN%, PYMAX%
6050 'local variables: none
6060 'global variables: none
6070 '************************************************************************
6080 CLS
6090 '
6100 TITLE$ = "- - - Radial Wavefunction for H.O. Potential - - -"
6110 '                                                            
6120 PXMIN% = 136: PXMAX% = 592: PYMIN% = 28: PYMAX% = 148
6130                                                     'choose screen region
6150 XTITLE$ = "r": YTITLE$ = "u(r)"
6160           'assign limits for graphics subroutine
6170 YMIN = WVFMIN: YMAX = WVFMAX
6180 '
6190  GOSUB 8000                                    'find scales and draw axes
6200 '
6210 RETURN
6220 '
7000 '************************************************************************
7010 'subroutine to draw the wavefunction
7020 'Input variables: X(I%), WVF(I%), PXMIN%, PXMAX%, PYMIN%, PYMAX%
7030 '                KCOEFF1, KCOEFF2, KCOEFF3, KCOEFF4, MATCHP%
7040 'output variable: none
7050 'local variables: PXWVF%, PYWVF%
7060 'global variable: NPOINTS%,
7070 '************************************************************************
7080 'clear upper & lower boxes
7090 LINE (PXMIN% + 1, PYMIN% + 1)-(PXMAX% - 1, KCOEFF4 - 1), 0, BF
7100 LINE (PXMIN% + 1, KCOEFF4 + 1)-(PXMAX% - 1, PYMAX% - 1), 0, BF
7110                                                  'draw wavefn from l.h.s.
7120 FOR I% = 0 TO MATCHP%
7130   PXWVF% = KCOEFF1 * X(I%) + KCOEFF2
7140   PYWVF% = KCOEFF3 * WVF(I%) + KCOEFF4
7150   IF I% = 0 THEN PSET (PXWVF%, PYWVF%) ELSE LINE -(PXWVF%, PYWVF%)
7160 NEXT I%
7170                                                  'draw wavefn from r.h.s.
7180 FOR I% = NPOINTS% TO MATCHP% STEP -1
7190   PXWVF% = KCOEFF1 * X(I%) + KCOEFF2
7200   PYWVF% = KCOEFF3 * WVF(I%) + KCOEFF4
7210   IF I% = NPOINTS% THEN PSET (PXWVF%, PYWVF%) ELSE LINE -(PXWVF%, PYWVF%)
7220 NEXT I%
7230 '
7240 RETURN
7250 '
8000 '************************************************************************
8010 'subroutine to draw box & set scales
8020 'input variables: XTITLE$, YTITLE$, TITLE$
8030 '                 XMIN, XMAX, YMIN, YMAX, PXMIN%, PXMAX%, PYMIN%, PYMAX%
8040 'output variables: KCOEFF1, KCOEFF2, KCOEFF3 ,KCOEFF4,
8050 'global variables: none
8060 'local variables: ROWTIT%, COLTIT%
8070 '                 DIV%, XSCALE, YSCALE, PXSCALE, PYSCALE
8080 '                 PMX%(J%), PMY%(J%), ROWX%, COLX%, ROWY%, COLY%
8090 '************************************************************************
8100 '
8110 LOCATE 1, 15: PRINT TITLE$
8120  DIV% = 4                    'set number of divisions (<5) on x- & y-axes
8130                                                  'find scale coefficients
8140 KCOEFF1 = (PXMAX% - PXMIN%) / (XMAX - XMIN)
8150 KCOEFF2 = (PXMAX% * XMIN - PXMIN% * XMAX) / (XMIN - XMAX)
8160 KCOEFF3 = (PYMAX% - PYMIN%) / (YMIN - YMAX)
8170 KCOEFF4 = (PYMAX% * YMAX - PYMIN% * YMIN) / (YMAX - YMIN)
8180 '
8190 ROWTIT% = PYMAX% / 8 + 3: COLTIT% = (PXMIN% + PXMAX%) / (2 * 8)
8210 '
8220 LINE (PXMIN%, PYMIN%)-(PXMAX%, PYMAX%), , B           'draw box for graph
8230 LINE (PXMIN%, KCOEFF4)-(PXMAX%, KCOEFF4)                     'draw x-axis
8240 LINE (KCOEFF2, PYMIN%)-(KCOEFF2, PYMAX%)                     'draw y-axis
8250 '
8260 XSCALE = (XMAX - XMIN) / DIV%                       'find scale for marks
8270 YSCALE = (YMAX - YMIN) / DIV%
8280 '
8290 PXSCALE = (PXMAX% - PXMIN%) / DIV%            'find pixel scale for marks
8300 PYSCALE = (PYMAX% - PYMIN%) / DIV%
8310 '
8320 ROWX% = PYMAX% / 8 + 2              'find row for marks along x-direction
8330 COLY% = PXMIN% / 8 - 10          'find column for marks along y-direction
8340 '
8350 FOR J% = 0 TO DIV%                          'start loop to mark divisions
8360   PMX(J%) = PXMIN% + PXSCALE * J%                    'find places to mark
8370   PMY(J%) = PYMIN% + PYSCALE * J%
8380    LINE (PMX(J%), PYMAX%)-(PMX(J%), PYMAX% - 5)               'draw marks
8390    LINE (PXMIN%, PMY(J%))-(PXMIN% + 5, PMY(J%))
8400 '
8410   IF J% / 2 <> INT(J% / 2) THEN 8650            'jump for alternate marks
8420 '
8430   COLX% = PMX(J%) / 8 - 3         'find column for each mark along x-axis
8440   ROWY% = PMY(J%) / 8 + 1            'find row for each mark along y-axis
8450 '
8460   LOCATE ROWX%, COLX%
8470    IF XMAX > 999.99 OR XMIN < -999.99 THEN 8510
8480    IF XMAX < .01 AND XMIN > -.01 THEN 8510              'format selection
8490   PRINT USING "+###.##"; XMIN + XSCALE * J%      'print x-scale divisions
8500   GOTO 8530
8510   PRINT USING "+#.##^^^^"; XMIN + XSCALE * J%    'print x-scale divisions
8520 '
8530   LOCATE ROWY%, COLY%
8540    IF YMAX > 999.99 OR YMIN < -999.99 THEN 8580
8550    IF YMAX < .01 AND YMIN > -.01 THEN 8580              'format selection
8560   PRINT USING "+###.##"; YMAX - YSCALE * J%      'print y-scale divisions
8570   GOTO 8600
8580   PRINT USING "+#.##^^^^"; YMAX - YSCALE * J%    'print y-scale divisions
8590 '
8600   IF J% <> 2 THEN 8650              'print names of variables along x & y
8610    LOCATE ROWX% + 1, COLTIT% + 5: PRINT XTITLE$; "->"
8620    LOCATE ROWY% - 1, COLY% - 2: PRINT "^"
8630    LOCATE ROWY%, COLY% - 2: PRINT "|"
8640    LOCATE ROWY% + 1, COLY% - 4: PRINT YTITLE$
8650 NEXT J%
8660 '
8670 RETURN
8680 '
9000 '************************************************************************
9010 'subroutine to draw potential
9020 'input variables: NPOINTS%, POT(I%), KCOEFF1, KCOEFF2, KCOEFF3 ,KCOEFF4,
9030 'global variables: none
9040 'local variables: PXPOT%, PYPOT%
9050 '************************************************************************
9060                                            'perform scale transformations
9070 FOR I% = 0 TO NPOINTS%
9080  PXPOT% = KCOEFF1 * X(I%) + KCOEFF2
9090  PYPOT% = KCOEFF3 * POT(I%) + KCOEFF4
9100  IF I% = 0 THEN PSET (PXPOT%, PYPOT%) ELSE LINE -(PXPOT%, PYPOT%)
9110                                                           'draw potential
9120 NEXT I%
9130 '
9140 RETURN
9150 '
10000 '***********************************************************************
10010 'subroutine to display header screen
10020 'input variables:  none
10030 'output variables: none
10040 'global variables: none
10050 'local variables:  none
10060 '***********************************************************************
10070 CLS : KEY OFF                                           'clear screen
10080 SCREEN 2, 0, 0: WIDTH 80                               'change screen
10090  '
10100 LINE (1, 1)-(638, 40), , B                                'draw boxes
10110 LINE (5, 3)-(633, 37), , B
10120 LINE (70, 50)-(570, 140), , B
10130 LINE (75, 53)-(565, 137), , B
10140                                           'print title, authors, etc.
10150 LOCATE 3, 7
10160 PRINT " C O M P U T A T I O N A L   P H Y S I C S :";
10170 PRINT " (An  Introduction)"
10180 LOCATE 9, 40
10190 PRINT "by";
10200 LOCATE 11, 20: PRINT "R.C. Verma, P.K. Ahluwalia & K.C. Sharma"
10210 LOCATE 13, 34
10220 PRINT "   * * * *"
10230 '
10240 BEEP
10250 LOCATE 15, 35: PRINT "* * * * * *"
10260 LOCATE 22, 25: PRINT "press any key to continue"
10270 LOCATE 23, 25: PRINT "-------------------------"
10280 '
10290 IF INKEY$ = "" THEN 10290                            'hold the screen
10300 CLS                                                     'clear screen
10310 RETURN
10320 '

