10 '**************************************************************************
20 'Chapter 12, Example 1: Monte Carlo Method for Nuclear Radioactivity
30 'COMPUTATIONAL PHYSICS: An Introduction
40 'by R. C. Verma, P. K. Ahluwalia and K. C. Sharma
50 '   * * * *    
60 '**************************************************************************
70 '
80 GOSUB 10000                                                    'show header
90 '
100 GOSUB 1000                  'introduce the problem &  get input parameters
110 '
120 GOSUB 3000                                            'draw axes for graph
130 GOSUB 2000                  'perform Monte Carlo method & draw decay curve
140 '
150 LOCATE 23, 1: PRINT "press any key to continue"
160 IF INKEY$ = "" THEN 160
170 '
180 CLS : LOCATE 12, 15: BEEP: PRINT " Thank you !"
190 END
200 '
1000 '************************************************************************
1010 'subroutine to introduce the problem & to get input variables
1020 'input variables:  none
1030 'output variables: N%, D%, P, TMIN, TMAX, NI%, DI%
1040 'global variables: none
1050 'local variables:  none
1060 'test cases: Tmax = 50, N0 = 200, D0 = 0, p = 0.1
1070 ' For a good run: keep value of p small 0 < p < 0.2
1080 '                 keep number of parent nuclei    100 < N% < 999
1090 '                 keep   10 < TMAX < 50                      
1100 '************************************************************************
1110 '
1120 LOCATE 5, 10: PRINT "This program describes a Monte Carlo Simulation"
1130 LOCATE 6, 15: PRINT "of Radioactive decay with stable daughter"
1140 LOCATE 7, 20: PRINT "and finds approximate half life of parent nucleus"
1150 LOCATE 8, 10: PRINT "---------------------------------------------------"
1160 LOCATE 12, 10: BEEP: PRINT " please give parameters of the sample"
1170 '
1180 LOCATE 14, 10: INPUT "Initial number of parent nuclei N0 = "; N%
1190  IF N% > 0 AND INT(N%) = N% THEN 1210
1200  BEEP: LOCATE 14, 10: PRINT SPACE$(65): GOTO 1180
1210 LOCATE 16, 10: INPUT "Initial number of daughter nuclei D0 = "; D%
1220  IF D% >= 0 AND INT(D%) = D% THEN 1240
1230  BEEP: LOCATE 16, 10: PRINT SPACE$(65): GOTO 1210
1240 LOCATE 18, 10: INPUT "decay probability p = "; P
1250  IF P >= 0 AND P <= 1 THEN 1280
1260  BEEP: LOCATE 18, 10: PRINT SPACE$(65): GOTO 1240
1270 '
1280 TMIN = 0                                              'initial time fixed
1290 LOCATE 20, 10: INPUT "Maximum time intervals"; TMAX
1300  IF TMAX > 0 THEN 1320
1305  BEEP: LOCATE 20, 10: PRINT SPACE$(65): GOTO 1290
1310 '
1320 H = 1                                               'time interval fixed
1330 '
1340 NI% = N%: DI% = D%
1350              'save initial no. of parent & daughter nuclei  for later use
1360 RETURN
1370 '
2000 '************************************************************************
2010 'subroutine to perform Monte-Carlo method for radioactive decay
2020 'input variables: N%, D%, P,
2030 'output variables: T, N%, D%, HALF
2040 'global variables: none
2050 'local variables: NU%, X, T
2060 '************************************************************************
2070 '
2080 RANDOMIZE TIMER
2090 '
2100 T = TMIN                                                 'initialize time
2110 '
2120 WHILE (N% >= 0 AND T <= TMAX)                     '1st loop starts over time
2130 LOCATE 2, 3
2140 PRINT USING "t = ###.#  Parent nuclei = #####  Daughter nuclei = ####"; T; N%; D%
2150      GOSUB 5000                                               'draw curve
2160    T = T + H                                              'increment time
2170    NU% = N%                                      'initialize no. of nuclei
2180    FOR I% = 1 TO NU%               '2nd loop over remaining parent nuclei
2190      X = RND                                          'fetch a random no.
2200      IF X < P THEN N% = N% - 1: D% = D% + 1        'check if a nucleus decays
2210    NEXT I%                                                 '2nd loop ends
2220    IF N% >= (.5 * NI%) THEN HALF = T
2225                                  'till left-over nuclei is less than half
2240 WEND                                                       '1st loop ends
2245 '
2250 BEEP
2255 IF P = 0 THEN CLS : LOCATE 17, 65: PRINT "No decay occurred!": GOTO 2330
2260                                            'since nuclei have not decayed
2265 LOCATE 17, 65: PRINT "Half-life ="
2270 LOCATE 18, 65: PRINT "-----------"
2280 LOCATE 20, 65
2290  IF HALF > 1000 THEN 2320                                  'choose format
2300  PRINT USING "###.#s"; HALF
2310  GOTO 2330
2320  PRINT USING "#.##^^^^s"; HALF                           'print half-life
2330 '
2340 RETURN
2350 '
3000 '************************************************************************
3010 'subroutine to draw axes for decay process
3020 'Input variables: NI%, TMIN, TMAX
3030 'output variables: TITLE$, SUBTITLE$, XTITLE$, YTITLE$
3040 '                  XMIN, XMAX, YMIN, YMAX, PXMIN%, PXMAX%, PYMIN%, PYMAX%
3050 '                  LT1, KT2, KN3, KN4, KD3, KD4
3060 'local variables: none
3070 'global variables: none
3080 '************************************************************************
3090 CLS
3100 '
3110 TITLE$ = "* * * Nuclear Radioactive Decay * * *"  'main title
3120 XTITLE$ = "t"                                                 'for x-axis
3130 '                                                          
3140 PXMIN% = 136: PXMAX% = 400: PYMIN% = 24: PYMAX% = 72
3150                                                     'choose screen region for 1st graph
3160 XMIN = TMIN: XMAX = TMAX
3170 YMIN = 0: YMAX = NI% + DI%
3180 SUBTITLE$ = "Parent Nuclei vs. time"
3190 YTITLE$ = "N"                                              'for 1st graph
3200 '
3210  GOSUB 4000                                    'find scales and draw axes
3220 KT1 = KCOEFF1: KT2 = KCOEFF2: KN3 = KCOEFF3: KN4 = KCOEFF4
3230 '
3240 PXMIN% = 136: PXMAX% = 400: PYMIN% = 112: PYMAX% = 160
3250                                                     'choose screen region for 2nd graph
3260                                    'assign limits for graphics subroutine
3270 SUBTITLE$ = "Daughter Nuclei vs. t"
3280 YTITLE$ = "D"
3290 '
3300  GOSUB 4000                                    'find scales and draw axes
3310 KD3 = KCOEFF3: KD4 = KCOEFF4
3320 '
3330 RETURN
3340 '
4000 '************************************************************************
4010 'subroutine to draw box & set scales
4020 'input variables: XTITLE$, YTITLE$, TITLE$, SUBTITLE$
4030 '                 XMIN, XMAX, YMIN, YMAX, PXMIN%, PXMAX%, PYMIN%, PYMAX%
4040 'output variables: KCOEFF1, KCOEFF2, KCOEFF3 ,KCOEFF4,
4050 'global variables: none
4060 'local variables: ROWTIT%, COLTIT%
4070 '                 DIV%, XSCALE, YSCALE, PXSCALE, PYSCALE
4080 '                 PMX%(J%), PMY%(J%), ROWX%, COLX%, ROWY%, COLY%
4090 '************************************************************************
4100 '
4110 LOCATE 1, 15: PRINT TITLE$
4120  DIV% = 4                    'set number of divisions (<5) on x- & y-axes
4130                                                  'find scale coefficients
4140 KCOEFF1 = (PXMAX% - PXMIN%) / (XMAX - XMIN)
4150 KCOEFF2 = (PXMAX% * XMIN - PXMIN% * XMAX) / (XMIN - XMAX)
4160 KCOEFF3 = (PYMAX% - PYMIN%) / (YMIN - YMAX)
4170 KCOEFF4 = (PYMAX% * YMAX - PYMIN% * YMIN) / (YMAX - YMIN)
4180 '
4190 ROWTIT% = PYMIN% / 8 + 1: COLTIT% = PXMAX% / 8 + 2
4200 LOCATE ROWTIT%, COLTIT%: PRINT SUBTITLE$               'print graph-title
4210 '
4220 LINE (PXMIN%, PYMIN%)-(PXMAX%, PYMAX%), , B           'draw box for graph
4230 LINE (PXMIN%, KCOEFF4)-(PXMAX%, KCOEFF4)                     'draw x-axis
4240 LINE (KCOEFF2, PYMIN%)-(KCOEFF2, PYMAX%)                     'draw y-axis
4250 '
4260 XSCALE = (XMAX - XMIN) / DIV%                       'find scale for marks
4270 YSCALE = (YMAX - YMIN) / DIV%
4280 '
4290 PXSCALE = (PXMAX% - PXMIN%) / DIV%            'find pixel scale for marks
4300 PYSCALE = (PYMAX% - PYMIN%) / DIV%
4310 '
4320 ROWX% = PYMAX% / 8 + 2              'find row for marks along x-direction
4330 COLY% = PXMIN% / 8 - 10          'find column for marks along y-direction
4340 '
4350 FOR J% = 0 TO DIV%                          'start loop to mark divisions
4360   PMX(J%) = PXMIN% + PXSCALE * J%                    'find places to mark
4370   PMY(J%) = PYMIN% + PYSCALE * J%
4380    LINE (PMX(J%), PYMAX%)-(PMX(J%), PYMAX% - 5)               'draw marks
4390    LINE (PXMIN%, PMY(J%))-(PXMIN% + 5, PMY(J%))
4400 '
4410   IF J% / 2 <> INT(J% / 2) THEN 4650            'jump for alternate marks
4420 '
4430   COLX% = PMX(J%) / 8 - 3         'find column for each mark along x-axis
4440   ROWY% = PMY(J%) / 8 + 1            'find row for each mark along y-axis
4450 '
4460   LOCATE ROWX%, COLX%
4470    IF XMAX > 999.99 OR XMIN < -999.99 THEN 4510
4480    IF XMAX < .01 AND XMIN > -.01 THEN 4510              'format selection
4490   PRINT USING "+###.##"; XMIN + XSCALE * J%      'print x-scale divisions
4500   GOTO 4530
4510   PRINT USING "+#.##^^^^"; XMIN + XSCALE * J%    'print x-scale divisions
4520 '
4530   LOCATE ROWY%, COLY%
4540    IF YMAX > 999.99 OR YMIN < -999.99 THEN 4580
4550    IF YMAX < .01 AND YMIN > -.01 THEN 4580              'format selection
4560   PRINT USING "+###.##"; YMAX - YSCALE * J%      'print y-scale divisions
4570   GOTO 4600
4580   PRINT USING "+#.##^^^^"; YMAX - YSCALE * J%    'print y-scale divisions
4590 '
4600   IF J% <> 2 THEN 4650              'print names of variables along x & y
4610    LOCATE ROWX%, COLTIT% + 3: PRINT XTITLE$; "->"
4620    LOCATE ROWY% - 1, COLY% - 2: PRINT "^"
4630    LOCATE ROWY%, COLY% - 2: PRINT "|"
4640    LOCATE ROWY% + 1, COLY% - 2: PRINT YTITLE$
4650 NEXT J%
4660 '
4670 RETURN
4680 '
5000 '************************************************************************
5010 'subroutine to draw decay curve
5020 'input variables: T, N%, D%, KT1, KT2, KN3 ,KN4, KD3, KD4
5030 'global variables: none
5040 'local variables: PXT1%, PYN1%, PYD1%, PXT2%, PYN2%, PYD2%
5050 '************************************************************************
5060                                            'perform scale transformations
5070  PXT2% = KT1 * T + KT2                                          'for time
5080  PYN2% = KN3 * N% + KN4                                 'for parent nuclei
5090  PYD2% = KD3 * D% + KD4                               'for daughter nuclei
5100 '
5110  IF T = TMIN THEN 5150                       'avoid drawing if first time
5120  LINE (PXT1%, PYN1%)-(PXT2%, PYN2%)                   'draw parent nuclei
5130  LINE (PXT1%, PYD1%)-(PXT2%, PYD2%)                 'draw daughter nuclei
5140 '
5150 PXT1% = PXT2%: PYN1% = PYN2%: PYD1% = PYD2%     'roll values for graphics
5160 '
5170 RETURN
5180 '
10000 '***********************************************************************
10010 'subroutine to display header screen
10020 'input variables:  none
10030 'output variables: none
10040 'global variables: none
10050 'local variables:  none
10060 '***********************************************************************
10070 CLS : KEY OFF                                           'clear screen
10080 SCREEN 2, 0, 0: WIDTH 80                               'change screen
10090  '
10100 LINE (1, 1)-(638, 40), , B                                'draw boxes
10110 LINE (5, 3)-(633, 37), , B
10120 LINE (70, 50)-(570, 140), , B
10130 LINE (75, 53)-(565, 137), , B
10140                                           'print title, authors, etc.
10150 LOCATE 3, 7
10160 PRINT " C O M P U T A T I O N A L   P H Y S I C S :";
10170 PRINT " (An  Introduction)"
10180 LOCATE 9, 40
10190 PRINT "by";
10200 LOCATE 11, 20: PRINT "R.C. Verma, P.K. Ahluwalia & K.C. Sharma"
10210 LOCATE 13, 34
10220 PRINT "   * * * *    "
10230 '
10240 BEEP
10250 LOCATE 15, 35: PRINT "* * * * * *"
10260 LOCATE 22, 25: PRINT "press any key to continue"
10270 LOCATE 23, 25: PRINT "-------------------------"
10280 '
10290 IF INKEY$ = "" THEN 10290                            'hold the screen
10300 CLS                                                     'clear screen
10310 RETURN
10320 '
