10 '**************************************************************************
20 'Chapter 13, Example 2: Random walk in one dimension
30 'COMPUTATIONAL PHYSICS: An Introduction
40 'by R. C. Verma, P. K. Ahluwalia and K. C. Sharma
50 '   * * * *    
60 '**************************************************************************
70 '
80  GOSUB 10000                                                   'show header
90 '
100 GOSUB 1000                   'introduce the problem & get input parameters
110 '
120 DIM XNET(TRIAL)                                            'set dimensions
130 '
140 GOSUB 2000                               'perform analysis for random walk
150 '
160 LOCATE 22, 1
170 IF INKEY$ = "" THEN 170
180 '
190 CLS : LOCATE 12, 15: BEEP: PRINT " Thank you !"
200 END
210 '
1000 '************************************************************************
1010 'subroutine to introduce the problem & to get input variables
1020 'input variables:  none
1030 'output variables: P, NS, TRIAL, A, X0
1040 'global variables: none
1050 'local variables:  none
1060 'test cases: P = 0.4, NS = 100, TRIAL = 10
1070 '************************************************************************
1080 '
1090 LOCATE 2, 2: PRINT "This program simulates random walk in one dimension &"
1100 LOCATE 4, 2: PRINT "finds mean displacement, mean square displacement,"
1110 LOCATE 8, 2: PRINT " variance & predicts the scaling law."
1120 LOCATE 10, 2: PRINT "----------------------------------------------------"
1130 '
1140 LOCATE 12, 5: INPUT "Give probability of walk in the right direction "; P
1150  IF P > 0 AND P <= 1 THEN 1170
1160  BEEP: LOCATE 12, 5: PRINT SPACE$(70): GOTO 1140
1170 LOCATE 14, 5: INPUT "Give no. of total steps "; NS
1180  IF NS > 0 AND INT(NS) = NS THEN 1200
1190  BEEP: LOCATE 14, 5: PRINT SPACE$(70): GOTO 1170
1200 LOCATE 16, 5: INPUT "Give no. of trials "; TRIAL
1210  IF TRIAL > 0 AND INT(TRIAL) = TRIAL THEN 1230
1220  BEEP: LOCATE 16, 5: PRINT SPACE$(70): GOTO 1200
1230 '
1240 A = 1                                                    'step size fixed
1250 X0 = 0                 'initial position fixed as middle column of screen
1260 '
1270 RETURN
1280 '
2000 '************************************************************************
2010 'subroutine to do study random walk
2020 'input variables: NS, X0, TRIAL, P
2030 'output variables: X
2040 'global variables: none
2050 'local variables: SUM, SUM2, X, DICE, XNET(J), PFACTOR, NU, AVX, AVX2
2060 '************************************************************************
2070 CLS
2080 LOCATE 1, 5: PRINT "* * * Analysis of Random Walk * * *"
2090 '
2100 SUM2 = 0: SUM = 0                                        'initialize sums
2110 '
2120 FOR J% = 1 TO TRIAL                                     'loop over trials
2130   BEEP: LOCATE 5, 15: PRINT "Trial no. = "; J%
2140   X = X0                                        'restore initial position
2150   RANDOMIZE TIMER                                       'rolling the dice
2160   FOR I% = 1 TO NS                             'loop over number of steps
2170     DICE = RND                                         'throwing the dice
2180     IF DICE < P THEN X = X + A                              'step forward
2190     IF DICE >= P THEN X = X - A                            'step backward
2200     GOSUB 3000                                          'show random walk
2210   NEXT I%
2220   XNET(J%) = X
2230   LOCATE 13, 1: PRINT "net displacement = "; XNET(J%)
2240   LOCATE 15, 1: PRINT "press any key to continue"
2250   IF INKEY$ = "" THEN 2250
2260   LOCATE 13, 1: PRINT SPACE$(78)
2270   LOCATE 15, 1: PRINT SPACE$(78)
2280   SUM2 = SUM2 + X * X: SUM = SUM + X                         'update sums
2290 NEXT J%
2300 '
2310  AVX2 = SUM2 / TRIAL: AVX = SUM / TRIAL
2320                                                     'find average x & x^2
2330  DX = AVX2 - AVX * AVX                                     'find variance
2340                                                      'print final results
2350 LOCATE 16, 15: PRINT "No of steps = "; NS
2360 LOCATE 18, 10: PRINT " < X > = "; AVX; ",  < X^2 > = "; AVX2
2370 LOCATE 20, 10: PRINT ", < DX^2 > = "; DX
2380 '
2390 PFACTOR = 4 * P * (1 - P) * A * A                      'check scaling law
2400 NU = .5 * LOG(DX / PFACTOR) / LOG(NS)
2410 LOCATE 22, 1: PRINT "exponent for scaling in 1-dim walk = "; NU
2420 '
2430 RETURN
2440 '
3000 '************************************************************************
3010 'subroutine to do show random walk
3020 'input variables: X
3030 'output variables: none
3040 'global variables: none
3050 'local variables: COL
3060 '************************************************************************
3070 '
3080  LOCATE 8, 1: PRINT SPACE$(80)                              'show walkman
3090  '
3100  COL = X + 40                    'assign column value in middle of screen
3110  IF COL > 80 THEN COL = COL - 80: GOTO 3110     'keep col between 1 to 80
3120  IF COL < 1 THEN COL = COL + 80: GOTO 3120
3130 '
3140  LOCATE 8, COL: PRINT "*"                                  'show walkman
3150 '
3160 RETURN
3170 '
10000 '***********************************************************************
10010 'subroutine to display header screen
10020 'input variables:  none
10030 'output variables: none
10040 'global variables: none
10050 'local variables:  none
10060 '***********************************************************************
10070 CLS : KEY OFF                                           'clear screen
10080 SCREEN 2, 0, 0: WIDTH 80                               'change screen
10090  '
10100 LINE (1, 1)-(638, 40), , B                                'draw boxes
10110 LINE (5, 3)-(633, 37), , B
10120 LINE (70, 50)-(570, 140), , B
10130 LINE (75, 53)-(565, 137), , B
10140                                           'print title, authors, etc.
10150 LOCATE 3, 7
10160 PRINT " C O M P U T A T I O N A L   P H Y S I C S :";
10170 PRINT " (An  Introduction)"
10180 LOCATE 9, 40
10190 PRINT "by";
10200 LOCATE 11, 20: PRINT "R.C. Verma, P.K. Ahluwalia & K.C. Sharma"
10210 LOCATE 13, 34
10220 PRINT "   * * * *    "
10230 '
10240 BEEP
10250 LOCATE 15, 35: PRINT "* * * * * *"
10260 LOCATE 22, 25: PRINT "press any key to continue"
10270 LOCATE 23, 25: PRINT "-------------------------"
10280 '
10290 IF INKEY$ = "" THEN 10290                            'hold the screen
10300 CLS                                                     'clear screen
10310 RETURN
10320 '

