10 '**************************************************************************
20 'Chapter 14, Example 4: Area of an ellipse using Monte Carlo method
30 'COMPUTATIONAL PHYSICS: An Introduction
40 'by R. C. Verma, P. K. Ahluwalia and K. C. Sharma
50 '   * * * *    
60 '**************************************************************************
70 '
80  GOSUB 10000                                                   'show header
90 '
100 PI = 3.1415926#                                        'define constant pi
110 '
120 GOSUB 1000                   'introduce the problem & get input parameters
130 '
140 GOSUB 2000                                     'perform Monte Carlo method
150 '
160 LOCATE 22, 1
170 IF INKEY$ = "" THEN 170
180 '
190 CLS : LOCATE 12, 15: BEEP: PRINT " Thank you !"
200 END
210 '
1000 '************************************************************************
1010 'subroutine to introduce the problem & to get input variables
1020 'input variables:  none
1030 'output variables: M%, N%, A, B
1040 'global variables: none
1050 'local variables:  none
1060 'test cases: M% = 25, N% = 100, a = 10, b = 20
1070 '************************************************************************
1080 '
1090 LOCATE 2, 5: PRINT " This program finds area of an ellipse"
1100 LOCATE 3, 10: PRINT " by Monte-Carlo Method"
1110 LOCATE 4, 5: PRINT "-----------------------------------------------"
1120  '
1130 LOCATE 8, 5: INPUT "Give no. of measurements = "; M%
1140  IF M% > 0 THEN 1160
1150  BEEP: LOCATE 8, 5: PRINT SPACE$(70): GOTO 1130
1160 LOCATE 10, 5: INPUT "Give no of trials = "; N%
1170  IF N% > 0 THEN 1190
1180  BEEP: LOCATE 10, 5: PRINT SPACE$(70): GOTO 1160
1190 LOCATE 12, 5: INPUT "Give semi-major & semi-minor axes: a,b = "; A, B
1200  IF A > 0 AND B > 0 THEN 1220
1210  BEEP: LOCATE 12, 5: PRINT SPACE$(70): GOTO 1190
1220 '
1230 RETURN
1240 '
2000 '************************************************************************
2010 'subroutine to find area of the ellipse
2020 'input variables: M%, N%, A, B, PI
2030 'output variables: AVAREA, SIGMA, AA
2040 'global variables: none
2050 'local variables: SUMAR, SUMARSQ, COUNT%, X, Y, AREA
2060 '************************************************************************
2070 '
2080  RANDOMIZE TIMER                                              'get a seed
2090 '
2100  SUMAR = 0: SUMARSQ = 0                      'initialize sum & sum-square
2110 '
2120 FOR J% = 1 TO M%                                   'loop for measurements
2130  COUNT% = 0                                            'initialize counts
2140  FOR I% = 1 TO N%                                        'loop for trials
2150   X = A * RND: Y = B * RND                                   'hit a point
2160   IF (X / A) ^ 2 + (Y / B) ^ 2 <= 1 THEN COUNT% = COUNT% + 1
2170                               'check if the point lies within the ellipse
2180  NEXT I%
2190  AREA = 4 * A * B * COUNT% / N%                           'calculate area
2200   SUMAR = SUMAR + AREA
2210   SUMARSQ = SUMARSQ + AREA * AREA
2220 NEXT J%
2230 '
2240  AVAREA = SUMAR / M%                   'find average area & St. deviation
2250  SIGMA = SQR(M% * SUMARSQ - SUMAR * SUMAR) / M%
2260  AA = PI * A * B                                         'find exact area
2270 '
2280 BEEP
2290 LOCATE 16, 2: PRINT USING "calculated area = ##.###^^^^, "; AVAREA
2300 LOCATE 18, 2: PRINT USING "Standard deviation = #.######"; SIGMA
2310 LOCATE 20, 2: PRINT USING "exact area = ##.###^^^^  "; AA
2320 '
2330 RETURN
2340 '
10000 '***********************************************************************
10010 'subroutine to display header screen
10020 'input variables:  none
10030 'output variables: none
10040 'global variables: none
10050 'local variables:  none
10060 '***********************************************************************
10070 CLS : KEY OFF                                           'clear screen
10080 SCREEN 2, 0, 0: WIDTH 80                               'change screen
10090  '
10100 LINE (1, 1)-(638, 40), , B                                'draw boxes
10110 LINE (5, 3)-(633, 37), , B
10120 LINE (70, 50)-(570, 140), , B
10130 LINE (75, 53)-(565, 137), , B
10140                                           'print title, authors, etc.
10150 LOCATE 3, 7
10160 PRINT " C O M P U T A T I O N A L   P H Y S I C S :";
10170 PRINT " (An  Introduction)"
10180 LOCATE 9, 40
10190 PRINT "by";
10200 LOCATE 11, 20: PRINT "R.C. Verma, P.K. Ahluwalia & K.C. Sharma"
10210 LOCATE 13, 34
10220 PRINT "   * * * *    "
10230 '
10240 BEEP
10250 LOCATE 15, 35: PRINT "* * * * * *"
10260 LOCATE 22, 25: PRINT "press any key to continue"
10270 LOCATE 23, 25: PRINT "-------------------------"
10280 '
10290 IF INKEY$ = "" THEN 10290                            'hold the screen
10300 CLS                                                     'clear screen
10310 RETURN
10320 '

