 '****************************************************************************
 'Chapter 15, Example 4: Logistic Eq. Lyapunov Exponents
 'COMPUTATIONAL PHYSICS: An Introduction
 'by R. C. Verma, P. K. Ahluwalia and K. C. Sharma
 '   * * * *    
 '****************************************************************************
 '
 DECLARE FUNCTION logistic (x, mu)
 DECLARE SUB header ()
 DECLARE SUB graph (mu, x)
 DECLARE SUB param (x0, ni, nf)
 DECLARE SUB calc (x0, ni, nf, mu, ly)
 '
 TYPE logistic                                              'declare variables
  x AS SINGLE
  mu AS SINGLE
 END TYPE
 '
TYPE param                                                  'declare variables
 x0 AS SINGLE
 ni AS INTEGER
 nf AS INTEGER
END TYPE
 '
TYPE calc                                                   'declare variables
 x0 AS SINGLE
 ni AS INTEGER
 nf AS INTEGER
 mu AS SINGLE
 ly AS SINGLE
END TYPE
 '
CALL header
							       'display header
CALL param(x0, ni, nf)
				 'introduce the program & get input parameters
 CLS
 VIEW (10, 30)-(620, 150), , 1                           'choose screen region
 WINDOW (3, 3)-(4, -3)
 LOCATE 1, 10: PRINT "Logistic Eq. Lyapunov Exponents"
 LOCATE 2, 10: PRINT "Xo = "; x0; ", Ni = "; ni; ", Nf = "; nf
 '
CALL calc(x0, ni, nf, mu, ly)
					      'study Logistic eq. & draw graph
 '
 END

'
SUB calc (x0, ni, nf, mu, ly)
 '****************************************************************************
 'subroutine to do the calculations & display graph
 'input variables:  x0, ni, nf
 'output variables: mu, ly
 'global variables: none
 'local variables:  x, y,
 '****************************************************************************
 '
 FOR mu = 3 TO 4 STEP .001                                    '1st loop starts
   LET ly = 0
   x = x0
   FOR i = 1 TO nf                                            '2nd loop starts
     y = logistic(x, mu)
     IF i > ni THEN
     ly = ly + LOG(ABS(mu * (1 - 2 * y)))
   END IF
   x = y
   NEXT i
  '
   ly = ly / (nf - ni)
   LOCATE 22, 1: PRINT USING "mu = #.#####,   ly  = ###.####"; mu; ly
   CALL graph(mu, ly)
 NEXT mu
 '
END SUB

'
SUB graph (mu, x)
'*****************************************************************************
 PSET (mu, x)
'
END SUB

 '
 SUB header
 '****************************************************************************
 'subroutine to display header screen
 'input variables:  none
 'output variables: none
 'global variables: none
 'local variables:  none
 '****************************************************************************
 CLS : KEY OFF                                                   'clear screen
 SCREEN 2, 0, 0: WIDTH 80                                       'change screen
  '
 LINE (1, 1)-(638, 40), , B                                        'draw boxes
 LINE (5, 3)-(633, 37), , B
 LINE (70, 50)-(570, 140), , B
 LINE (75, 53)-(565, 137), , B
						   'print title, authors, etc.
 LOCATE 3, 7
 PRINT " C O M P U T A T I O N A L   P H Y S I C S :";
 PRINT " (An  Introduction)"
 LOCATE 9, 40
 PRINT "by";
 LOCATE 11, 20: PRINT "R.C. Verma, P.K. Ahluwalia & K.C. Sharma"
 LOCATE 13, 34
 PRINT "   * * * *    "
 '
 BEEP
 LOCATE 15, 35: PRINT "* * * * * *"
 LOCATE 22, 25: PRINT "press any key to continue"
 LOCATE 23, 25: PRINT "-------------------------"
 '
10290 IF INKEY$ = "" THEN 10290                               'hold the screen
 CLS                                                             'clear screen
 '
 END SUB

'
FUNCTION logistic (x, mu)
 '****************************************************************************
 '
 logistic = mu * x * (1 - x)
 '
END FUNCTION

'
SUB param (x0, ni, nf)
 '****************************************************************************
 'subroutine to introduce the program & get input parameters
 'input variables:  none
 'output variables: x0, ni, nf
 'global variables: none
 'local variables:  none
 'test case: x0 = 0.5, ni = 1, nf = 10
 '****************************************************************************
 '
 LOCATE 2, 5: PRINT "This program studies Logistic Equation"
 LOCATE 3, 5: PRINT "--------------------------------------"
 '
1100 LOCATE 5, 5: INPUT "Give value of X0 : 0 < X0 < 1"; x0
  IF x0 > 0 AND x0 < 1 THEN 1150
  BEEP: LOCATE 5, 5: PRINT SPACE$(75): GOTO 1100
'
1150 LOCATE 7, 5: INPUT "Give initial & final values: Ni, Nf"; ni, nf
  IF nf > 0 AND ni > 0 AND nf > ni THEN 1200
  BEEP: LOCATE 7, 5: PRINT SPACE$(75): GOTO 1150
  '
1200 CLS
 '
END SUB

