'*****************************************************************************
'Chapter 15, Example 5: CHAOS: nonlinear pendulum, eq of motion, a=F(x,v,t)
'COMPUTATIONAL PHYSICS: An Introduction
'by R. C. Verma, P. K. Ahluwalia and K. C. Sharma
'   * * * *    
'*****************************************************************************
 '
DECLARE SUB header ()
DECLARE SUB param (w!, c!, f!, h!)
DECLARE SUB prep (w!, c!, f!)
DECLARE SUB calc (w!, c!, f!, h!)
'
TYPE param
 w AS SINGLE
 c AS SINGLE
 f AS SINGLE
 h AS SINGLE
END TYPE
'
CALL header                                                       'show header
'
DEF Fnf (x, v, t) = -SIN(x) - c * v - f * COS(w * t)
'
CALL param(w, c, f, h)                                              'get input
CALL prep(w, c, f)                                'prepare screen for graphics
CALL calc(w, c, f, h)                                             'study chaos
'
LOCATE 23, 23: PRINT "Press any key to continue the map"
100 IF INKEY$ = "" THEN 100
'
END
 '

'
SUB calc (w, c, f, h)
'*****************************************************************************
'subroutine to plot phase space trajectory
'input variables:  fny(,), w, c, f, h
'output variables: none
'global variables: none
'local variables:  x, v, t,  k1, k2, k3, k4
'*****************************************************************************
'
FOR v = -3 TO 3 STEP .5
  x = -10
  t = 0
3035 PSET (x, v)
3040  IF x > 10 THEN x = x - 20: GOTO 3040
3050  IF x < -10 THEN x = x + 20: GOTO 3050
3060  IF v > 3 THEN v = v - 6: GOTO 3060
3070  IF v < -3 THEN v = v + 6: GOTO 3070
  LOCATE 22, 1
  PRINT USING "t = ####.###, x = +###.####, v = +##.####"; t; x; v
  k1 = h * Fnf(x, v, t)                              'calculate k1, k2, k3, k4
  k2 = h * Fnf(x + h * v / 2, v + k1 / 2, t + h / 2)
  k3 = h * Fnf(x + h * v / 2 + h * k1 / 4, v + k2 / 2, t + h / 2)
  k4 = h * Fnf(x + h * v + h * k2 / 2, v + k3, t + h)
  '
  x = x + h * (v + (k1 + k2 + k3) / 6)                               'update x
  v = v + (k1 + 2 * k2 + 2 * k3 + k4) / 6                            'update v
  t = t + h                                                    'increment time
  IF INKEY$ = "" THEN 3035
NEXT v
'
END SUB

'
SUB header
 '****************************************************************************
 'subroutine to display header screen
 'input variables:  none
 'output variables: none
 'global variables: none
 'local variables:  none
 '****************************************************************************
 CLS : KEY OFF                                                   'clear screen
 SCREEN 2, 0, 0: WIDTH 80                                       'change screen
  '
 LINE (1, 1)-(638, 40), , B                                        'draw boxes
 LINE (5, 3)-(633, 37), , B
 LINE (70, 50)-(570, 140), , B
 LINE (75, 53)-(565, 137), , B
						    'print title, authors, etc.
 LOCATE 3, 7
 PRINT " C O M P U T A T I O N A L   P H Y S I C S :";
 PRINT " (An  Introduction)"
 LOCATE 9, 40
 PRINT "by";
 LOCATE 11, 20: PRINT "R.C. Verma, P.K. Ahluwalia & K.C. Sharma"
 LOCATE 13, 34
 PRINT "   * * * *    "
 '
 BEEP
 LOCATE 15, 35: PRINT "* * * * * *"
 LOCATE 22, 25: PRINT "press any key to continue"
 LOCATE 23, 25: PRINT "-------------------------"
 '
10100 IF INKEY$ = "" THEN 10100                               'hold the screen
 CLS                                                             'clear screen
 END SUB

'
SUB param (w, c, f, h)
'*****************************************************************************
'subroutine to introduce the program & get input parameters
'input variables:  none
'output variables: w, c, f, h
'global variables: none
'local variables:  none
'test case: w = 1, c = 0 or 0.1, f = 0 or 1 or 1.3 or 1.7, h = 0.01
'*****************************************************************************
'
LOCATE 2, 5: PRINT "This program studies Chaos: "
LOCATE 3, 5: PRINT "through nonlinear pendulum  "
LOCATE 4, 5: PRINT "----------------------------"
'
1100 LOCATE 6, 5: INPUT "Give angular frequency of pendulum: w <= 10 "; w
  IF w > 0 AND w <= 10 THEN 1200
  BEEP: LOCATE 6, 5: PRINT SPACE$(75): GOTO 1100
1200 LOCATE 8, 5: INPUT "Give coefficient of damping: c < 1 "; c
  IF c >= 0 AND c < 1 THEN 1300
  BEEP: LOCATE 8, 5: PRINT SPACE$(75): GOTO 1200
1300 LOCATE 10, 5: INPUT "Give amplitude of driving force: f <= 2 "; f
  IF ABS(f) >= 0 AND ABS(f) <= 2 THEN 1400
  BEEP: LOCATE 10, 5: PRINT SPACE$(75): GOTO 1300
1400 LOCATE 12, 5: INPUT "Give step size: 0 < h < 1 "; h
  IF h > 0 AND h < 1 THEN 1500
  BEEP: LOCATE 12, 5: PRINT SPACE$(75): GOTO 1400
'
1500 '
'
END SUB

'
SUB prep (w, c, f)
'*****************************************************************************
'subroutine to prepare screen for graphics
'input variables:  w, c, f
'output variables: none
'global variables: none
'local variables:  none
'*****************************************************************************
CLS
VIEW (20, 25)-(600, 145), , 1                            'choose screen region
WINDOW (-10, 3)-(10, -3)
'
LINE (-10, 0)-(10, 0)
LINE (0, 3)-(0, -3)
'
LOCATE 1, 10: PRINT "Nonlinear pendulum"
LOCATE 12, 1: PRINT "v"
LOCATE 19, 40: PRINT "x->"
LOCATE 3, 10: PRINT USING "w = ##.###, c = +##.###, f = +##.###"; w; c; f
'
END SUB

