'*****************************************************************************
'Chapter 15, Example 7: Lorenz equations
'COMPUTATIONAL PHYSICS: An Introduction
'by R. C. Verma, P. K. Ahluwalia and K. C. Sharma
'   * * * *    
'*****************************************************************************
'
DECLARE SUB header ()
DECLARE SUB param (sig, r, B, x, y, z, h, tmax)
DECLARE SUB prep (h, tmax)
DECLARE SUB calc (sig, r, B, x, y, z, h, tmax)
'
TYPE param
sig AS SINGLE
r AS SINGLE
B AS SINGLE
x AS SINGLE
y AS SINGLE
z AS SINGLE
h AS SINGLE
tmax AS SINGLE
END TYPE
'
CALL header                                                       'show header
'
CALL param(sig, r, B, x, y, z, h, tmax)                             'get input
'
CALL prep(h, tmax)                                'prepare screen for graphics
'
CALL calc(sig, r, B, x, y, z, h, tmax)                            'study chaos
'
END

'
SUB calc (sig, r, B, x, y, z, h, tmax)
'*****************************************************************************
'subroutine to study Lorenz equation
'input variables: sig, r, B, x, y, z, h, tmax
'output variables: none
'global variables: none
'local variables: xp, yp, zp
'*****************************************************************************
'
65 PSET (x, z)
'
xp = sig * (y - x)                                           'Lorenz equations
yp = r * x - y - x * z
zp = x * y - B * z
'
x = x + h * xp                                                 'update x, y, z
y = y + h * yp
z = z + h * zp
t = t + h                                                      'time increment
'
145 IF t <= tmax THEN 65
'
END SUB

'
SUB header
 '****************************************************************************
 'subroutine to display header screen
 'input variables:  none
 'output variables: none
 'global variables: none
 'local variables:  none
 '****************************************************************************
 CLS : KEY OFF                                                   'clear screen
 SCREEN 2, 0, 0: WIDTH 80                                       'change screen
  '
 LINE (1, 1)-(638, 40), , B                                        'draw boxes
 LINE (5, 3)-(633, 37), , B
 LINE (70, 50)-(570, 140), , B
 LINE (75, 53)-(565, 137), , B
						   'print title, authors, etc.
 LOCATE 3, 7
 PRINT " C O M P U T A T I O N A L   P H Y S I C S :";
 PRINT " (An  Introduction)"
 LOCATE 9, 40
 PRINT "by";
 LOCATE 11, 20: PRINT "R.C. Verma, P.K. Ahluwalia & K.C. Sharma"
 LOCATE 13, 34
 PRINT "   * * * *    "
 '
 BEEP
 LOCATE 15, 35: PRINT "* * * * * *"
 LOCATE 22, 25: PRINT "press any key to continue"
 LOCATE 23, 25: PRINT "-------------------------"
 '
10100 IF INKEY$ = "" THEN 10100                               'hold the screen
 CLS                                                             'clear screen
 END SUB

'
SUB param (sig, r, B, x, y, z, h, tmax)
'*****************************************************************************
'subroutine to introduce the program & get input parameters
'input variables:  none
'output variables: sig, r, B, x, y, z, h, tmax
'global variables: none
'local variables:  none
'Test case:       h = 0.001, tmax = 1
'*****************************************************************************
'
LOCATE 2, 5: PRINT "This program studies Lorenz equation"
LOCATE 4, 5: PRINT "----------------------------"
LOCATE 6, 5: PRINT "Please give  the following parameters:"
'
sig = 10: r = 16: B = 8 / 3                  'fix constants used by Lorenz eq.
'
x = z = 0: y = 1                       'initial conditions for Lorenz equation
'
1100 LOCATE 12, 5: INPUT "Give step size & max. Time: h < 0.1, Tmax: ", h, tmax
   IF h > 0 AND h < .1 AND tmax > 0 THEN 1200
   BEEP: LOCATE 12, 5: PRINT SPACE$(75): GOTO 1100
'
1200 '
'
END SUB

'
SUB prep (h, tmax)
'*****************************************************************************
'subroutine to prepare screen for graphics
'input variables:  none
'output variables: none
'global variables: none
'local variables:  none
'*****************************************************************************
'
CLS
VIEW (10, 10)-(620, 154), , 1                            'choose screen region
WINDOW (-18, 46)-(20, -10)
'
LOCATE 1, 1: PRINT "Lorenz equations"
LOCATE 22, 1: PRINT USING "h = #.###, Tmax = ####.###"; h; tmax
'
END SUB

