'*****************************************************************************
'Chapter 15, Example 8: Lorenz equations, Map of successive maxima only
'COMPUTATIONAL PHYSICS: An Introduction
'by R. C. Verma, P. K. Ahluwalia and K. C. Sharma
'   * * * *    
'*****************************************************************************
'
DECLARE SUB header ()
DECLARE SUB param (sig, r, B, x0, y0, z0, t0, h, tmax)
DECLARE SUB prep (h, tmax)
DECLARE SUB calc (sig, r, B, x0, y0, z0, t0, h, tmax)
'
TYPE param
sig AS SINGLE
r AS SINGLE
B AS SINGLE
x0 AS SINGLE
y0 AS SINGLE
z0 AS SINGLE
t0 AS SINGLE
h AS SINGLE
tmax AS SINGLE
nt AS INTEGER
END TYPE
'
CALL header                                                       'show header
'
CALL param(sig, r, B, x0, y0, z0, t0, h, tmax)                      'get input
'
CALL prep(h, tmax)                                'prepare screen for graphics
'
CALL calc(sig, r, B, x0, y0, z0, t0, h, tmax)
							'study Lorenz equation
END

'
SUB calc (sig, r, B, x0, y0, z0, t0, h, tmax)
'*****************************************************************************
'subroutine to study Lorenz equations, Map of successive maxima
'input variables: sig, r, B, x0, y0, z0, t0, h, tmax
'output variables: none
'global variables: none
'local variables: x, y, z, t, xp, yp, zp, nt, am(j), xi(j), zi(j)
'*****************************************************************************
'
nt = tmax / h + 5                                           'find no. of steps
'
DIM zi(nt), xi(nt)  AS SINGLE                                  'set dimensions
'
x = x0: y = y0: z = z0: t = t0                          'initialize quantities
j = 1
'
FOR I = 1 TO nt
  xp = sig * (y - x)                                         'Lorenz equations
  yp = r * x - y - x * z
  zp = x * y - B * z
  '
  x = x + h * xp                                               'update x, y, z
  y = y + h * yp
  z = z + h * zp
  '
  zi(I) = z: xi(I) = x
  LOCATE 19, 1
  PRINT USING " i = ####      x = +#.####^^^^     z = +#.####^^^^"; I; x; z
  t = t + h                                                    'increment time
'
IF I < 4 THEN 500                                              'map maxima
  IF (zi(I) - zi(I - 1)) < 0 AND (zi(I - 1) - zi(I - 2)) > 0 THEN am = zi(I - 1) ELSE 500
  LOCATE 21, 1
  PRINT USING " j = ####     x(####) = +#.####^^^^    am(####) = +#.####^^^^"; j; (I - 1); xi(I - 1); j; am
  PSET (xold, am)                                              'plot points
  xold = am
  j = j + 1
  IF j > nt THEN 600
500 NEXT I
'
600 '
'
END SUB

'
SUB header
 '****************************************************************************
 'subroutine to display header screen
 'input variables:  none
 'output variables: none
 'global variables: none
 'local variables:  none
 '****************************************************************************
 CLS : KEY OFF                                                   'clear screen
 SCREEN 2, 0, 0: WIDTH 80                                       'change screen
  '
 LINE (1, 1)-(638, 40), , B                                        'draw boxes
 LINE (5, 3)-(633, 37), , B
 LINE (70, 50)-(570, 140), , B
 LINE (75, 53)-(565, 137), , B
						    'print title, authors, etc.
 LOCATE 3, 7
 PRINT " C O M P U T A T I O N A L   P H Y S I C S :";
 PRINT " (An  Introduction)"
 LOCATE 9, 40
 PRINT "by";
 LOCATE 11, 20: PRINT "R.C. Verma, P.K. Ahluwalia & K.C. Sharma"
 LOCATE 13, 34
 PRINT "   * * * *    "
 '
 BEEP
 LOCATE 15, 35: PRINT "* * * * * *"
 LOCATE 22, 25: PRINT "press any key to continue"
 LOCATE 23, 25: PRINT "-------------------------"
 '
10100 IF INKEY$ = "" THEN 10100                               'hold the screen
 CLS                                                             'clear screen
 END SUB

'
SUB param (sig, r, B, x0, y0, z0, t0, h, tmax)
'*****************************************************************************
'subroutine to introduce the program & get input parameters
'input variables:  none
'output variables: sig, r, B, x0, y0, z0, t0, h, tmax, nt
'global variables: none
'local variables:  none
'Test case:        h=.01, tmax=250
'*****************************************************************************
'
LOCATE 2, 5: PRINT "This program studies Lorenz equation"
LOCATE 3, 5: PRINT "   & map maxima "
LOCATE 4, 5: PRINT "------------------------------------"
LOCATE 6, 5: PRINT "Please give  the following parameters:"
'
sig = 10: r = 24.74: B = 8 / 3                       'constants used by Lorenz
'
x0 = z0 = 0: y0 = 1: t0 = 0                                'initial conditions
'
1100 LOCATE 10, 5: INPUT " Give step size h  < 0.1  ", h
  IF h > 0 AND h < .1 THEN 1200
  BEEP: LOCATE 10, 5: PRINT SPACE$(72): GOTO 1100
'
1200 LOCATE 12, 5: PRINT " Give maximum time <= "; 2000 * h
  LOCATE 14, 5: INPUT "Tmax  = "; tmax
  IF tmax > 0 AND (tmax <= 2000 * h + 1) THEN 1300
  BEEP: LOCATE 14, 5: PRINT SPACE$(72): GOTO 1200
1300
'
END SUB

'
SUB prep (h, tmax)
'*****************************************************************************
'subroutine to prepare screen for graphics
'input variables:  none
'output variables: none
'global variables: none
'local variables:  none
'*****************************************************************************
CLS
VIEW (10, 20)-(620, 140), , 1                            'choose screen region
WINDOW (0, 54)-(54, 25)
'
LOCATE 1, 10: PRINT "Lorenz equations: Map of successive maxima"
LOCATE 2, 20: PRINT USING "h = ##.###, Tmax = ####.###"; h; tmax
'
END SUB

