10 '**************************************************************************
20 'Chapter 1, Example 2: Motion of a Spherical Body in a Viscous Medium
30 'COMPUTATIONAL PHYSICS: An Introduction
40 'by R. C. Verma, P. K. Ahluwalia and K. C. Sharma
50 '   * * * *   
60 '**************************************************************************
70  CLS : KEY OFF
80  GOSUB 10000                                                   'show header
90 '
100 PI = 3.1415926#                    'define constants pi & drag coefficient
110 '
120 GOSUB 1000                'introduce the problem & to get input parameters
130  CLS
140  N% = (TF - TI) / H       'find number of steps to cover the time interval
150 '
160 VIS = 6 * PI * ETA * RAD / MASS                     'calculate drag factor
170  IF VIS <> 0 THEN 230                               'check if drag is zero
180  CLS : BEEP
190  LOCATE 12, 1: PRINT "Viscous force vanishes."
195    PRINT " Velocity would not saturate!"
200  LOCATE 15, 15: INPUT "Do you want me to work? press Y/N"; CHC$
210    IF CHC$ = "y" OR CHC$ = "Y" THEN 230
215    IF CHC$ = "n" OR CHC$ = "N" THEN 250
220    BEEP: LOCATE 15, 15: PRINT SPACE$(65): GOTO 200
225 '
230 GOSUB 2000                                           'perform Euler method
240 '
250 CLS : LOCATE 12, 15: BEEP: PRINT " Thank you !"
260 END
270 '
1000 '************************************************************************
1010 'subroutine to introduce the problem & to get input variables
1020 'input variables:  none
1030 'output variables: RAD, MASS, ETA, GRAV, VINIT, TI, TF, H
1040 '                  TUNIT$, VUNIT$, AUNIT$
1050 'global variables: none
1060 'local variables:  units
1070 'test cases:RAD=1, MASS=1, ETA=0.1, GRAV=9.8, VINIT=0,  TI=0, TF=2, H=0.01
1080 '           RAD=1, MASS=1, ETA=0.1, GRAV=9.8, VINIT=20, TI=0, TF=2, H=0.01
1090 '           RAD=1, MASS=1, ETA=0.1, GRAV=9.8, VINIT=-20,TI=0, TF=2, H=0.01
1100 '************************************************************************
1110 '
1120 LOCATE 2, 10: PRINT "This program describes motion of a spherical ball"
1130 LOCATE 3, 15: PRINT "falling in a viscous medium."
1140 LOCATE 4, 10: PRINT "---------------------------------------------------"
1150                                                   'choose system of units
1160 LOCATE 5, 10: BEEP: PRINT "Which systems of units you want to work in?"
1170 LOCATE 7, 5: PRINT SPACE$(75)
1175 LOCATE 7, 15: INPUT "1) m.k.s., 2) c.g.s.     Press 1 or 2."; UNITS
1180   IF UNITS = 1 OR UNITS = 2 THEN 1210
1190   BEEP: PRINT "Try again!"
1200   GOTO 1170
1210 '
1220 TUNIT$ = "s"
1230 IF UNITS = 1 THEN VUNIT$ = "m/s": AUNIT$ = "m/s2"
1240 IF UNITS = 2 THEN VUNIT$ = "cm/s": AUNIT$ = "cm/s2"
1250 '
1260 LOCATE 9, 5: BEEP: PRINT "Please feed the input parameters"
1265 LOCATE 10, 1: PRINT SPACE$(78)         'physical properties of the system
1270 LOCATE 10, 1: INPUT "Give radius of the  ball "; RAD
1275   IF RAD >= 0 THEN 1290
1280   BEEP: PRINT "Please give suitable value for radius!"
1285   GOTO 1265
1290 LOCATE 12, 1: PRINT SPACE$(78)
1300 LOCATE 12, 1: INPUT "Give mass  of the  ball "; MASS
1305   IF MASS > 0 THEN 1320
1310   BEEP: PRINT "Please give positive value for mass!"
1315   GOTO 1290
1320 LOCATE 14, 1: PRINT SPACE$(78)
1325 LOCATE 14, 1: INPUT "Give coefficient of viscosity of the medium "; ETA
1330   IF ETA >= 0 THEN 1360
1340   BEEP: PRINT "Please give postive value for the viscosity!"
1350   GOTO 1320
1360 LOCATE 16, 1: PRINT SPACE$(78)
1365 LOCATE 16, 1: INPUT "Give acceleration due to gravity "; GRAV
1370  IF GRAV <> 0 THEN 1375 ELSE BEEP: GOTO 1360
1375 LOCATE 18, 1: PRINT SPACE$(78)                'initial conditions
1380 LOCATE 18, 1: INPUT "Give initial and final times: Ti, Tf"; TI, TF
1385  IF TF > TI THEN 1395
1390  BEEP: GOTO 1375
1395 LOCATE 20, 1: PRINT SPACE$(78)
1400 LOCATE 20, 1: INPUT "Give initial velocity of the ball "; VINIT
1410 LOCATE 22, 1: PRINT SPACE$(78)
1420 LOCATE 22, 1: INPUT "Give time step h "; H
1425   IF H > 0 THEN 1435
1430   BEEP: GOTO 1410
1435                                    'for comparatively large r and small m
1440                                              'time step h should be small
1450 RETURN
1460 '
2000 '************************************************************************
2010 'subroutine to perform Euler method
2020 'input variables:  TI, VINIT, GRAV, H, N%, VIS, TUNIT$, VUNIT$, AUNIT$
2030 'output variables: N%, T, X, V, A, VNUM, VEXACT
2040 'global variables: none
2050 'local variables:  NUM%
2060 '************************************************************************
2070 '
2080 T = TI: V = VINIT: A = GRAV - VIS * V                 'initial conditions
2090 '
2100 IF N% > 40 THEN NUM% = INT(N% / 20) ELSE NUM% = 2 'for selective printout
2110 '
2120 PRINT "---------------------------------"       'prepare table for output
2130 PRINT " Time    Acceleration  Velocity "
2140 PRINT "     "; TUNIT$; "       "; AUNIT$; "          "; VUNIT$
2150 PRINT "---------------------------------"
2160 '
2170 FOR I% = 0 TO N%
2180   IF I% = 0 THEN 2220                                         'print only
2190   T = T + H
2200   V = V + A * H                            'Euler method to find velocity
2210   A = GRAV - VIS * V                              'acceleration decrement
2220   IF (I% MOD NUM%) <> 0 AND I% <> N% THEN 2240         'print selectively
2230   PRINT USING "+###.###   +#.###^^^^   +#.###^^^^"; T; A; V
2240 NEXT I%
2250 '
2260 PRINT "---------------------------------"                     'table ends
2270 '
2280 VNUM = V                               'numerical value of final velocity
2290 IF VIS <> 0 THEN 2320                         'check if viscosity is zero
2300 VEXACT = VINIT + (TF - TI) * GRAV'analytical value when viscosity is zero
2310  GOTO 2340
2320 VEXACT = GRAV / VIS           'analytical value when viscosity is nonzero
2330 '
2340 PRINT : PRINT "Final Velocity:": PRINT
2350 PRINT "Numerical result = "; VNUM; " "; VUNIT$
2360 PRINT "Analytical result = "; VEXACT; " "; VUNIT$
2370 '
2380 BEEP: PRINT : PRINT "press any key to continue"
2390 IF INKEY$ = "" THEN 2390
2400 '
2410 RETURN
2420 '
10000 '***********************************************************************
10010 'subroutine to display header screen
10020 'input variables:  none
10030 'output variables: none
10040 'global variables: none
10050 'local variables:  none
10060 '***********************************************************************
10070 CLS : KEY OFF                                           'clear screen
10080 SCREEN 2, 0, 0: WIDTH 80                               'change screen
10090  '
10100 LINE (1, 1)-(638, 40), , B                                'draw boxes
10110 LINE (5, 3)-(633, 37), , B
10120 LINE (70, 50)-(570, 140), , B
10130 LINE (75, 53)-(565, 137), , B
10140                                           'print title, authors, etc.
10150 LOCATE 3, 7
10160 PRINT " C O M P U T A T I O N A L   P H Y S I C S :";
10170 PRINT " (An  Introduction)"
10180 LOCATE 9, 40
10190 PRINT "by";
10200 LOCATE 11, 20: PRINT "R.C. Verma, P.K. Ahluwalia & K.C. Sharma"
10210 LOCATE 13, 34
10220 PRINT "   * * * *    "
10230 '
10240 BEEP
10250 LOCATE 15, 35: PRINT "* * * * * *"
10260 LOCATE 22, 25: PRINT "press any key to continue"
10270 LOCATE 23, 25: PRINT "-------------------------"
10280 '
10290 IF INKEY$ = "" THEN 10290                            'hold the screen
10300 CLS                                                     'clear screen
10310 RETURN
10320 '

