10 '**************************************************************************
20 'Chapter 1, Example 3: Motion of a Falling Body including Air Drag
25 '                       (using subscripted variables)
30 'COMPUTATIONAL PHYSICS: An Introduction
40 'by R. C. Verma, P. K. Ahluwalia and K. C. Sharma
50 '   * * * *   
60 '**************************************************************************
70  CLS : KEY OFF
80  GOSUB 10000                                                   'show header
90 '
100 PI = 3.1415926#                    'define constants pi & drag coefficient
110 DC = .46                            'drag coefficient for a spherical body
120 '
130 GOSUB 1000                'introduce the problem & to get input parameters
140 CLS
150  N% = (TF - TI) / H       'find number of steps to cover the time interval
160 DIM T(N%), V(N%), A(N%), X(N%), Y(N%)
170 DIM PX%(N%), PY%(N%), PMX%(5), PMY%(5)
180                                     'set dimensions of the array variables
190 DEC = .5 * DC * RHO * PI * RAD * RAD / MASS         'calculate drag factor
200  IF DEC <> 0 THEN 260                               'check if drag is zero
210  CLS : BEEP
215 LOCATE 12, 1: PRINT "Drag factor is zero."
220    PRINT "    Velocity would not saturate!"
225  LOCATE 15, 15: INPUT "Do you want me to work? press Y/N"; CHC$
230    IF CHC$ = "y" OR CHC$ = "Y" THEN 260
235    IF CHC$ = "n" OR CHC$ = "N" THEN 270
240    BEEP: LOCATE 15, 15: PRINT SPACE$(65): GOTO 225
250 '
260 GOSUB 2000                                           'perform Euler method
270 '
280 CLS : LOCATE 12, 15: BEEP: PRINT "Program ends here!"
290 END
300 '
1000 '************************************************************************
1010 'subroutine to introduce the problem & to get input variables
1020 'input variables:  none
1030 'output variables: RAD, MASS, RHO, GRAV, VINIT, TI, TF, H
1040 '                  TUNIT$, VUNIT$, AUNIT$
1050 'global variables: none
1060 'local variables:  units
1070 'test cases:RAD=1, MASS=1, RHO=0.1, GRAV=9.8, VINIT=0,  TI=0, TF=2, H=0.01
1080 '           RAD=1, MASS=1, RHO=0.1, GRAV=9.8, VINIT=20, TI=0, TF=2, H=0.01
1090 '           RAD=1, MASS=1, RHO=0.1, GRAV=9.8, VINIT=-20,TI=0, TF=2, H=0.01
1100 '************************************************************************
1110 '
1120 LOCATE 2, 10: PRINT "This program describes motion of a spherical ball"
1130 LOCATE 3, 15: PRINT "falling in air."
1140 LOCATE 4, 10: PRINT "---------------------------------------------------"
1150                                                   'choose system of units
1160 LOCATE 5, 10: BEEP: PRINT "Which systems of units you want to work in?"
1170 LOCATE 7, 15: PRINT SPACE$(65)
1175 LOCATE 7, 15: INPUT "1) m.k.s., 2) c.g.s.     Press 1 or 2."; UNITS
1180   IF UNITS = 1 OR UNITS = 2 THEN 1205
1190   BEEP: PRINT "Try again!"
1200   GOTO 1170
1205 TUNIT$ = "s"
1210 IF UNITS = 1 THEN VUNIT$ = "m/s": AUNIT$ = "m/s2"
1215 IF UNITS = 2 THEN VUNIT$ = "cm/s": AUNIT$ = "cm/s2"
1220 '
1225 LOCATE 8, 1: BEEP: PRINT "Please feed the input parameters"
1230 LOCATE 10, 1: PRINT SPACE$(78)         'physical properties of the system
1235 LOCATE 10, 1: INPUT "Give radius of the  ball "; RAD
1240   IF RAD >= 0 THEN 1265
1245   BEEP: PRINT "Please give suitable value for radius!"
1250   GOTO 1230
1265 LOCATE 12, 1: PRINT SPACE$(78)         'physical properties of the system
1270 LOCATE 12, 1: INPUT "Give mass  of the  ball  "; MASS
1280   IF MASS > 0 THEN 1310
1290   BEEP: PRINT "Please give positive value for mass!"
1300   GOTO 1265
1310 '
1320 LOCATE 14, 1: PRINT SPACE$(78)
1325 LOCATE 14, 1: INPUT "Give density of the medium (air) "; RHO
1330   IF RHO >= 0 THEN 1360
1340   BEEP: PRINT "Please give suitable value for density!"
1350   GOTO 1320
1360 LOCATE 16, 1: PRINT SPACE$(78)
1365 LOCATE 16, 1: INPUT "Give acceleration due to gravity "; GRAV
1370  IF GRAV <> 0 THEN 1380
1375  BEEP: GOTO 1360                                      'initial conditions
1380 LOCATE 18, 1: PRINT SPACE$(78)
1385 LOCATE 18, 1: INPUT "Give initial and final times: Ti, Tf"; TI, TF
1390   IF TF > TI THEN 1420
1400   BEEP: PRINT "Please give suitable time interval Ti, Tf <1000!"
1410   GOTO 1380
1420 LOCATE 20, 1: INPUT "Give initial velocity of the ball, Vo "; VINIT
1430 '
1440  HMIN = (TF - TI) / 2000                       'minimum step size allowed
1450 LOCATE 22, 1: PRINT SPACE$(78)
1455 LOCATE 22, 1: INPUT "Give time step h   "; H
1460  IF H >= HMIN THEN 1500
1470  BEEP: PRINT "Please give h > "; HMIN; " due to limit on array-memory"
1480  BEEP: GOTO 1450
1500                                    'for comparatively large r and small m
1510                                              'time step h should be small
1520 RETURN
1530 '
2000 '************************************************************************
2010 'subroutine to perform Euler method
2020 'input variables:  TI, VINIT, GRAV, H, N%, DEC, TUNIT$, VUNIT$, AUNIT$
2030 'output variables: N%, T(I%), V(I%), A(I%), VNUM, VEXACT
2040 'global variables: none
2050 'local variables:  NUM%
2060 '************************************************************************
2070 '
2080 T(0) = TI: V(0) = VINIT: A(0) = GRAV - DEC * V(0) * ABS(V(0))'initial conditions
2090 '
2100 IF N% > 40 THEN NUM% = INT(N% / 20) ELSE NUM% = 2 'for selective printout
2110 '
2120 PRINT "---------------------------------"       'prepare table for output
2130 PRINT " Time    Acceleration  Velocity "
2140 PRINT "     "; TUNIT$; "       "; AUNIT$; "          "; VUNIT$
2150 PRINT "---------------------------------"
2160 '
2170 FOR I% = 0 TO N%
2180   IF I% = 0 THEN 2220                                         'print only
2190   T(I%) = T(I% - 1) + H
2200   V(I%) = V(I% - 1) + A(I% - 1) * H        'Euler method to find velocity
2210   A(I%) = GRAV - DEC * V(I%) * ABS(V(I%))         'acceleration decrement
2220   IF (I% MOD NUM%) <> 0 AND I% <> N% THEN 2240         'print selectively
2230   PRINT USING "+###.###   +#.###^^^^   +#.###^^^^"; T(I%); A(I%); V(I%)
2240 NEXT I%
2250 '
2260 PRINT "---------------------------------"                     'table ends
2270 '
2280 VNUM = V(N%)                           'numerical value of final velocity
2290 IF DEC <> 0 THEN 2320                              'check if drag is zero
2300 VEXACT = VINIT + (TF - TI) * GRAV     'analytical value when drag is zero
2310  GOTO 2340
2320 VEXACT = SQR(GRAV / DEC)           'analytical value when drag is nonzero
2330 '
2340 PRINT : PRINT "Final Velocity:": PRINT
2350 PRINT "Numerical result = "; VNUM; " "; VUNIT$
2360 PRINT "Analytical result = "; VEXACT; " "; VUNIT$
2370 '
2380 BEEP: PRINT : PRINT "press any key to continue"
2390 IF INKEY$ = "" THEN 2390
2400 '
2410 RETURN
2420 '
10000 '***********************************************************************
10010 'subroutine to display header screen
10020 'input variables:  none
10030 'output variables: none
10040 'global variables: none
10050 'local variables:  none
10060 '***********************************************************************
10070 CLS : KEY OFF                                           'clear screen
10080 SCREEN 2, 0, 0: WIDTH 80                               'change screen
10090  '
10100 LINE (1, 1)-(638, 40), , B                                'draw boxes
10110 LINE (5, 3)-(633, 37), , B
10120 LINE (70, 50)-(570, 140), , B
10130 LINE (75, 53)-(565, 137), , B
10140                                           'print title, authors, etc.
10150 LOCATE 3, 7
10160 PRINT " C O M P U T A T I O N A L   P H Y S I C S :";
10170 PRINT " (An  Introduction)"
10180 LOCATE 9, 40
10190 PRINT "by";
10200 LOCATE 11, 20: PRINT "R.C. Verma, P.K. Ahluwalia & K.C. Sharma"
10210 LOCATE 13, 34
10220 PRINT "   * * * *    "
10230 '
10240 BEEP
10250 LOCATE 15, 35: PRINT "* * * * * *"
10260 LOCATE 22, 25: PRINT "press any key to continue"
10270 LOCATE 23, 25: PRINT "-------------------------"
10280 '
10290 IF INKEY$ = "" THEN 10290                            'hold the screen
10300 CLS                                                     'clear screen
10310 RETURN
10320 '

