10 '**************************************************************************
20 'Chapter 3, Example 1: Motion of a body attached to a spring
30 'COMPUTATIONAL PHYSICS: An Introduction
40 'by R. C. Verma, P. K. Ahluwalia and K. C. Sharma
50 '   * * * *   
60 '**************************************************************************
70  CLS : KEY OFF
80  GOSUB 10000                                                   'show header
90 '
100 PI = 3.1415926#                    'define constants pi & drag coefficient
110 '
120 GOSUB 1000                'introduce the problem & to get input parameters
130 NSTEP% = (TF - TI) / H
140                           'find number of steps to cover the time interval
150 DIM T(NSTEP% + 1), S(NSTEP% + 1), V(NSTEP% + 1), A(NSTEP% + 1)
160 DIM X(NSTEP% + 1), Y(NSTEP% + 1), E(NSTEP% + 1)
170 DIM PX%(NSTEP% + 1), PY%(NSTEP% + 1), PMX%(5), PMY%(5)
180 '                                                    'dimension the arrays
190 GOSUB 2000                                       'perform Half-step method
200 GOSUB 3000                                                    'draw graphs
210 '
220 CLS : LOCATE 12, 15: BEEP: PRINT "Program ends here!"
230 END
240 '
1000 '************************************************************************
1010 'subroutine to introduce the problem & to get input variables
1020 'input variables:  none
1030 'output variables: KSPRING, MASS, TI, TF, SI, VI, TPERIOD
1040 '                  SAMP, VAMP, AAMP
1050 '                  H, TUNIT$, SUNIT$, VUNIT$, AUNIT$, MUNIT$, EUNIT$
1060 'global variables: none
1070 'local variables:  UNITS, OMEGA, HMIN
1080 'test cases: KSPRING=1, MASS=1, So=1, Vo=0, Ti=0: TF=15, H=0.01
1090            'For large k, small m keep time interval small/ large damping
1100 '************************************************************************
1110 '
1120 LOCATE 2, 10
1125   PRINT "This program describes motion of 1-DIM. Harmonic oscillator"
1130 LOCATE 3, 10: PRINT "---------------------------------------------------"
1140                                                   'choose system of units
1150 LOCATE 4, 10: BEEP: PRINT "Which systems of units you want to work in?"
1160 LOCATE 6, 15: PRINT "1) m.k.s., 2) c.g.s."
1170 LOCATE 8, 15: PRINT SPACE$(65)
1175 LOCATE 8, 15: INPUT "Press 1 or 2."; UNITS
1180   IF UNITS = 1 OR UNITS = 2 THEN 1210
1190   BEEP: PRINT "Try again!"
1200   GOTO 1170
1210 '
1220   TUNIT$ = "s"
1230 IF UNITS <> 1 THEN 1250
1240  SUNIT$ = "m": MUNIT$ = "kg": VUNIT$ = "m/s": AUNIT$ = "m/s2"
1250 IF UNITS <> 2 THEN 1270
1260  SUNIT$ = "cm": MUNIT$ = "g": VUNIT$ = "cm/s": AUNIT$ = "cm/s2"
1270 '
1280 LOCATE 9, 1: BEEP: PRINT "Please feed the input parameters"
1290                                        'physical properties of the system
1295 LOCATE 11, 1: PRINT SPACE$(78)
1300 LOCATE 11, 1: INPUT "Give spring constant: k "; KSPRING
1305   IF KSPRING > 0 THEN 1320
1310   BEEP: PRINT "Please give suitable value for k!"
1315   GOTO 1295
1320 LOCATE 13, 1: PRINT SPACE$(78)
1325 LOCATE 13, 1: INPUT "Give value of mass attached: m "; MASS
1330   IF MASS > 0 THEN 1345
1335   BEEP: PRINT "Please give +ve value of m!"
1340   GOTO 1320
1345 '
1350   TPERIOD = 2 * PI / SQR(KSPRING / MASS)
1355                      'expected time period for input purpose and graphics
1360   OMEGA = 2 * PI / TPERIOD
1380 '                                                          'time interval
1390 LOCATE 15, 1: PRINT SPACE$(78)
1395 LOCATE 15, 1: INPUT "Give initial and final times: Ti, Tf"; TI, TF
1400   IF TF > TI THEN 1430
1410   BEEP: PRINT "Give suitable time interval Ti, Tf!"
1420   GOTO 1390
1430 '
1440  HMIN = (TF - TI) / 1000                       'minimum step size allowed
1450 LOCATE 17, 1: PRINT SPACE$(78)
1455 LOCATE 17, 1: INPUT "Give time step h "; H
1460  IF H >= HMIN AND H < TPERIOD / 10 THEN 1485
1470  BEEP: PRINT "Please give "; HMIN; " < h < "; TPERIOD / 10
1480  GOTO 1450                                  'array size & h < time period
1485 '
1490 LOCATE 19, 1: PRINT SPACE$(78): LOCATE 20, 1: PRINT SPACE$(78)
1495 LOCATE 19, 1: INPUT "Give initial position: Xo "; SI
1500 LOCATE 20, 1: INPUT "Give initial velocity: Vo "; VI
1505   IF SI = 0 AND VI = 0 THEN 1510 ELSE 1550
1510   BEEP: PRINT "With X0 = 0 & V0 = 0, oscillator will always be at rest!"
1520         PRINT "Please keep either Xo or Vo nonzero"
1530   GOTO 1490
1540 '
1550         'Using initial conditions find max. S, V & A for graphics purpose
1560 SAMP = SQR(SI ^ 2 + (VI / OMEGA) ^ 2)                  'Max. displacement
1570 VAMP = SAMP * OMEGA                                    'Max. displacement
1580 AAMP = SAMP * OMEGA * OMEGA                            'Max. acceleration
1590 '
1600 RETURN
1610 '
2000 '************************************************************************
2010 'subroutine to perform Feynman-Newton Half-step method
2020 'input variables:  KSPRING. MASS, TI, TF, SI, VI, H, NSTEP%,
2030 '                  TUNIT$, SUNIT$, VUNIT$, AUNIT$, PI
2040 'output variables: N%, T(I%), S(I%), V(I%), A(I%)
2050 'global variables: none
2060 'local variables:  NUM%,
2070 '************************************************************************
2080 '
2090 IF NSTEP% > 40 THEN NUM% = INT(NSTEP% / 20) ELSE NUM% = 2
2100 '
2110  T(0) = TI: S(0) = SI: V(0) = VI                      'initial conditions
2120   A(0) = -KSPRING * S(0) / MASS                     'initial acceleration
2130   V(1) = V(0) + .5 * A(0) * H                      'velocity at half-step
2140 '
2150 PRINT " Position, Velocity, Acceleration & Energy of Harmonic Oscillator"
2160 PRINT "----------------------------------------------------------------"
2170 PRINT "  Time         A           V          S       E"
2180 PRINT "   "; TUNIT$; "         "; AUNIT$; "        "; VUNIT$;
2185 PRINT "          "; SUNIT$; "          "; EUNIT$
2190 PRINT "----------------------------------------------------------------"
2200 '                                                       'half-step method
2210 FOR I% = 0 TO NSTEP%
2220   IF I% = 0 THEN 2270                                         'print only
2230   T(I%) = T(I% - 1) + H
2240   S(I%) = S(I% - 1) + V(I%) * H                         'distance formula
2250   A(I%) = -KSPRING * S(I%) / MASS                       'new acceleration
2260   V(I% + 1) = V(I%) + A(I%) * H                         'velocity formula
2270   E(I%) = (MASS * V(I%) * V(I%) + KSPRING * S(I%) * S(I%)) / 2
2275                                                        'find total energy
2280   IF (I% MOD NUM%) <> 0 AND I% <> NSTEP% THEN 2300     'print selectively
2290   PRINT USING "##.###   +#.####^^^^   +#.####^^^^   +##.####^^^^  +##.####^^^^"; T(I%); A(I%); V(I%); S(I%); E(I%)
2300 NEXT I%
2310 '
2320 PRINT "----------------------------------------------------------------"
2330                                                               'table ends
2340 BEEP: PRINT : PRINT "press any key to continue": PRINT
2350 IF INKEY$ = "" THEN 2350
2360 '
2370 RETURN
2380 '
3000 '************************************************************************
3010 'subroutine to draw: displacement, velocity & acceleration vs. time
3020 'input variables:  N%, T(I%), S(I%), V(I%), A(I%),
3030 'output variables: X(I%), Y(I%), TITLE$, SUBTITLE$, XTITLE$, YTITLE$
3040 '                  XMIN, XMAX, YMIN, YMAX, PXMIN%, PXMAX%, PYMIN%, PYMAX%
3050 'global variables: none
3060 'local variables:  none
3070 '************************************************************************
3080 CLS                                                         'clear screen
3090 '
3100 TITLE$ = "* * *  Motion of 1-Dimensional Harmonic Oscillator  * * *"
3110 '
3120                                         'draw displacement vs. time graph
3130 PXMIN% = 136: PXMAX% = 456: PYMIN% = 28: PYMAX% = 148
3140                                                     'choose screen region
3150 SUBTITLE$ = "Displacement vs. time": XTITLE$ = "t": YTITLE$ = "S"
3160 '
3170 FOR I% = 0 TO NSTEP%
3180  X(I%) = T(I%): Y(I%) = S(I%)         'assign variables for S vs. t graph
3190 NEXT I%
3200 '
3210 XMIN = TI: XMAX = TF                                 'limits for graphics
3220 YMAX = SAMP: YMIN = -YMAX
3230 '
3240 GOSUB 4000                                                 'display graph
3250 '
3260                                             'draw velocity vs. time graph
3270 SUBTITLE$ = "Velocity vs. time": XTITLE$ = "t": YTITLE$ = "V"
3280 '
3290 FOR I% = 0 TO NSTEP%
3300   Y(I%) = V(I%)                       'assign variables for V vs. t graph
3310 NEXT I%
3320 '
3330 YMAX = VAMP: YMIN = -YMAX                            'limits for graphics
3340 '
3350 GOSUB 4000                                                 'display graph
3360 '
3370                                         'draw acceleration vs. time graph
3380 SUBTITLE$ = "Acceleration vs. time": XTITLE$ = "t": YTITLE$ = "A"
3390 '
3400 FOR I% = 0 TO NSTEP%
3410  Y(I%) = A(I%)                        'assign variables for A vs. t graph
3420 NEXT I%
3430 '
3440 YMAX = AAMP: YMIN = -YMAX                            'limits for graphics
3450 '
3460 GOSUB 4000                                                 'display graph
3470                                              'draw phase space trajectory
3480 SUBTITLE$ = "Phase space curve": XTITLE$ = "X": YTITLE$ = "V"
3490 '
3500 FOR I% = 0 TO NSTEP%
3510  X(I%) = S(I%): Y(I%) = V(I%)         'assign variables for V vs. S graph
3520 NEXT I%
3530 '
3540 XMAX = SAMP: XMIN = -XMAX                            'limits for graphics
3550 YMAX = VAMP: YMIN = -YMAX
3560 '
3570 GOSUB 4000                                                 'display graph
3580 '
3590 RETURN
3600 '
4000 '************************************************************************
4010 'subroutine to display graph between x(i) & y(i) variables
4020 'input variables: N%, X(I%), Y(I%), XTITLE$, YTITLE$, TITLE$, SUBTITLE$
4030 '                 XMIN, XMAX, YMIN, YMAX, PXMIN%, PXMAX%, PYMIN%, PYMAX%
4040 'output variables: none
4050 'global variables: none
4060 'local variables: PXMIN%, PXMAX%, PYMIN%, PYMAX%
4070 '                 KCOEFF1, KCOEFF2, KCOEFF3 ,KCOEFF4, PX%(I%), PY%(I%)
4080 '                 DIV%, XSCALE, YSCALE, PXSCALE, PYSCALE
4090 '                 PMX%(J%), PMY%(J%), ROWX%, COLX%, ROWY%, COLY%
4100 '************************************************************************
4110 CLS                                                         'clear screen
4120 LOCATE 1, 15: PRINT SPACE$(64): PRINT TITLE$
4130  DIV% = 4                    'set number of divisions (<5) on x- & y-axes
4140                                                  'find scale coefficients
4150 KCOEFF1 = (PXMAX% - PXMIN%) / (XMAX - XMIN)
4160 KCOEFF2 = (PXMAX% * XMIN - PXMIN% * XMAX) / (XMIN - XMAX)
4170 KCOEFF3 = (PYMAX% - PYMIN%) / (YMIN - YMAX)
4180 KCOEFF4 = (PYMAX% * YMAX - PYMIN% * YMIN) / (YMAX - YMIN)
4190 '
4200 ROWTIT% = PYMIN% / 8 - 1: COLTIT% = PXMIN% / 8 + 1
4210 LOCATE ROWTIT% - 1, COLTIT%: PRINT SUBTITLE$           'print graph-title
4220 '
4230 LINE (PXMIN%, PYMIN%)-(PXMAX%, PYMAX%), , B           'draw box for graph
4240 LINE (PXMIN%, KCOEFF4)-(PXMAX%, KCOEFF4)                     'draw x-axis
4250 LINE (KCOEFF2, PYMIN%)-(KCOEFF2, PYMAX%)                     'draw y-axis
4260 '
4270 XSCALE = (XMAX - XMIN) / DIV%                       'find scale for marks
4280 YSCALE = (YMAX - YMIN) / DIV%
4290 '
4300 PXSCALE = (PXMAX% - PXMIN%) / DIV%            'find pixel scale for marks
4310 PYSCALE = (PYMAX% - PYMIN%) / DIV%
4320 '
4330 ROWX% = PYMAX% / 8 + 2              'find row for marks along x-direction
4340 COLY% = PXMIN% / 8 - 10          'find column for marks along y-direction
4350 '
4360 FOR J% = 0 TO DIV%                          'start loop to mark divisions
4370   PMX(J%) = PXMIN% + PXSCALE * J%                    'find places to mark
4380   PMY(J%) = PYMIN% + PYSCALE * J%
4390    LINE (PMX(J%), PYMAX%)-(PMX(J%), PYMAX% - 5)               'draw marks
4400    LINE (PXMIN%, PMY(J%))-(PXMIN% + 5, PMY(J%))
4410 '
4420   IF J% / 2 <> INT(J% / 2) THEN 4660            'jump for alternate marks
4430 '
4440   COLX% = PMX(J%) / 8 - 3         'find column for each mark along x-axis
4450   ROWY% = PMY(J%) / 8                'find row for each mark along y-axis
4460 '
4470   LOCATE ROWX%, COLX%
4480    IF XMAX > 999.99 OR XMIN < -999.99 THEN 4520
4490    IF XMAX < .1 AND XMIN > -.1 THEN 4520                'format selection
4500   PRINT USING "+###.##"; XMIN + XSCALE * J%      'print x-scale divisions
4510   GOTO 4540
4520   PRINT USING "+#.##^^^^"; XMIN + XSCALE * J%    'print x-scale divisions
4530 '
4540   LOCATE ROWY%, COLY%
4550    IF YMAX > 999.99 OR YMIN < -999.99 THEN 4590
4560    IF YMAX < .1 AND YMIN > -.1 THEN 4590                'format selection
4570   PRINT USING "+###.##"; YMAX - YSCALE * J%      'print y-scale divisions
4580   GOTO 4610
4590   PRINT USING "+#.##^^^^"; YMAX - YSCALE * J%    'print y-scale divisions
4600 '
4610   IF J% <> 2 THEN 4660              'print names of variables along x & y
4620    LOCATE ROWX% - 1, PXMAX% / 8 + 3: PRINT XTITLE$; "->"
4630    LOCATE ROWY% - 1, COLY% - 1: PRINT "^"
4640    LOCATE ROWY%, COLY% - 1: PRINT "|"
4650    LOCATE ROWY% + 1, COLY% - 1: PRINT YTITLE$
4660 NEXT J%
4670 '
4680 FOR I% = 0 TO NSTEP%
4690    PX%(I%) = KCOEFF1 * X(I%) + KCOEFF2     'perform scale transformations
4700    PY%(I%) = KCOEFF3 * Y(I%) + KCOEFF4                  'along x & y axes
4710    IF I% = 0 THEN 4730
4720    LINE (PX%(I% - 1), PY%(I% - 1))-(PX%(I%), PY%(I%))         'draw graph
4730 NEXT I%
4740 '
4750 LOCATE 22, 1: BEEP: PRINT "press any key to continue"
4760 IF INKEY$ = "" THEN 4760
4770 '
4780 RETURN
4790 '
10000 '***********************************************************************
10010 'subroutine to display header screen
10020 'input variables:  none
10030 'output variables: none
10040 'global variables: none
10050 'local variables:  none
10060 '***********************************************************************
10070 CLS : KEY OFF                                           'clear screen
10080 SCREEN 2, 0, 0: WIDTH 80                               'change screen
10090  '
10100 LINE (1, 1)-(638, 40), , B                                'draw boxes
10110 LINE (5, 3)-(633, 37), , B
10120 LINE (70, 50)-(570, 140), , B
10130 LINE (75, 53)-(565, 137), , B
10140                                           'print title, authors, etc.
10150 LOCATE 3, 7
10160 PRINT " C O M P U T A T I O N A L   P H Y S I C S :";
10170 PRINT " (An  Introduction)"
10180 LOCATE 9, 40
10190 PRINT "by";
10200 LOCATE 11, 20: PRINT "R.C. Verma, P.K. Ahluwalia & K.C. Sharma"
10210 LOCATE 13, 34
10220 PRINT "   * * * *    "
10230 '
10240 BEEP
10250 LOCATE 15, 35: PRINT "* * * * * *"
10260 LOCATE 22, 25: PRINT "press any key to continue"
10270 LOCATE 23, 25: PRINT "-------------------------"
10280 '
10290 IF INKEY$ = "" THEN 10290                            'hold the screen
10300 CLS                                                     'clear screen
10310 RETURN
10320 '

