10 '**************************************************************************
20 'Chapter 3, Example 2: Motion of a Damped Oscillator
30 'COMPUTATIONAL PHYSICS: An Introduction
40 'by R. C. Verma, P. K. Ahluwalia and K. C. Sharma
50 '   * * * *    
60 '**************************************************************************
70  CLS : KEY OFF
80  GOSUB 10000                                                   'show header
90 '
100 DEF FNA (X, V) = -(KSPRING * X + DAMPC * V) / MASS
110                                                   'define acceleration fn.
120 PI = 3.1415926#                    'define constants pi & drag coefficient
130 '
140 GOSUB 1000             'to introduce the problem & to get input parameters
150 '
160 NSTEP% = (TF - TI) / H
170                           'find number of steps to cover the time interval
180 DIM T(NSTEP% + 1), S(NSTEP% + 1), V(NSTEP% + 1), A(NSTEP% + 1)
190 DIM X(NSTEP% + 1), Y(NSTEP% + 1)
200 DIM PX%(NSTEP% + 1), PY%(NSTEP% + 1), PMX%(5), PMY%(5)
210 '                                                    'dimension the arrays
220 GOSUB 2000                                       'perform Half-step method
230 GOSUB 3000                                                    'draw graphs
240 '
250 CLS : LOCATE 12, 15: BEEP: PRINT "Program ends here!"
260 END
270 '
1000 '************************************************************************
1010 'subroutine to introduce the problem & to get input variables
1020 'input variables:  PI
1030 'output variables: KSPRING, MASS, DAMPC, TI, TF, SI, VI, TPERIOD
1035 '                  SAMP, VAMP, AAMP
1040 '                  H, TUNIT$, SUNIT$, VUNIT$, AUNIT$, MUNIT$
1050 'global variables: none
1060 'local variables:  UNITS, OMEGA
1070 'test cases: k=1, m=1, Vi=0, Si=1, tI=0: TF=15, H=0.01
1080            'For large k, small m keep time interval small/ large damping
1090 '************************************************************************
1100 '
1110 LOCATE 2, 10: PRINT "This program describes motion of a damped oscillator"
1120 LOCATE 3, 10: PRINT "----------------------------------------------------"
1130                                                   'choose system of units
1140 LOCATE 4, 10: BEEP: PRINT "Which systems of units you want to work in?"
1150 LOCATE 6, 15: PRINT "1) m.k.s., 2) c.g.s."
1160 LOCATE 8, 15: PRINT SPACE$(65)
1170 LOCATE 8, 15: INPUT "Press 1 or 2."; UNITS
1180   IF UNITS = 1 OR UNITS = 2 THEN 1210
1190   BEEP: PRINT "Try again!"
1200   GOTO 1160
1210 '
1220   TUNIT$ = "s"
1230 IF UNITS <> 1 THEN 1250
1240  SUNIT$ = "m": MUNIT$ = "kg": VUNIT$ = "m/s": AUNIT$ = "m/s2"
1250 IF UNITS <> 2 THEN 1270
1260  SUNIT$ = "cm": MUNIT$ = "g": VUNIT$ = "cm/s": AUNIT$ = "cm/s2"
1270 '
1280 LOCATE 9, 1: BEEP: PRINT "Please feed the input parameters"
1290                                        'physical properties of the system
1295 LOCATE 11, 1: PRINT SPACE$(78)
1300 LOCATE 11, 1: INPUT "Give spring constant: k "; KSPRING
1305   IF KSPRING > 0 THEN 1320
1310   BEEP: PRINT "Please give suitable value for k!"
1315   GOTO 1295
1320 LOCATE 12, 1: PRINT SPACE$(78)
1325 LOCATE 12, 1: INPUT "Give value of mass attached: m "; MASS
1330   IF MASS > 0 THEN 1345
1335   BEEP: PRINT "Please give +ve value of m!"
1340   GOTO 1320
1345 LOCATE 13, 1: PRINT SPACE$(78)
1350 LOCATE 13, 1: INPUT "Give damping coefficient: c "; DAMPC
1355   IF DAMPC >= 0 THEN 1370
1360   BEEP: PRINT "Please give suitable value for the coefficient!"
1365   GOTO 1345
1370 '
1375 TPERIOD = 2 * PI / SQR(KSPRING / MASS)
1380                      'expected time period for input purpose and graphics
1390 LOCATE 15, 1: PRINT SPACE$(78)
1395 LOCATE 15, 1: INPUT "Give initial and final times: Ti, Tf"; TI, TF
1400   IF TF > TI THEN 1430
1410   BEEP: PRINT "Give suitable time interval Ti, Tf!"
1420   GOTO 1390
1430 '
1440  HMIN = (TF - TI) / 1000                       'minimum step size allowed
1450 LOCATE 17, 1: PRINT SPACE$(78)
1455 LOCATE 17, 1: INPUT "Give time step h "; H
1460  IF H >= HMIN AND H < TPERIOD / 10 THEN 1485
1470  BEEP: PRINT "Please give "; HMIN; " < h < "; TPERIOD / 10
1480  GOTO 1450                                  'array size & h < time period
1485 LOCATE 19, 1: PRINT SPACE$(78): LOCATE 20, 1: PRINT SPACE$(78)
1490 LOCATE 19, 1: INPUT "Give initial position: Xo "; SI
1495 LOCATE 20, 1: INPUT "Give initial velocity: Vo "; VI
1500   IF SI = 0 AND VI = 0 THEN 1510 ELSE 1550
1510   BEEP: PRINT "With X0 = 0 & V0 = 0, oscillator will always be at rest!"
1520         PRINT "Please keep either Xo or Vo nonzero"
1530   GOTO 1485
1540         'Using initial conditions find max. S, V & A for graphics purpose
1550 OMEGA = 2 * PI / TPERIOD
1560 SAMP = SQR(SI ^ 2 + (VI / OMEGA) ^ 2)                  'Max. displacement
1570 VAMP = SAMP * OMEGA                                    'Max. displacement
1580 AAMP = SAMP * OMEGA * OMEGA                            'Max. acceleration
1590 '
1600 RETURN
1610 '
2000 '************************************************************************
2010 'subroutine to perform Feynman-Newton Half-step method
2020 'input variables:  KSPRING. MASS, DAMPC, TI, SI, VI, H, NSTEP%,
2030 '                  TUNIT$, SUNIT$, VUNIT$, AUNIT$
2040 'output variables: T(I%), S(I%), V(I%), A(I%)
2050 'global variables: none
2060 'local variables:  NUM%,
2070 '************************************************************************
2080 '
2090 IF NSTEP% > 40 THEN NUM% = INT(NSTEP% / 20) ELSE NUM% = 2
2100 '
2110  T(0) = TI: S(0) = SI: V(0) = VI                      'initial conditions
2120   A(0) = FNA(S(0), V(0))                            'initial acceleration
2130   V(1) = V(0) + .5 * A(0) * H                      'velocity at half-step
2140 '
2150 PRINT " Position, Velocity and Acceleration of Oscillator"
2160 PRINT "----------------------------------------------------"
2170 PRINT "  Time         A           V          S"
2180 PRINT "   "; TUNIT$; "         "; AUNIT$; "        "; VUNIT$; "           "; SUNIT$
2190 PRINT "----------------------------------------------------"
2200 '                                                       'half-step method
2210 FOR I% = 0 TO NSTEP%
2220   IF I% = 0 THEN 2270                                         'print only
2230   T(I%) = T(I% - 1) + H
2240   S(I%) = S(I% - 1) + V(I%) * H                         'distance formula
2250   A(I%) = FNA(S(I%), V(I%))                             'new acceleration
2260   V(I% + 1) = V(I%) + A(I%) * H                         'velocity formula
2270   IF (I% MOD NUM%) <> 0 AND I% <> NSTEP% THEN 2290     'print selectively
2280   PRINT USING "##.###   +#.####^^^^   +##.####^^^^  +##.####^^^^"; T(I%); A(I%); V(I%); S(I%)
2290 NEXT I%
2300 '
2310 PRINT "----------------------------------------------------"
2320                                                               'table ends
2330 PRINT : PRINT "press any key to continue": PRINT
2340 IF INKEY$ = "" THEN 2340
2350 '
2360 RETURN
2370 '
3000 '************************************************************************
3010 'subroutine to draw graphs: displacement, velocity & acceleration vs. time
3020 'input variables:  N%, T(I%), S(I%), V(I%), A(I%),
3030 'output variables: X(I%), Y(I%), TITLE$, SUBTITLE$, XTITLE$, YTITLE$
3040 '                  PXMIN%, PXMAX%, PYMIN%, PYMAX%
3050 'global variables: none
3060 'local variables:  none
3070 '************************************************************************
3080 CLS                                                         'clear screen
3090 '
3100 TITLE$ = "* * *  Motion of a Damped Oscillator  * * *"
3110 '
3120                                         'draw displacement vs. time graph
3130 PXMIN% = 120: PXMAX% = 280: PYMIN% = 20: PYMAX% = 70
3135                                                     'choose screen region
3140 SUBTITLE$ = "Displacement vs. time": XTITLE$ = "t": YTITLE$ = "S"
3150 '
3160 FOR I% = 0 TO NSTEP%
3170  X(I%) = T(I%): Y(I%) = S(I%)         'assign variables for S vs. t graph
3180 NEXT I%
3190 '
3200 XMIN = TI: XMAX = TF                                 'limits for graphics
3210 YMAX = SAMP: YMIN = -YMAX
3220 '
3230 GOSUB 4000                                                 'display graph
3240 '
3250                                             'draw velocity vs. time graph
3260 PXMIN% = 440: PXMAX% = 600: PYMIN% = 20: PYMAX% = 70
3265                                                     'choose screen region
3270 SUBTITLE$ = "Velocity vs. time": XTITLE$ = "t": YTITLE$ = "V"
3280 '
3290 FOR I% = 0 TO NSTEP%
3300   Y(I%) = V(I%)                       'assign variables for V vs. t graph
3310 NEXT I%
3320 '
3330 YMAX = VAMP: YMIN = -YMAX                            'limits for graphics
3340 '
3350 GOSUB 4000                                                 'display graph
3360 '
3370                                         'draw acceleration vs. time graph
3380 PXMIN% = 120: PXMAX% = 280: PYMIN% = 108: PYMAX% = 164
3385                                                     'choose screen region
3390 SUBTITLE$ = "Acceleration vs. time": XTITLE$ = "t": YTITLE$ = "A"
3400 '
3410 FOR I% = 0 TO NSTEP%
3420  Y(I%) = A(I%)                        'assign variables for A vs. t graph
3430 NEXT I%
3440 '
3450 YMAX = AAMP: YMIN = -YMAX                            'limits for graphics
3460 '
3470 GOSUB 4000                                                 'display graph
3480 '
3490                                              'draw phase space trajectory
3500 PXMIN% = 446: PXMAX% = 600: PYMIN% = 108: PYMAX% = 164
3505                                                     'choose screen region
3510 SUBTITLE$ = "Phase space curve": XTITLE$ = "X": YTITLE$ = "V"
3520 '
3530 FOR I% = 0 TO NSTEP%
3540  X(I%) = S(I%): Y(I%) = V(I%)         'assign variables for V vs. S graph
3550 NEXT I%
3560 '
3570 XMAX = SAMP: XMIN = -XMAX                            'limits for graphics
3580 YMAX = VAMP: YMIN = -YMAX
3590 '
3600 GOSUB 4000                                                 'display graph
3610 '
3620 LOCATE 23, 1: BEEP: PRINT "Press any key to continue"
3630 IF INKEY$ = "" THEN 3630
3640 '
3650 RETURN
3660 '
4000 '************************************************************************
4010 'subroutine to display graph between x(i) & y(i) variables
4020 'input variables: N%, X(I%), Y(I%), XTITLE$, YTITLE$, TITLE$, SUBTITLE$
4030 '                 XMIN, XMAX, YMIN, YMAX, PXMIN%, PXMAX%, PYMIN%, PYMAX%
4040 'output variables: none
4050 'global variables: none
4060 'local variables: PXMIN%, PXMAX%, PYMIN%, PYMAX%
4070 '                 KCOEFF1, KCOEFF2, KCOEFF3 ,KCOEFF4, PX%(I%), PY%(I%)
4080 '                 DIV%, XSCALE, YSCALE, PXSCALE, PYSCALE
4090 '                 PMX%(J%), PMY%(J%), ROWX%, COLX%, ROWY%, COLY%
4100 '************************************************************************
4110 LOCATE 1, 15: PRINT TITLE$
4120  DIV% = 4                    'set number of divisions (<5) on x- & y-axes
4130                                                  'find scale coefficients
4140 KCOEFF1 = (PXMAX% - PXMIN%) / (XMAX - XMIN)
4150 KCOEFF2 = (PXMAX% * XMIN - PXMIN% * XMAX) / (XMIN - XMAX)
4160 KCOEFF3 = (PYMAX% - PYMIN%) / (YMIN - YMAX)
4170 KCOEFF4 = (PYMAX% * YMAX - PYMIN% * YMIN) / (YMAX - YMIN)
4180 '
4190 ROWTIT% = PYMIN% / 8 - 1: COLTIT% = PXMIN% / 8 + 1
4200 LOCATE ROWTIT%, COLTIT%: PRINT SUBTITLE$               'print graph-title
4210 '
4220 LINE (PXMIN%, PYMIN%)-(PXMAX%, PYMAX%), , B           'draw box for graph
4230 LINE (PXMIN%, KCOEFF4)-(PXMAX%, KCOEFF4)                     'draw x-axis
4240 LINE (KCOEFF2, PYMIN%)-(KCOEFF2, PYMAX%)                     'draw y-axis
4250 '
4260 XSCALE = (XMAX - XMIN) / DIV%                       'find scale for marks
4270 YSCALE = (YMAX - YMIN) / DIV%
4280 '
4290 PXSCALE = (PXMAX% - PXMIN%) / DIV%            'find pixel scale for marks
4300 PYSCALE = (PYMAX% - PYMIN%) / DIV%
4310 '
4320 ROWX% = PYMAX% / 8 + 1              'find row for marks along x-direction
4330 COLY% = PXMIN% / 8 - 10          'find column for marks along y-direction
4340 '
4350 FOR J% = 0 TO DIV%                          'start loop to mark divisions
4360   PMX(J%) = PXMIN% + PXSCALE * J%                    'find places to mark
4370   PMY(J%) = PYMIN% + PYSCALE * J%
4380    LINE (PMX(J%), PYMAX%)-(PMX(J%), PYMAX% - 5)               'draw marks
4390    LINE (PXMIN%, PMY(J%))-(PXMIN% + 5, PMY(J%))
4400 '
4410   IF J% / 2 <> INT(J% / 2) THEN 4650            'jump for alternate marks
4420 '
4430   COLX% = PMX(J%) / 8 - 3         'find column for each mark along x-axis
4440   ROWY% = PMY(J%) / 8                'find row for each mark along y-axis
4450 '
4460   LOCATE ROWX%, COLX%
4470    IF XMAX > 999.99 OR XMIN < -999.99 THEN 4510
4480    IF XMAX < .1 AND XMIN > -.1 THEN 4510                'format selection
4490   PRINT USING "+###.##"; XMIN + XSCALE * J%      'print x-scale divisions
4500   GOTO 4530
4510   PRINT USING "+#.##^^^^"; XMIN + XSCALE * J%    'print x-scale divisions
4520 '
4530   LOCATE ROWY%, COLY%
4540    IF YMAX > 999.99 OR YMIN < -999.99 THEN 4580
4550    IF YMAX < .1 AND YMIN > -.1 THEN 4580                'format selection
4560   PRINT USING "+###.##"; YMAX - YSCALE * J%      'print y-scale divisions
4570   GOTO 4600
4580   PRINT USING "+#.##^^^^"; YMAX - YSCALE * J%    'print y-scale divisions
4590 '
4600   IF J% <> 2 THEN 4650              'print names of variables along x & y
4610    LOCATE ROWX% - 2, PXMAX% / 8 + 3: PRINT XTITLE$; "->"
4620    LOCATE ROWY% - 1, COLY% - 1: PRINT "^"
4630    LOCATE ROWY%, COLY% - 1: PRINT "|"
4640    LOCATE ROWY% + 1, COLY% - 1: PRINT YTITLE$
4650 NEXT J%
4660 '
4670 FOR I% = 0 TO NSTEP%
4680    PX%(I%) = KCOEFF1 * X(I%) + KCOEFF2     'perform scale transformations
4690    PY%(I%) = KCOEFF3 * Y(I%) + KCOEFF4                  'along x & y axes
4700    IF I% = 0 THEN 4720
4710    LINE (PX%(I% - 1), PY%(I% - 1))-(PX%(I%), PY%(I%))         'draw graph
4720 NEXT I%
4730 '
4740 RETURN
4750 '
10000 '***********************************************************************
10010 'subroutine to display header screen
10020 'input variables:  none
10030 'output variables: none
10040 'global variables: none
10050 'local variables:  none
10060 '***********************************************************************
10070 CLS : KEY OFF                                           'clear screen
10080 SCREEN 2, 0, 0: WIDTH 80                               'change screen
10090  '
10100 LINE (1, 1)-(638, 40), , B                                'draw boxes
10110 LINE (5, 3)-(633, 37), , B
10120 LINE (70, 50)-(570, 140), , B
10130 LINE (75, 53)-(565, 137), , B
10140                                           'print title, authors, etc.
10150 LOCATE 3, 7
10160 PRINT " C O M P U T A T I O N A L   P H Y S I C S :";
10170 PRINT " (An  Introduction)"
10180 LOCATE 9, 40
10190 PRINT "by";
10200 LOCATE 11, 20: PRINT "R.C. Verma, P.K. Ahluwalia & K.C. Sharma"
10210 LOCATE 13, 34
10220 PRINT "   * * * *    "
10230 '
10240 BEEP
10250 LOCATE 15, 35: PRINT "* * * * * *"
10260 LOCATE 22, 25: PRINT "press any key to continue"
10270 LOCATE 23, 25: PRINT "-------------------------"
10280 '
10290 IF INKEY$ = "" THEN 10290                            'hold the screen
10300 CLS                                                     'clear screen
10310 RETURN
10320 '
