10 '**************************************************************************
20 'Chapter 4, Example 1: Travelling Pulse
30 'COMPUTATIONAL PHYSICS: An Introduction
40 'by R. C. Verma, P. K. Ahluwalia and K. C. Sharma
50 '   * * * *    
60 '**************************************************************************
70  CLS : KEY OFF
80  GOSUB 10000                                                   'show header
90 '
100 DIM PMX%(5), PMY%(5)                                      'give dimensions
110 '                                                         'define pulse fn
120 DEF FNF (X, T) = AMP * SIN(2 * PI * (X - V * T) / LAM)
130 '
140 PI = 3.1415926#                                       'define constants pi
150 '
160 GOSUB 1000             'to introduce the problem & to get input parameters
170 '
180 GOSUB 5000                                       'draw axes and set scales
190 GOSUB 2000                              'find amplitude of pulse & draw it
200 '
210 LOCATE 22, 1: BEEP: PRINT "press any key to continue"
220 IF INKEY$ = "" THEN 220
230 '
240 CLS : LOCATE 12, 15: BEEP: PRINT "Program ends here!"
250 END
260 '
1000 '************************************************************************
1010 'subroutine to introduce the problem & to get input variables
1020 'input variables:  none
1030 'output variables: LAM, V, TI, TF, XI, XF, H, HT, YMIN, YMAX
1040 'global variables: none
1050 'local variables:  none
1060 'test cases: LAM =5, VEL =1, AMP = 10
1070 '            H=0.1, HT=0.2, XMIN=0, XMAX=10, TMIN=0, TMAX=10
1080 '************************************************************************
1090 '
1100 LOCATE 2, 10: PRINT "This program shows a Travelling Pulse"
1110 LOCATE 4, 10: PRINT "---------------------------------------------------"
1120 '
1130 LOCATE 9, 5: BEEP: PRINT "Please feed the input parameters"
1135 LOCATE 11, 1: PRINT SPACE$(80)
1140 LOCATE 11, 1                           'physical properties of the system
1150 INPUT "Give amplitude, wavelength and velocity of pulse"; AMP, LAM, V
1160   IF AMP > 0 AND LAM > 0 AND V > 0 THEN 1200
1170   BEEP
1175 PRINT "Please give suitable values for amplitude, wavelength & velocity!"
1180   GOTO 1135
1190                                                            'time interval
1200 LOCATE 13, 1: PRINT SPACE$(80)
1205 LOCATE 13, 1: INPUT "Give initial and final times, & Ti, Tf"; TMIN, TMAX
1210   IF TMAX > TMIN THEN 1250
1220   BEEP: PRINT "Please give suitable time interval Ti, Tf!"
1230   GOTO 1200
1240 '
1250 LOCATE 15, 1: PRINT SPACE$(80)
1255 LOCATE 15, 1: INPUT "Give time step: h "; HT
1260  IF HT > 0 THEN 1300
1270  BEEP
1280  PRINT "Please give suitable value of time step!"
1290  GOTO 1250
1300 LOCATE 17, 1: PRINT SPACE$(80)
1310 LOCATE 17, 1
1315 INPUT "Give initial and final X positions, Xi, Xf"; XI, XMAX
1320   IF XMAX > XI THEN 1350
1330   BEEP: PRINT "Please give suitable X-domain Xi, Xf!"
1340   GOTO 1300
1350 '
1360 YMAX = AMP: YMIN = -YMAX                         'max., min, for graphics
1370 '
1380 LOCATE 19, 1: PRINT SPACE$(80)
1385 LOCATE 19, 1: INPUT "Give position-step: hx "; H
1390  IF H > 0 THEN 1430
1400  BEEP: PRINT "Please give suitable value of time step!"
1410  PRINT
1420  GOTO 1380
1430 '
1440 RETURN
1450 '
2000 '************************************************************************
2010 'subroutine to calculate and draw pulse amplitude
2020 'input variables:  TMIN, TMAX, XI, XMAX, FNF(X,T), LAM, V, H, HT
2030 'output variables: X, Y
2040 'global variables: none
2050 'local variables:  none
2060 '************************************************************************
2070 '
2080 T = TMIN                                                 'initialize time
2090 '
2100 WHILE T <= TMAX                                           'loop over time
2110   XMIN = XI + V * T                     'initialize position on the graph
2120   X = XMIN
2130   Y = FNF(XMIN, T)
2140   GOSUB 3000                    'clean the box & get first point on graph
2150   WHILE X <= XMIN + LAM                            'loop for space points
2160     Y = FNF(X, T)                                        'pulse amplitude
2170     IF X >= XMIN + H AND X <= XMAX THEN GOSUB 4000            'draw pulse
2180     X = X + H                                      'increment space point
2190   WEND
2200   T = T + HT                                              'increment time
2210 WEND
2220 '
2230 RETURN
2240 '
2250 'local variables: PX%(I%), PY%(I%), PMX%(J%), PMY%(J%),
2260 '                 DIV%, XSCALE, YSCALE, PXSCALE, PYSCALE
2270 '                 ROWX%, COLX%, ROWY%, COLY%
3000 '************************************************************************
3010 'subroutine for clearing the box
3020 'input variables: XMIN, Y, T, PXMIN%, PXMAX%, PYMIN%, PYMAX%
3030 '                 KCOEFF1: KCOEFF2: KCOEFF3: KCOEFF4
3040 'output variable: X1%, Y1%
3050 'local variables: none
3060 'global variable: none
3070 '************************************************************************
3080 LINE (PXMIN% + 1, PYMIN% + 1)-(PXMAX% - 1, PYMAX% - 1), 0, BF
3090 LINE (PXMIN%, KCOEFF4)-(PXMAX%, KCOEFF4)                     'draw x-axis
3100  X1% = KCOEFF1 * XMIN + KCOEFF2            'perform scale transformations
3110  Y1% = KCOEFF3 * Y + KCOEFF4                            'along x & y axes
3120 LOCATE 22, 60: PRINT USING "t = ##.##"; T             'print current time
3130 '
3140 RETURN
3150 '
4000 '************************************************************************
4010 'subroutine to graph the pulse
4020 'input variables: X, Y, T, V, X1%, Y1%
4030 '                 KCOEFF1: KCOEFF2: KCOEFF3: KCOEFF4
4040 'Output variable: None
4050 'local variables: X2%, Y2%
4060 'Global variable: NI
4070 '************************************************************************
4080                                     'scale transformation for x- & y-axes
4090  X2% = KCOEFF1 * X + KCOEFF2               'perform scale transformations
4100  Y2% = KCOEFF3 * Y + KCOEFF4                            'along x & y axes
4110                                           'present position and amplitude
4120 IF X > XMIN THEN LINE (X1%, Y1%)-(X2%, Y2%)        'join with last point
4130 X1% = X2%: Y1% = Y2%                                     'roll the values
4140 '
4150 RETURN
4160 '
5000 '************************************************************************
5010 'subroutine to draw axes and set scales
5020 'input variables:  XMIN, XMAX, YMIN. YMAX
5030 'output variables: TITLE$, XTITLE$, YTITLE$,
5040 '                  KCOEFF1: KCOEFF2: KCOEFF3: KCOEFF4
5050 '                  PXMIN%, PXMAX%, PYMIN%, PYMAX%
5060 'global variables: none
5070 '************************************************************************
5080 CLS                                                         'clear screen
5090 DIV% = 4                                       'no. of marks on x, y axes
5100 PXMIN% = 136: PXMAX% = 456: PYMIN% = 24: PYMAX% = 144
5110                                                     'choose screen region
5120 TITLE$ = "*** Travelling Pulse ***"
5130 XTITLE$ = "x": YTITLE$ = "y"
5140 '
5150 KCOEFF1 = (PXMAX% - PXMIN%) / (XMAX - XMIN)
5160 KCOEFF2 = (PXMAX% * XMIN - PXMIN% * XMAX) / (XMIN - XMAX)
5170 KCOEFF3 = (PYMAX% - PYMIN%) / (YMIN - YMAX)
5180 KCOEFF4 = (PYMAX% * YMAX - PYMIN% * YMIN) / (YMAX - YMIN)
5190 '
5200 ROWTIT% = PYMIN% / 8 - 1: COLTIT% = PXMIN% / 8 + 2
5210 LOCATE ROWTIT%, COLTIT%: PRINT TITLE$                  'print graph-title
5220 '
5230 LINE (PXMIN%, PYMIN%)-(PXMAX%, PYMAX%), , B           'draw box for graph
5240 LINE (PXMIN%, KCOEFF4)-(PXMAX%, KCOEFF4)                     'draw x-axis
5250 '
5260 XSCALE = (XMAX - XMIN) / DIV%                       'find scale for marks
5270 YSCALE = (YMAX - YMIN) / DIV%
5280 '
5290 PXSCALE = (PXMAX% - PXMIN%) / DIV%            'find pixel scale for marks
5300 PYSCALE = (PYMAX% - PYMIN%) / DIV%
5310 '
5320 ROWX% = PYMAX% / 8 + 2              'find row for marks along x-direction
5330 COLY% = PXMIN% / 8 - 10          'find column for marks along y-direction
5340 '
5350 FOR J% = 0 TO DIV%                          'start loop to mark divisions
5360   PMX(J%) = PXMIN% + PXSCALE * J%                    'find places to mark
5370   PMY(J%) = PYMIN% + PYSCALE * J%
5380    LINE (PMX(J%), PYMAX%)-(PMX(J%), PYMAX% - 5)               'draw marks
5390    LINE (PXMIN%, PMY(J%))-(PXMIN% + 5, PMY(J%))
5400 '
5410   IF J% / 2 <> INT(J% / 2) THEN 5650            'jump for alternate marks
5420 '
5430   COLX% = PMX(J%) / 8 - 3         'find column for each mark along x-axis
5440   ROWY% = PMY(J%) / 8 + 1            'find row for each mark along y-axis
5450 '
5460   LOCATE ROWX%, COLX%
5470    IF XMAX > 999.99 OR XMIN < -999.99 THEN 5510
5480    IF XMAX < .1 AND XMIN > -.1 THEN 5510                'format selection
5490   PRINT USING "+###.##"; XMIN + XSCALE * J%      'print x-scale divisions
5500   GOTO 5530
5510   PRINT USING "+#.##^^^^"; XMIN + XSCALE * J%    'print x-scale divisions
5520 '
5530   LOCATE ROWY%, COLY%
5540    IF YMAX > 999.99 OR YMIN < -999.99 THEN 5580
5550    IF YMAX < .1 AND YMIN > -.1 THEN 5580                'format selection
5560   PRINT USING "+###.##"; YMAX - YSCALE * J%      'print y-scale divisions
5570   GOTO 5600
5580   PRINT USING "+#.##^^^^"; YMAX - YSCALE * J%    'print y-scale divisions
5590 '
5600   IF J% <> 2 THEN 5650              'print names of variables along x & y
5610    LOCATE ROWX%, PXMAX% / 8 + 5: PRINT XTITLE$; " ->"
5620    LOCATE ROWY% - 1, COLY% - 1: PRINT "^"
5630    LOCATE ROWY%, COLY% - 1: PRINT "|"
5640    LOCATE ROWY% + 1, COLY% - 1: PRINT YTITLE$
5650 NEXT J%
5660 '
5670 RETURN
5680 '
10000 '***********************************************************************
10010 'subroutine to display header screen
10020 'input variables:  none
10030 'output variables: none
10040 'global variables: none
10050 'local variables:  none
10060 '***********************************************************************
10070 CLS : KEY OFF                                           'clear screen
10080 SCREEN 2, 0, 0: WIDTH 80                               'change screen
10090  '
10100 LINE (1, 1)-(638, 40), , B                                'draw boxes
10110 LINE (5, 3)-(633, 37), , B
10120 LINE (70, 50)-(570, 140), , B
10130 LINE (75, 53)-(565, 137), , B
10140                                           'print title, authors, etc.
10150 LOCATE 3, 7
10160 PRINT " C O M P U T A T I O N A L   P H Y S I C S :";
10170 PRINT " (An  Introduction)"
10180 LOCATE 9, 40
10190 PRINT "by";
10200 LOCATE 11, 20: PRINT "R.C. Verma, P.K. Ahluwalia & K.C. Sharma"
10210 LOCATE 13, 34
10220 PRINT "   * * * *    "
10230 '
10240 BEEP
10250 LOCATE 15, 35: PRINT "* * * * * *"
10260 LOCATE 22, 25: PRINT "press any key to continue"
10270 LOCATE 23, 25: PRINT "-------------------------"
10280 '
10290 IF INKEY$ = "" THEN 10290                            'hold the screen
10300 CLS                                                     'clear screen
10310 RETURN
10320 '
