10 '**************************************************************************
20 'Chapter 5, Example 1: Young's Double Slit Interference Experiment
30 'COMPUTATIONAL PHYSICS: An Introduction
40 'by R. C. Verma, P. K. Ahluwalia and K. C. Sharma
50 '   * * * *    
60 '**************************************************************************
70  CLS : KEY OFF
80  GOSUB 10000                                                   'show header
90 '
100 DIM PMX%(5), PMY%(5)                                       'set dimensions
110 '
120 DEF FNF (X, Y, T) = AMPD * ((COS(KVEC * X - WFRQ * T)) / X + (COS(KVEC * Y - WFRQ * T)) / Y)
130                                        'define function for wave amplitude
140 PI = 3.1415926#                                        'define constant pi
150 '
160 GOSUB 1000                   'introduce the problem & get input parameters
170 GOSUB 3000                                       'draw axes and set scales
180 GOSUB 2000                                        'find intensity of light
190 '
200 LOCATE 22, 1
210 IF INKEY$ = "" THEN 210                                   'hold the screen
220 '
230 CLS : LOCATE 12, 15: BEEP: PRINT "Program ends here!"
240 END
250 '
1000 '************************************************************************
1010 'subroutine to introduce the problem & to get input variables
1020 'input variables:  PI
1030 'output variables: X, X1, X2, Y1, Y2, H, NPTS%, NTIME%, AMPD, KVEC, WFRQ
1040 '                  XMAX. XMIN, YMIN, YMAX, ZMIN, ZMAX
1050 'global variables: none
1060 'local variables:  AMPSUM, I%, CL, SL, LAM, DIS
1070 'test cases:  given with INPUT statements
1080 '************************************************************************
1090 '
1100 LOCATE 2, 10: PRINT "This program demonstrate Interference of Light"
1110 LOCATE 3, 15: PRINT "through Young's double slit experiment"
1120 LOCATE 4, 10: PRINT "---------------------------------------------------"
1130 BEEP
1140 LOCATE 6, 5: PRINT "Please feed the following parameters."
1150 '
1160                                        'physical properties of the system
1170 LOCATE 8,1: INPUT "Wavelength of light (500 angstrom) = ";LAM  
1180  IF LAM >0 THEN 1200 
1190  BEEP: LOCATE 8,1: PRINT SPACE$(78): GOTO 1170
1200 LOCATE 10,1: INPUT "Amplitude  of light (unit = 1) = ";AMPD     
1210  IF AMPD>0 THEN 1230 
1220  BEEP: LOCATE 10,1: PRINT SPACE$(78): GOTO 1200
1230 LOCATE 12,1:  INPUT "Screen length (20 mm) ";YMAX                
1240  IF YMAX>0 THEN 1260 
1250  BEEP: LOCATE 12,1: PRINT SPACE$(78): GOTO 1230
1260 LOCATE 14,1:  INPUT "Distance from slit & screen (200 mm) = ";DIS
1270  IF DIS >0 THEN 1290 
1280  BEEP: LOCATE 14,1: PRINT SPACE$(78): GOTO 1260
1290 LOCATE 16,1:  INPUT "Slit spacing (0.01 mm) ";SL                 
1300  IF SL >0 THEN 1320 
1310  BEEP: LOCATE 16,1: PRINT SPACE$(78): GOTO 1290
1320 LOCATE 18,1:  INPUT "No. of points on the screen (32) ";NPTS%    
1330  IF NPTS%>0 THEN 1350 
1340  BEEP: LOCATE 18,1: PRINT SPACE$(78): GOTO 1320
1350 LOCATE 20,1:  INPUT "Time step for one cycle (8) ";NTIME%   
1360  IF NTIME% >0 THEN 1380 
1370  BEEP: LOCATE 20,1: PRINT SPACE$(78): GOTO 1350
1380                                                             'set graphics
1390 ZMAX=AMPD*2/DIS : ZMAX=ZMAX*ZMAX : ZMIN=0 :YMIN = 0
1400 H= YMAX/NPTS%                          'find step size to scan the screen
1410 '
1420 LAM=LAM*.0000001 : CL = 30               'speed of light (x 10 ^ 10 mm/s)
1430 KVEC=2*PI/LAM : WFRQ=CL*LAM                 'find wave vector & frequency
1440 '
1450 X1=0 : Y1=.5*(YMAX-SL) : X2=X1: Y2=.5*(YMAX+SL)    'fix location of slits
1460 X=DIS+X1
1470 '
1480 GOSUB 3000                                      'draw two-slit experiment
1490 '
1500 RETURN
1510 '
2000 '************************************************************************
2010 'subroutine to calculate and draw pulse amplitude
2020 'input variables:  X, X1, X2, Y1, Y2, H, NPTS%, NTIME%
2030 '                  YMIN, FNF(X, Y, T), AMPD, KVEC, WFRQ    
2040 'output variables: Y, INTS
2050 'global variables: none
2060 'local variables:  I%, T%, R1, R2, AMP 
2070 '************************************************************************
2080 '
2090 FOR I% = 0 TO NPTS%                              'loop over screen points
2100   Y=YMIN+I%*H                                        'scanning the screen
2110   R1= (X-X1)*(X-X1)+(Y-Y1)*(Y-Y1)              'find distance from hole 1
2120   R2= (X-X2)*(X-X2)+(Y-Y2)*(Y-Y2)              'find distance from hole 2
2130   R1=SQR(R1) : R2=SQR(R2)
2140   INTS=0                                            'initialize intensity
2150   FOR T% = 1 TO NTIME%                             'loop over time period
2160     AMP= FNF(R1,R2,T%)                          'find resultant amplitude
2170     INTS = INTS+AMP*AMP                                 'update intensity
2180   NEXT T%
2190   INTS=INTS/NTIME%                                     'average intensity
2200   GOSUB 4000                                              'draw intensity
2210 NEXT I%
2220 GOSUB 4220                                                     'clear box
2230 IF INKEY$="" THEN 2230                                   'hold the screen
2240 '
2250 RETURN
2260 '
3000 '************************************************************************
3010 'subroutine to draw axes of the graph
3020 'input variables: XMAX. XMIN, YMIN, YMAX, ZMIN, ZMAX
3030 'output variable: YSCALE, ZSCALE, YS1%, YS2%,
3040 'local variables: SW
3050 'global variable: none
3060 '************************************************************************
3070 '
3080 YSCALE=160/(YMAX-YMIN)                                    'set x,y scales
3090 ZSCALE=200/(ZMAX-ZMIN)
3100 CLS                                                         'clear screen
3110 '
3120 YS1%= 93 : YS2%= 118 : SW=2                            'set slit settings
3130 '
3140 LOCATE 1,10:PRINT "Young's Double Slit Interference Experiment"
3150 '
3160 LINE (40,24)- (600,186), ,B             'draw main box & slit arrangement
3170 LINE (400,24) - (400,186)
3180 LINE (100,24)-(100,YS1%-SW)
3190 LINE (100,YS1%+SW)-(100,YS2%-SW)
3200 LINE (100,YS2%+SW)-(100,186)
3210 LINE (98,YS1%-SW)-(102,YS1%-SW)
3220 LINE (98,YS1%+SW)-(102,YS1%+SW)
3230 LINE (98,YS2%-SW)-(102,YS2%-SW)
3240 LINE (98,YS2%+SW)-(102,YS2%+SW)
3250 LINE (100,.5*(YS1%+YS2%))-(400,.5*(YS1%+YS2%))
3260 '
3270 LOCATE 14,6:PRINT "*"                           'show the source of light
3280 '
3290 RETURN
3300 '
4000 '************************************************************************
4010 'subroutine to draw light ray trajectory
4020 'input variables: Y, YS1%, YS2%, YSCALE, ZSCALE, YMAX, ZMIN, INTS
4030 'output variable: none
4040 'local variables: X1%, X2%, Y2%, Z%
4050 'global variable: none
4060 '************************************************************************
4070 '                                    
4080 X2%=400 : X1%=100 :Z% = 450                  'fix pixel for x-coordinates
4090 '
4100 Y2%=(YMAX-Y)*YSCALE+26              'scale transformation for x- & y-axes
4110 Z%=(INTS-ZMIN)*ZSCALE+400
4120 '
4130 LINE (101,25)-(399,.5*(YS1%+YS2%)-1), 0, BF                  'clear boxes
4140 LINE (101,.5*(YS1%+YS2%)+1)-(399,185), 0, BF
4150 '
4160 LINE (X1%,YS1%)-(X2%,Y2%)                          'join with last points
4170 LINE (X1%,YS2%)-(X2%,Y2%)                         
4180 LINE (X2%,Y2%) - (Z%,Y2%)
4190 '
4200 RETURN
4210 '
4220 '************************************************************************
4230 'subroutine to clear boxes
4240 'input variables: YS1%, YS2%
4250 'output variable: none
4260 'local variables: none
4270 'global variable: none
4280 '************************************************************************
4290                                                             'clear boxes
4300 LINE (101,25)-(399,.5*(YS1%+YS2%)-1), 0, BF
4310 LINE (101,.5*(YS1%+YS2%)+1)-(399,183), 0, BF
4320 '
4330 RETURN
4340 '
10000 '***********************************************************************
10010 'subroutine to display header screen
10020 'input variables:  none
10030 'output variables: none
10040 'global variables: none
10050 'local variables:  none
10060 '***********************************************************************
10070 CLS : KEY OFF                                           'clear screen
10080 SCREEN 2, 0, 0: WIDTH 80                               'change screen
10090  '
10100 LINE (1, 1)-(638, 40), , B                                'draw boxes
10110 LINE (5, 3)-(633, 37), , B
10120 LINE (70, 50)-(570, 140), , B
10130 LINE (75, 53)-(565, 137), , B
10140                                           'print title, authors, etc.
10150 LOCATE 3, 7
10160 PRINT " C O M P U T A T I O N A L   P H Y S I C S :";
10170 PRINT " (An  Introduction)"
10180 LOCATE 9, 40
10190 PRINT "by";
10200 LOCATE 11, 20: PRINT "R.C. Verma, P.K. Ahluwalia & K.C. Sharma"
10210 LOCATE 13, 34
10220 PRINT "   * * * *    "
10230 '
10240 BEEP
10250 LOCATE 15, 35: PRINT "* * * * * *"
10260 LOCATE 22, 25: PRINT "press any key to continue"
10270 LOCATE 23, 25: PRINT "-------------------------"
10280 '
10290 IF INKEY$ = "" THEN 10290                            'hold the screen
10300 CLS                                                     'clear screen
10310 '
10320 RETURN
10330 '
