10 '**************************************************************************
20 'Chapter 5, Example 2: Diffraction due to Diffraction Grating
30 'COMPUTATIONAL PHYSICS: An Introduction
40 'by R. C. Verma, P. K. Ahluwalia and K. C. Sharma
50 '   * * * *   
60 '**************************************************************************
70  CLS : KEY OFF
80  GOSUB 10000                                                   'show header
90 '        
100 DEF FNF (X, T) = AMPD * COS(KVEC * X - WFRQ * T) / X'define amplitude function
110 '
120 DIM XS(100), YS(100), YGS%(100)     'set dimensions of the array variables
130 PI = 3.1415926#                                        'define constant pi
140 '
150 GOSUB 1000                   'introduce the problem & get input parameters
160 GOSUB 3000                              'draw diffraction slit arrangement
170 GOSUB 2000                                          'find & draw intensity
180 '
190 LOCATE 22, 1
200 IF INKEY$ = "" THEN 200                                   'hold the screen
210 '
220 CLS : LOCATE 12, 15: BEEP: PRINT " Thank you !"
230 '
240 END
250 '
1000 '************************************************************************
1010 'subroutine to introduce the problem & to get input variables
1020 'input variables:  PI
1030 'output variables: DIS, NPOINTS%, NTIME%, NSLIT%, XS(J%), YS(J%),
1040 '                   H, YMIN, YMAX, ZMIN, ZMAX, AMPD, KVEC, WFRQ
1050 'global variables: none
1060 'local variables:  LAM, CL, SZ, SL, BL
1070 'test cases: given in the INPUT statements
1080 '************************************************************************
1090 '
1100 LOCATE 1, 10: PRINT "This program shows the diffraction phenomenon"
1110 LOCATE 2, 15: PRINT " for N-slit experiment"
1120 LOCATE 3, 10: PRINT "-------------------------------------------------"
1130 BEEP
1140 LOCATE 4, 5: PRINT "Please give the following parameters:"
1150 '
1160 LOCATE 6, 5:  INPUT "Wavelength of light (500 angstrom) = "; LAM
1170  IF LAM > 0 THEN 1190
1180  BEEP: LOCATE 6, 5: PRINT SPACE$(75): GOTO 1160
1190 LOCATE 8, 5:  INPUT "Amplitude  of light (unit = 1) = "; AMPD
1200  IF AMPD > 0 THEN 1220
1210  BEEP: LOCATE 8, 5: PRINT SPACE$(75): GOTO 1190
1220 LOCATE 10, 5: INPUT "Screen length (10 mm) "; YMAX
1230  IF YMAX > 0 THEN 1250
1240  BEEP: LOCATE 10, 5: PRINT SPACE$(75): GOTO 1220
1250 LOCATE 12, 5: INPUT "Distance from slit & screen (200 mm) = "; DIS
1260  IF DIS > 0 THEN 1280
1270  BEEP: LOCATE 12, 5: PRINT SPACE$(75): GOTO 1250
1280 LOCATE 14, 5: INPUT "Slit size   (0.01 mm) "; SZ
1290  IF SZ > 0 THEN 1310
1300  BEEP: LOCATE 14, 5: PRINT SPACE$(75): GOTO 1280
1310 LOCATE 16, 5: INPUT "No. of slits (10) "; NSLIT%
1320  IF NSLIT% > 0 THEN 1340
1330  BEEP: LOCATE 16, 5: PRINT SPACE$(75): GOTO 1310
1340 LOCATE 18, 5: INPUT "No. of points on the screen (32)"; NPOINTS%
1350  IF NPOINTS% > 0 THEN 1370
1360  BEEP: LOCATE 18, 5: PRINT SPACE$(75): GOTO 1340
1370 LOCATE 20, 5: INPUT "Time step for one cycle (8)"; NTIME%
1380  IF NTIME% > 0 THEN 1400
1390  BEEP: LOCATE 20, 5: PRINT SPACE$(75): GOTO 1370
1400                                                        'find slit spacing
1410 SL = SZ / (NSLIT% - 1): BL = (YMAX - (NSLIT% + 1) * SL) * .5
1420 '
1430 FOR J% = 1 TO NSLIT%
1440   YS(J%) = J% * SL + BL: XS(J%) = 0                  'find grating points
1450 NEXT J%
1460                                                   'set range for graphics
1470 ZMAX = AMPD * NSLIT% / DIS: ZMAX = ZMAX * ZMAX: ZMIN = 0: YMIN = 0
1480 H = YMAX / NPOINTS%                   'find step size for scanning screen
1490 '
1500 LAM = LAM * .0000001: CL = 30        'fix speed of light (x 10 ^ 10 mm/s)
1510 KVEC = 2 * PI / LAM: WFRQ = CL * LAM        'find wave vector & frequency
1520 '
1530 RETURN
1540 '
2000 '************************************************************************
2010 'subroutine to find & draw intensity at different points on the screen
2020 'input variables: DIS, NPOINTS%, NTIME%, NSLIT%, XS(J%), YS(J%),
2030 '                 H, YMIN, YMAX, ZMIN, ZMAX, AMPD, KVEC, WFRQ
2040 'output variables: Y, INTS
2050 'global variables:
2060 'local variables:  I%, T%, J%, X, R, SUMA, AMP1,
2070 '************************************************************************
2080 '
2090 X = DIS                           'take distance between grating & screen
2100 '
2110 FOR I% = 0 TO NPOINTS%                        'loop over points on screen
2120   Y = YMIN + I% * H                                      'scan the screen
2130   INTS = 0                                          'initialize intensity
2140 '
2150   GOSUB 6000                                                   'clear box
2160   GOSUB 4000                               'draw light ray from each slit
2170 '
2180   FOR T% = 1 TO NTIME%                             'loop over time period
2190     SUMA = 0                                  'initialize total amplitude
2200     FOR J% = 1 TO NSLIT%                         'loop over grating slits
2210       R = (X - XS(J%)) * (X - XS(J%)) + (Y - YS(J%)) * (Y - YS(J%))
2220       R = SQR(R)                       'find distance from slit to screen
2230       AMP1 = FNF(R, T)                               'find wave amplitude
2240       SUMA = SUMA + AMP1                          'update total amplitude
2250     NEXT J%
2260     INTS = INTS + SUMA * SUMA                           'update intensity
2270   NEXT T%
2280   INTS = INTS / NTIME%                            'find average intensity
2290 '
2300   GOSUB 5000                                    'draw intensity histogram
2310 '
2320 NEXT I%
2330 '
2340 GOSUB 6000                                                     'clear box
2350 '
2360 RETURN
2370 '
3000 '************************************************************************
3010 'subroutine to draw axes of the graph
3020 'input variables: YMIN,YMAX, ZMIN, ZMAX, NSLIT%,
3030 'output variable: YSCALE, ZSCALE, YGS%(J%)
3040 'local variables: GA, GB
3050 'global variable: none
3060 '************************************************************************
3070 CLS                                                         'clear screen
3080 YSCALE = 160 / (YMAX - YMIN)                                 'find scales
3090 ZSCALE = 200 / (ZMAX - ZMIN)
3100 CLS
3110 '
3120 GA = -60 / (NSLIT% - 1): GB = 134 - GA      'find coordinates for grating
3130 '
3140 LOCATE 1, 10: PRINT "Diffraction pattern with Grating"
3150 '
3160 LINE (40, 24)-(600, 186), , B                   'draw grating arrangement
3170 LINE (400, 24)-(400, 184)
3180 LINE (100, 24)-(100, 184)
3190 LINE (98, 74)-(102, 74)
3200 LINE (98, 134)-(102, 134)
3210 LINE (100, 104)-(400, 104)
3220 '
3230 FOR J% = 1 TO NSLIT%
3240  YGS%(J%) = GA * J% + GB
3250  PRESET (100, YGS%(J%))                               'draw grating slits
3260 NEXT J%
3270 '
3280 LOCATE 14, 6: PRINT "*"                             'show source of light
3290 '
3300 RETURN
3310 '
4000 '************************************************************************
4010 'subroutine to draw rays of light
4020 'input variables: YMAX, Y, YSCALE, NSLIT%, YGS%(J%)
4030 'output variable: none
4040 'local variables: X1%, X2%, Y2%, J%
4050 'global variable: none
4060 '************************************************************************
4070                          'perform scale transformation for x- & y-axes
4080 X1% = 100: X2% = 400
4090 Y2% = (YMAX - Y) * YSCALE + 26
4100  '
4110 FOR J% = 1 TO NSLIT%
4120  LINE (40, 104)-(X1%, YGS%(J%))              'start light ray from source
4130  LINE (X1%, YGS%(J%))-(X2%, Y2%)                    'join with last point
4140 NEXT J%
4150  '
4160 RETURN
4170 '
5000 '************************************************************************
5010 'subroutine to draw intensity histogram
5020 'input variables: YMAX, Y, H, ZMIN, YSCALE, ZSCALE, INTS,
5030 'output variable: none
5040 'local variables: X2%, YI2%, YF2%, Z%
5050 'global variable: none
5060 '************************************************************************
5070                                            'perform scale transformations
5080 X2% = 400
5090 YI2% = (YMAX - Y - .5 * H) * YSCALE + 26
5100 YF2% = (YMAX - Y + .5 * H) * YSCALE + 26
5110 Z% = (INTS - ZMIN) * ZSCALE + 400
5120 '
5130 LINE (X2%, YI2%)-(Z%, YF2%), 1, BF                        'draw solid box
5140 '
5150 RETURN
5160 '
6000 '************************************************************************
6010 'subroutine to clear boxes
6020 'input variables: none
6030 'output variable: none
6040 'local variables: none
6050 'global variable: none
6060 '************************************************************************
6070 '
6080 LINE (41, 25)-(99, 183), 0, BF
6090 LINE (101, 25)-(399, 103), 0, BF
6100 LINE (101, 105)-(399, 183), 0, BF
6110 '
6120 RETURN
6130 '
10000 '***********************************************************************
10010 'subroutine to display header screen
10020 'input variables:  none
10030 'output variables: none
10040 'global variables: none
10050 'local variables:  none
10060 '********************************************************************
10070 CLS : KEY OFF                                           'clear screen
10080 SCREEN 2, 0, 0: WIDTH 80                               'change screen
10090  '
10100 LINE (1, 1)-(638, 40), , B                                'draw boxes
10110 LINE (5, 3)-(633, 37), , B
10120 LINE (70, 50)-(570, 140), , B
10130 LINE (75, 53)-(565, 137), , B
10140                                           'print title, authors, etc.
10150 LOCATE 3, 7
10160 PRINT " C O M P U T A T I O N A L   P H Y S I C S :";
10170 PRINT " (An  Introduction)"
10180 LOCATE 9, 40
10190 PRINT "by";
10200 LOCATE 11, 20: PRINT "R.C. Verma, P.K. Ahluwalia & K.C. Sharma"
10210 LOCATE 13, 34
10220 PRINT "   * * * *    "
10230 '
10240 BEEP
10250 LOCATE 15, 35: PRINT "* * * * * *"
10260 LOCATE 22, 25: PRINT "press any key to continue"
10270 LOCATE 23, 25: PRINT "-------------------------"
10280 '
10290 IF INKEY$ = "" THEN 10290                            'hold the screen
10300 CLS                                                     'clear screen
10310 RETURN
10320 '

