10 '**************************************************************************
20 'Chapter 5, Example 3: Polarization of Electromagnetic Wave
30 'COMPUTATIONAL PHYSICS: An Introduction
40 'by R. C. Verma, P. K. Ahluwalia and K. C. Sharma
50 '   * * * *   
60 '**************************************************************************
70  CLS : KEY OFF
80  GOSUB 10000                                                   'show header
90 '
100 PI = 3.1415926#                                        'define constant pi
110                                        'define amplitude function of light
120 DEF FNF (A, X, T, P) = A * SIN(KV * X - WF * T - PI * P / 180)
130 '
140 GOSUB 1000                  'introduce the problem &  get input parameters
150 GOSUB 3000                                             'draw axes of graph
160 GOSUB 2000                                  'find and draw Electric vector
170 '
180 LOCATE 22, 1
190 IF INKEY$ = "" THEN 190                                       'hold screen
200 '
210 CLS : LOCATE 12, 15: BEEP: PRINT " Thank you !"
220 END
230 '
1000 '************************************************************************
1010 'subroutine to introduce the problem & to get input variables
1020 'input variables:  none
1030 'output variables:EXI, EYI, PHX, PHY, H, KV,WF
1040 '                 EXMIN, EXMAX, EYMIN, EYMAX, TMAX
1050 'global variables: none
1060 'local variables: MAX
1070 'test cases:
1080 '************************************************************************
1090 '
1100 LOCATE 1, 10: PRINT "This program shows motion of"
1110 LOCATE 2, 15: PRINT "polarization vector of light."
1120 LOCATE 3, 10: PRINT "----------------------------------------------"
1130 LOCATE 5, 10: PRINT "Please give the following quantities:"
1140 '
1150 LOCATE 7, 5: INPUT "Wave vector & frequency of the light = "; KV, WF
1160  IF KV > 0 AND WF > 0 THEN 1180
1170  BEEP:  LOCATE 7, 5: PRINT SPACE$(72): GOTO 1150
1180 LOCATE 9, 5: INPUT "X, Y components of its Electric Field = "; EXI, EYI
1190  IF EXI > 0 AND EYI > 0 THEN 1210
1200  BEEP:  LOCATE 9, 5: PRINT SPACE$(72): GOTO 1180
1210 '
1220 LOCATE 11, 5: INPUT "Phases of x,y components = "; PHX, PHY
1230 '
1240 LOCATE 13, 5: INPUT "Maximum time = "; TMAX
1250  IF TMAX > 0 THEN 1270
1260  BEEP:  LOCATE 13, 5: PRINT SPACE$(72): GOTO 1240
1270 '
1280 H = .1                                                   'time step fixed
1290 '
1300 IF EXI >= EYI THEN MAX = EXI ELSE MAX = EYI      'set limits for graphics
1310  EXMAX = MAX: EXMIN = -EXMAX
1320  EYMAX = MAX: EYMIN = -EYMAX
1330 '
1340 RETURN
1350 '
2000 '************************************************************************
2010 'subroutine to find and draw E-components
2020 'input variables: TMAX, PHX, PHY, EXI, EYI, H, FNF()
2030 'output variables: EX, EY, T
2040 'global variables: none
2050 'local variables:  X
2060 '************************************************************************
2070 '
2080 X = 0: T = 0                        'initialize time and fix x coordinate
2090 '
2100 WHILE T < TMAX                                            'loop over time
2110    GOSUB 4070                                                  'clear box
2120   EX = FNF(EXI, X, T, PHX)              'find x component of Field vector
2130   EY = FNF(EYI, X, T, PHY)                     'find y component of field
2140    GOSUB 5000                                          'draw field vector
2150   T = T + H                                               'increment time
2160 WEND
2170 '
2180 LOCATE 22, 1
2190 '
2200 RETURN
2210 '
3000 '************************************************************************
3010 'subroutine to draw axes of the graph
3020 'input variables: EYMIN, EYMAX
3030 'output variable: XSCALE, YSCALE, XO%, YO%, XL%, YL%, XM%, YM%
3040 'local variables: J%, K%, XPT, YPT, XD%, YD%
3050 'global variable: none
3060 '************************************************************************
3070 CLS
3080 YSCALE = 128 / (EYMAX - EYMIN)                                'find scale
3090 XSCALE = YSCALE * 2.5                           'multiply by aspect ratio
3100 '
3110 XL% = 64                             'fix pixel coordinates of one corner
3120 YL% = 16
3130 YM% = (EYMAX - EYMIN) * YSCALE + YL%   'find other coordinates of the box
3140 XM% = (EXMAX - EXMIN) * XSCALE + XL%
3150                                                              'print title
3160 LOCATE 1, 10: PRINT "Polarization: Electric Field Vector Orientation"
3170 '
3180  LINE (XL%, YL%)-(XM%, YM%), , B                           'draw main box
3190   XO% = .5 * (XL% + XM%): YO% = .5 * (YL% + YM%)             'find origin
3200  LINE (XL%, YO%)-(XM%, YO%)                                  'draw x-axis
3210  LINE (XO%, YL%)-(XO%, YM%)                                  'draw y-axis
3220 '
3230   YD% = (YM% - YL%) / 4                    'find step size to mark y-axis
3240 FOR J% = 0 TO 4
3250   XD% = (XM% - XL%) / 4                    'find step size to mark x-axis
3260   YPT = J% * YD% + YL%
3270   LINE (XL% - 4, YPT)-(XL%, YPT)                             'mark y-axis
3280   K% = YPT / 8 + 1                                    '8 for pixels ratio
3290   LOCATE K%, 1
3295   PRINT USING "+#.##"; (4 - J%) * (EYMAX - EYMIN) / 4 + EYMIN
3300 NEXT J%
3310 '
3320 LOCATE YO% / 8 - 1, 1: PRINT "^"
3330 LOCATE YO% / 8, 1: PRINT "|"                                'label y-axis
3340 LOCATE YO% / 8 + 1, 1: PRINT "Ey"
3350 '
3360 FOR J% = 0 TO 4
3370   XPT = J% * XD% + XL%
3380   LINE (XPT, YM%)-(XPT, YM% + 4)                             'mark x-axis
3390   K% = XPT / 8
3400   LOCATE 20, K%: PRINT USING "##.##"; J% * (EXMAX - EXMIN) / 4 + EXMIN
3410 NEXT J%
3420 '
3430 LOCATE YM% / 8 + 3, XO% / 8: PRINT "Ex -> "                 'label x-axis
3440 '
3450 RETURN
3460 '
4000 '************************************************************************
4010 'subroutine to clear 4 quadrants of main box
4020 'input variables:  XL%, YL%, XO%, YO%, XM%, YM%
4030 'output variables: none
4040 'global variables: none
4050 'local variables: none
4060 '************************************************************************
4070 '
4080 LINE (XL% + 1, YL% + 1)-(XO% - 1, YO% - 1), 0, BF
4090 LINE (XL% + 1, YO% + 1)-(XO% - 1, YM% - 1), 0, BF
4100 LINE (XO% + 1, YL% + 1)-(XM% - 1, YO% - 1), 0, BF
4110 LINE (XO% + 1, YO% + 1)-(XM% - 1, YM% - 1), 0, BF
4120 '
4130 RETURN
4140 '
5000 '************************************************************************
5010 'subroutine to draw field vector
5020 'input variables:T, EX,EY, EXMIN,EYMAX, XSCALE,YSCALE, XL%,YL%, XO%,YO%
5030 'output variable: None
5040 'local variables: X2%, Y2%
5050 'global variable:
5060 '************************************************************************
5070                                   'scale transformation for x- & y-axes
5080 X2% = (EX - EXMIN) * XSCALE + XL%
5090 Y2% = (EYMAX - EY) * YSCALE + YL%
5100 '
5110 LOCATE 3, 70: PRINT USING "t = ##.##"; T              'print current time
5120 LINE (XO%, YO%)-(X2%, Y2%)                             'draw field vector
5130 '
5140 RETURN
5150 '
10000 '***********************************************************************
10010 'subroutine to display header screen
10020 'input variables:  none
10030 'output variables: none
10040 'global variables: none
10050 'local variables:  none
10060 '***********************************************************************
10070 CLS : KEY OFF                                              'clear screen
10080 SCREEN 2, 0, 0: WIDTH 80                                  'change screen
10090  '
10100 LINE (1, 1)-(638, 40), , B                                   'draw boxes
10110 LINE (5, 3)-(633, 37), , B
10120 LINE (70, 50)-(570, 140), , B
10130 LINE (75, 53)-(565, 137), , B
10140                                              'print title, authors, etc.
10150 LOCATE 3, 7
10160 PRINT " C O M P U T A T I O N A L   P H Y S I C S :";
10170 PRINT " (An  Introduction)"
10180 LOCATE 9, 40
10190 PRINT "by";
10200 LOCATE 11, 20: PRINT "R.C. Verma, P.K. Ahluwalia & K.C. Sharma"
10210 LOCATE 13, 34
10220 PRINT "   * * * *    "
10230 '
10240 BEEP
10250 LOCATE 15, 35: PRINT "* * * * * *"
10260 LOCATE 22, 25: PRINT "press any key to continue"
10270 LOCATE 23, 25: PRINT "-------------------------"
10280 '
10290 IF INKEY$ = "" THEN 10290                               'hold the screen
10300 CLS                                                        'clear screen
10310 RETURN
10320 '

