10 '**************************************************************************
20 'Chapter 5, Example 4: Birefriegence & Polarization of Electromagnetic Wave
30 'COMPUTATIONAL PHYSICS: An Introduction
40 'by R. C. Verma, P. K. Ahluwalia and K. C. Sharma
50 '   * * * *   
60 '**************************************************************************
70  CLS : KEY OFF
80  GOSUB 10000                                                   'show header
90 '
100 PI = 3.1415926#                                        'define constant pi
110                                 'define function for electric field vector
120 DEF FNF (A, X, T, P) = A * SIN(KV * X - WF * T - P * PI / 180)
130 '
140 GOSUB 1000                   'introduce the problem & get input parameters
150 GOSUB 3000                                                      'draw axes
160 GOSUB 2000                                              'find E-components
170 '
180 BEEP: PRINT "press any key to continue"
190 IF INKEY$ = "" THEN 190
200 CLS : PRINT " Thank you !"
210 '
220 END
230 '
1000 '************************************************************************
1010 'subroutine to introduce the problem & to get input variables
1020 'input variables:  none
1030 'output variables: KV, WF, EXI, EYI, PHX, PHY, WID, NX, NY, H
1040 '                  EXMIN, EXMAX, EYMIN, EYMAX, TMAX
1050 'global variables: none
1060 'local variables:  MAX
1070 'test cases: K=1, W=1, PHX=0, PHY=90, WID=2, TMAX=10
1080 '************************************************************************
1090 '
1100 LOCATE 1, 5: PRINT "This programs demonstrate "
1110 LOCATE 2, 10: PRINT "the phenomenon of birefriegence"
1120 LOCATE 3, 5: PRINT "------------------------------------------------"
1130 '
1140 LOCATE 5, 10: PRINT "Please give the following quantities:"
1150 '
1160 LOCATE 7, 5: INPUT "Wave vector & frequency of the light = "; KV, WF
1170  IF KV > 0 AND WF > 0 THEN 1190
1180  BEEP:  LOCATE 7, 5: PRINT SPACE$(72): GOTO 1160
1190 LOCATE 9, 5: INPUT "X, Y components of its Electric Field = "; EXI, EYI
1200  IF EXI > 0 AND EYI > 0 THEN 1220
1210  BEEP:  LOCATE 9, 5: PRINT SPACE$(72): GOTO 1190
1220 '
1230 LOCATE 11, 5: INPUT "Phases of x,y components = "; PHX, PHY
1240 '
1250 LOCATE 13, 5: INPUT "Width of crystal = "; WID
1260  IF WID >= 0 THEN 1280
1270  BEEP:  LOCATE 13, 5: PRINT SPACE$(72): GOTO 1250
1280 LOCATE 15, 5: INPUT "Refractive indices along x,y direction = "; NX, NY
1290  IF NX >= 1 AND NY >= 1 THEN 1310
1300  BEEP:  LOCATE 15, 5: PRINT SPACE$(72): GOTO 1280
1310 LOCATE 17, 5: INPUT "Maximum time = "; TMAX
1320  IF TMAX > 0 THEN 1340
1330  BEEP:  LOCATE 17, 5: PRINT SPACE$(72): GOTO 1310
1340 '
1350 H = .1                                              'time step size fixed
1360                                                  'set limits for graphics
1370 IF EXI >= EYI THEN MAX = EXI ELSE MAX = EYI
1380  EXMAX = MAX: EXMIN = -MAX
1390  EYMAX = MAX: EYMIN = -MAX
1400 '
1410 RETURN
1420 '
2000 '************************************************************************
2010 'subroutine to find and components of polarization vector
2020 'input variables:  H, TMAX, KV, WF, WID, EXI, EYI, PHX, PHY, PI
2030 'output variables: EX, EY, EX1, EY1
2040 'global variables: none
2050 'local variables:  T, THETA, THETA1,
2060 '************************************************************************
2070 '
2080 T = 0                                                    'initialize time
2090 '
2100 WHILE T < TMAX
2110    X = 0                                                           'fix x
2120   EX = FNF(EXI, X, T, PHX)
2130   EY = FNF(EYI, X, T, PHY)
2140    X = WID / NX          'find effective width for x-component of E-field
2150   EX1 = FNF(EXI, X, T, PHX)
2160    X = WID / NY          'find effective width for y-component of E-field
2170   EY1 = FNF(EYI, X, T, PHY)
2180                                 'find angle of orientation without medium
2190   IF EX = 0 THEN EX = 1E-09                       'avoid division by zero
2200    THETA = ATN(EY / EX): THETA = THETA * 180 / PI
2210   IF EX > 0 AND EY < 0 THEN THETA = THETA + 360
2220   IF EX < 0 THEN THETA = THETA + 180
2230                                    'find angle of orientation with medium
2240   IF EX1 = 0 THEN EX1 = 1E-09                     'avoid division by zero
2250    THETA1 = ATN(EY1 / EX1): THETA1 = THETA1 * 180 / PI
2260    IF EX1 > 0 AND EY1 < 0 THEN THETA1 = THETA1 + 360
2270    IF EX1 < 0 THEN THETA1 = THETA1 + 180
2280    GOSUB 5000                                                'clear boxes
2290    GOSUB 6000                                         'draw field vectors
2300   T = T + H                                               'increment time
2310 WEND
2320 '
2330 RETURN
2340 '
3000 '************************************************************************
3010 'subroutine to draw axes of the graphs
3020 'input variables: EXMIN, EXMAX, EYMIN, EYMAX
3030 'output variables: XSCALE, YSCALE,  XL1%, YL1%, XM1%, YM1%, XO1%, YO1%,
3040 '                  XL2%, YL2%, XM2%, YM2%, XO2%, YO2%, XL%, YL%, TITLE$
3050 'local variables: none
3060 'global variable: none
3070 '************************************************************************
3080 CLS
3090 YSCALE = 64 / (EYMAX - EYMIN)                'find scales for x- & y-axes
3100 XSCALE = YSCALE * 2.5
3110                                          'draw box and axes for 1st graph
3120  TITLE$ = "Incoming Electric Field Vector"
3130  XL% = 64: YL% = 16                 'fix coordinates of one corner of box
3140 GOSUB 4000                                            'draw and mark axes
3150  YL1% = YL%: XL1% = XL%: YM1% = YM%: XM1% = XM%: YO1% = YO%: XO1% = XO%
3160                                      'preserve origin & other coordinates
3170                                          'draw box and axes for 2nd graph
3180  TITLE$ = "Outgoing Electric Field Vector"
3190  XL% = 364: YL% = 16                'fix coordinates of one corner of box
3200 GOSUB 4000                                            'draw and mark axes
3210  YL2% = YL%: XL2% = XL%: YM2% = YM%: XM2% = XM%: YO2% = YO%: XO2% = XO%
3220                                      'preserve origin & other coordinates
3230 RETURN
3240 '
4000 '************************************************************************
4010 'subroutine to draw axes of a graph
4020 'input variables: EXMIN, EXMAX, EYMIN, EYMAX, XSCALE, YSCALE, XL%, YL%
4030 'output variables: XL1%, YL1%, XM1%, YM1%, XO1%, YO1%,
4040 '                  XL2%, YL2%, XM2%, YM2%, XO2%, YO2%
4050 'local variables: COL%, XPT%, YPT%, J%, K%, XD%, YD%
4060 'global variables: none
4070 '************************************************************************
4080 '
4090 COL% = XL% / 8 - 6                       'draw box and axes for 1st graph
4100 LOCATE 1, COL%
4110 PRINT TITLE$
4120 '
4130 YM% = (EYMAX - EYMIN) * YSCALE + YL%       'find other coordinates of box
4140 XM% = (EXMAX - EXMIN) * XSCALE + XL%
4150 '
4160 XO% = .5 * (XL% + XM%): YO% = .5 * (YL% + YM%)               'find origin
4170  LINE (XL%, YO%)-(XM%, YO%)                                  'draw x-axis
4180  LINE (XO%, YL%)-(XO%, YM%)                                  'draw y-axis
4190 '
4200 LOCATE YO% / 8 - 1, XL% / 8 - 1: PRINT "^"
4210 LOCATE YO% / 8, XL% / 8 - 1: PRINT "|"                      'label y-axis
4220 LOCATE YO% / 8 + 1, XL% / 8 - 1: PRINT "Ey"
4230 '
4240 LOCATE YM% / 8 + 3, XO% / 8: PRINT "Ex -> "            'label x-axis
4250 '
4260 RETURN
4270 '
5000 '************************************************************************
5010 'subroutine to clear boxes
5020 'input variables: XL1%, YL1%, XM1%, YM1%, XO1%, YO1%
5030 '                  XL2%, YL2%, XM2%, YM2%, XO2%, YO2%
5040 'output variable: none
5050 'local variables: X%, Y%, X1%, Y1%
5060 'global variable: NI
5070 '************************************************************************
5080 '                                             'clear quadrants of 1st box
5090 LINE (XL1%, YL1%)-(XO1% - 1, YO1% - 1), 0, BF
5100 LINE (XL1%, YO1% + 1)-(XO1% - 1, YM1%), 0, BF
5110 LINE (XO1% + 1, YL1%)-(XM1%, YO1% - 1), 0, BF
5120 LINE (XO1% + 1, YO1% + 1)-(XM1%, YM1%), 0, BF
5130 '                                             'clear quadrants of 2nd box
5140 LINE (XL2%, YL2%)-(XO2% - 1, YO2% - 1), 0, BF
5150 LINE (XL2%, YO2% + 1)-(XO2% - 1, YM2%), 0, BF
5160 LINE (XO2% + 1, YL2%)-(XM2%, YO2% - 1), 0, BF
5170 LINE (XO2% + 1, YO2% + 1)-(XM2%, YM2%), 0, BF
5180 '
5190 RETURN
5200 '
6000 '************************************************************************
6010 'subroutine to draw Electric field vectors & print results
6020 'input variables: EXMIN, EYMAX, EX, EY, EX1, EY1, T, THETA, THETA1
6030 '                 XO1%, YO1%, XO2%, YO2%, XSCALE, YSCALE
6040 'output variable: none
6050 'local variables: X%, Y%, X1%, Y1%
6060 'global variable: none
6070 '************************************************************************
6080 '
6090 X% = (EX - EXMIN) * XSCALE + XL1%
6100 Y% = (EYMAX - EY) * YSCALE + YL1%
6110 '
6120 X1% = (EX1 - EXMIN) * XSCALE + XL2%
6130 Y1% = (EYMAX - EY1) * YSCALE + YL2%
6140 '
6150 LINE (XO1%, YO1%)-(X%, Y%)                    'draw incoming field vector
6160 LINE (XO2%, YO2%)-(X1%, Y1%)                  'draw outgoing field vector
6170                                                            'print results
6180 LOCATE 17, 15
6190  PRINT USING "t   = +###.##,               t   = +###.##"; T; T
6200 LOCATE 19, 15
6210  PRINT USING "Ex  = +###.##,               Ex  = +###.##"; EX; EX1
6220 LOCATE 20, 15
6230  PRINT USING "Ey  = +###.##,               Ey  = +###.##"; EY; EY1
6240 LOCATE 21, 15
6250  PRINT USING "phi = +###.##,               phi = +###.##"; THETA; THETA1
6260 '
6270 RETURN
6280 '
10000 '***********************************************************************
10010 'subroutine to display header screen
10020 'input variables:  none
10030 'output variables: none
10040 'global variables: none
10050 'local variables:  none
10060 '***********************************************************************
10070 CLS : KEY OFF                                           'clear screen
10080 SCREEN 2, 0, 0: WIDTH 80                               'change screen
10090  '
10100 LINE (1, 1)-(638, 40), , B                                'draw boxes
10110 LINE (5, 3)-(633, 37), , B
10120 LINE (70, 50)-(570, 140), , B
10130 LINE (75, 53)-(565, 137), , B
10140                                           'print title, authors, etc.
10150 LOCATE 3, 7
10160 PRINT " C O M P U T A T I O N A L   P H Y S I C S :";
10170 PRINT " (An  Introduction)"
10180 LOCATE 9, 40
10190 PRINT "by";
10200 LOCATE 11, 20: PRINT "R.C. Verma, P.K. Ahluwalia & K.C. Sharma"
10210 LOCATE 13, 34
10220 PRINT "   * * * *    "
10230 '
10240 BEEP
10250 LOCATE 15, 35: PRINT "* * * * * *"
10260 LOCATE 22, 25: PRINT "press any key to continue"
10270 LOCATE 23, 25: PRINT "-------------------------"
10280 '
10290 IF INKEY$ = "" THEN 10290                            'hold the screen
10300 CLS                                                     'clear screen
10310 RETURN
10320 '

