10 '**************************************************************************
20 'Chapter 6, Example 1: To Draw Electric Field due to a Point Charge
30 'COMPUTATIONAL PHYSICS: An Introduction
40 'by R. C. Verma, P. K. Ahluwalia and K. C. Sharma
50 '   * * * *   
60 '**************************************************************************
70  CLS : KEY OFF
80  GOSUB 10000                                                   'show header
90 '
100 KUNIT = 1                                'define constant k (c.g.s. units)
110 '
120 GOSUB 1000             'to introduce the problem & to get input parameters
130 GOSUB 2000                                 'find and draw field components
140 '
150 LOCATE 22, 1
160 '
170 END
180 '
1000 '************************************************************************
1010 'subroutine to introduce the problem & to get input variables
1020 'input variables:  none
1030 'output variables: Q, X1, Y1, XMIN, YMIN, XMAX, YMAX, MX, MY
1040 'global variables: none
1050 'local variables:  D
1060 'test cases: Q = 1
1070 '************************************************************************
1080 '
1090 LOCATE 2, 10: PRINT "This program draws electric field"
1100 LOCATE 3, 15: PRINT "due to a single charge"
1110 LOCATE 4, 10: PRINT "-------------------------------------"
1120 LOCATE 10, 10: BEEP: PRINT "Please give the following parameters:"
1130 '
1140 LOCATE 15, 5: INPUT "Give Charge Q "; Q
1150  IF Q <> 0 THEN 1170
1160  LOCATE 15, 5: PRINT SPACE$(72): BEEP: GOTO 1140
1170 '
1180 X1 = 320: Y1 = 108                            'location of charge Q fixed
1190 XMAX = 520: YMAX = 184: XMIN = 120: YMIN = 24                'set graphics
1200 D = 10                                          'set dimension of squares
1210 MX = (XMAX - XMIN) / D: MY = (YMAX - YMIN) / D       'find grid structure
1220 '
1230 RETURN
1240 '
2000 '************************************************************************
2010 'subroutine to do numerical work
2020 'input variables: MX, MY, Q, XMIN, YMIN, D, X1, Y1,
2030 'output variables: none
2040 'global variables: none
2050 'local variables: J%, K%, X, Y, NX, NY, EX, EY, E, XD, YD, R
2060 '************************************************************************
2070 CLS
2080 '
2090 LY% = X1 / 8                                'locating charge for graphics
2100 LX% = Y1 / 8
2110 LOCATE LX%, LY%: IF Q > 0 THEN PRINT "+" ELSE PRINT "-"      'show charge
2120 '
2130 LOCATE 1, 10: PRINT "Electric Field due to a point charge"
2140 '
2150  FOR K% = 1 TO MY                                       'loop over y-axis
2160    Y = YMIN + (K% - .5) * D                               'find current y
2170    FOR J% = 1 TO MX                                     'loop over x axis
2180      X = XMIN + (J% - .5) * D                             'find current x
2190      XD = (X - X1): YD = (Y - Y1)               'find distance components
2200      R = SQR(XD ^ 2 + YD ^ 2)                              'find distance
2205      IF R = 0 THEN 2260                                'avoid singularity
2210       EX = KUNIT * Q * XD / R ^ 3
2220       EY = KUNIT * Q * YD / R ^ 3                  'find field components
2230       E = SQR(EX * EX + EY * EY)                    'find field magnitude
2240      NX = EX * D / E: NY = EY * D / E             'scale field components
2250      LINE (X - .5 * NX, Y - .5 * NY)-(X + .5 * NX, Y + .5 * NY)'draw line
2260   NEXT J%
2270 NEXT K%
2280 '
2290 RETURN
2300 '
10000 '***********************************************************************
10010 'subroutine to display header screen
10020 'input variables:  none
10030 'output variables: none
10040 'global variables: none
10050 'local variables:  none
10060 '***********************************************************************
10070 CLS : KEY OFF                                              'clear screen
10080 SCREEN 2, 0, 0: WIDTH 80                                  'change screen
10090  '
10100 LINE (1, 1)-(638, 40), , B                                   'draw boxes
10110 LINE (5, 3)-(633, 37), , B
10120 LINE (70, 50)-(570, 140), , B
10130 LINE (75, 53)-(565, 137), , B
10140                                              'print title, authors, etc.
10150 LOCATE 3, 7
10160 PRINT " C O M P U T A T I O N A L   P H Y S I C S :";
10170 PRINT " (An  Introduction)"
10180 LOCATE 9, 40
10190 PRINT "by";
10200 LOCATE 11, 20: PRINT "R.C. Verma, P.K. Ahluwalia & K.C. Sharma"
10210 LOCATE 13, 34
10220 PRINT "   * * * *    "
10230 '
10240 BEEP
10250 LOCATE 15, 35: PRINT "* * * * * *"
10260 LOCATE 22, 25: PRINT "press any key to continue"
10270 LOCATE 23, 25: PRINT "-------------------------"
10280 '
10290 IF INKEY$ = "" THEN 10290                               'hold the screen
10300 CLS                                                        'clear screen
10310 RETURN
10320 '

