10 '**************************************************************************
20 'Chapter 6, Example 3: Equipotential surfaces for a system of N - charges
30 'COMPUTATIONAL PHYSICS: An Introduction
40 'by R. C. Verma, P. K. Ahluwalia and K. C. Sharma
50 '   * * * *   
60 '**************************************************************************
70 '
80 GOSUB 10000                                                    'show header
90 '
100 KUNIT = 1                                     'k = 1 (c.g.s. units chosen)
110 '
120 GOSUB 1000             'to introduce the problem & to get input parameters
130 GOSUB 2000                                                   'set graphics
140 GOSUB 3000                                       'find & draw field vector
150 '
160 LOCATE 21, 1
170 '
180 END
190 '
1000 '************************************************************************
1010 'subroutine to introduce the problem & to get input variables
1020 'input variables:  none
1030 'output variables: N%, Q(I%), X(I%), Y(I%),
1040 'global variables: none
1050 'local variables:  I%
1060 '************************************************************************
1070 '
1080 LOCATE 2, 10: PRINT "This program draws equipotential curves"
1090 LOCATE 3, 15: PRINT "for a system of N charges"
1100 LOCATE 4, 10: PRINT "-------------------------------------"
1110 LOCATE 6, 10: BEEP: PRINT "Please give the following parameters:"
1120 '
1130 XMIN = 120: XMAX = 520: YMIN = 24: YMAX = 184
1140                                     'choose region of screen for graphics
1150 LOCATE 8, 5: INPUT "No. of charges = "; N%
1160  IF N% > 0 THEN 1190
1170  LOCATE 8, 5: PRINT SPACE$(72): BEEP: GOTO 1150
1180 '
1190 LOCATE 10, 5
1200 PRINT " Give location of charges with in the following bounds:": PRINT
1210  PRINT XMIN; " < x < "; XMAX: PRINT
1220  PRINT YMIN; " < y < "; YMAX
1230 '
1240 FOR I% = 1 TO N%
1250   LOCATE 16, 5: BEEP: PRINT "Charge No. "; I%
1260   LOCATE 18, 5: PRINT SPACE$(72)
1270   LOCATE 18, 5: INPUT "Charge (Q), position (x, y) "; Q(I%), X(I%), Y(I%)
1280   LOCATE 19, 5: PRINT SPACE$(72)
1290     IF Q(I%) = 0 THEN 1330                    'check for right parameters
1300     IF X(I%) > XMAX OR X(I%) < XMIN THEN 1330
1310     IF Y(I%) > YMAX OR Y(I%) < YMIN THEN 1330
1320      GOTO 1350
1330   LOCATE 19, 5: PRINT "Give nonzero charge & position within the bounds!"
1340   BEEP: GOTO 1270
1350 NEXT I%
1360 '
1370 RETURN
1380 '
2000 '************************************************************************
2010 'subroutine to show charge locations & set grid size for graphics
2020 'input variables: N%, Q(I%), X(I%), Y(I%), XMIN, XMAX, YMIN, YMAX
2030 'output variables:, DX, DY, MX, MY
2040 'global variables: none
2050 'local variables:  X%, Y%
2060 '************************************************************************
2070 CLS
2080 LOCATE 1, 15: PRINT "Equipotential surfaces for "; N%; "charges"
2090 '
2100 FOR I% = 1 TO N%
2105   FOR PT% = -2 TO 2
2110     PSET (X(I%) + PT%, Y(I%))                      'draw - to show charges
2120      IF Q(I%) < 0 THEN 2130
2125     PSET (X(I%), Y(I%) + PT%)                'make it + if positive charge
2130   NEXT PT%
2140 NEXT I%
2150 '
2160 MX = 20: MY = 20                                       'no of steps fixed
2170 DX = (XMAX - XMIN) / MX: DY = (YMAX - YMIN) / MY
2180                                              'find dimensions of grid box
2190 RETURN
2200 '
3000 '************************************************************************
3010 'subroutine to  find components of electric field
3020 'input variables:  N%, Q(I%), X(I%), Y(I%), XMIN, YMIN, DX, DY, MX, MY
3030 'output variables: none
3040 'global variables: none
3050 'local variables:  X, XD, YD, R, EX, EY, SEX, SEY, EF, NX, NY, I%, J%, K%
3060 '************************************************************************
3070 '
3080 FOR K% = 1 TO MY
3090   Y = YMIN + (K% - .5) * DY                         'loop for y component
3100   FOR J% = 1 TO MX
3110     SEX = 0: SEY = 0                         'initialize field components
3120     X = XMIN + (J% - .5) * DX                       'loop for x component
3130     FOR I% = 1 TO N%                                   'loop over charges
3140      XD = (X - X(I%)): YD = (Y - Y(I%))                'location of point
3150      R = SQR(XD ^ 2 + YD ^ 2)                              'find distance
3155      IF R = 0 THEN 3180                                'avoid singularity
3160      EX = KUNIT * Q(I%) * XD / R ^ 1.5: EY = KUNIT * Q(I%) * YD / R ^ 1.5
3165                                                         'field components
3170      SEX = SEX + EX: SEY = SEY + EY              'update field components
3180     NEXT I%
3190     EF = SQR(SEX * SEX + SEY * SEY)                   'magnitude of field
3195     IF EF = 0 THEN 3220               'avoid drawing if net field is zero
3200     NX = -SEY * DX / EF: NY = SEX * DY / EF
3205                                  'find direction of equipotential surface
3210     LINE ((X - .5 * NX), Y - .5 * NY)-((X + .5 * NX), Y + .5 * NY)
3215                                                                'draw line
3220   NEXT J%
3230 NEXT K%
3240 '
3250 RETURN
3260 '
10000 '***********************************************************************
10010 'subroutine to display header screen
10020 'input variables:  none
10030 'output variables: none
10040 'global variables: none
10050 'local variables:  none
10060 '***********************************************************************
10070 CLS : KEY OFF                                           'clear screen
10080 SCREEN 2, 0, 0: WIDTH 80                               'change screen
10090  '
10100 LINE (1, 1)-(638, 40), , B                                'draw boxes
10110 LINE (5, 3)-(633, 37), , B
10120 LINE (70, 50)-(570, 140), , B
10130 LINE (75, 53)-(565, 137), , B
10140                                           'print title, authors, etc.
10150 LOCATE 3, 7
10160 PRINT " C O M P U T A T I O N A L   P H Y S I C S :";
10170 PRINT " (An  Introduction)"
10180 LOCATE 9, 40
10190 PRINT "by";
10200 LOCATE 11, 20: PRINT "R.C. Verma, P.K. Ahluwalia & K.C. Sharma"
10210 LOCATE 13, 34
10220 PRINT "   * * * *    "
10230 '
10240 BEEP
10250 LOCATE 15, 35: PRINT "* * * * * *"
10260 LOCATE 22, 25: PRINT "press any key to continue"
10270 LOCATE 23, 25: PRINT "-------------------------"
10280 '
10290 IF INKEY$ = "" THEN 10290                            'hold the screen
10300 CLS                                                     'clear screen
10310 RETURN
10320 '

