10 '**************************************************************************
20 'Chapter 7, Example 1: Charged Particle in an Electric field
25 '                  (Motion of an electron in CRT-Electric field)
30 'COMPUTATIONAL PHYSICS: An Introduction
40 'by R. C. Verma, P. K. Ahluwalia and K. C. Sharma
50 '   * * * *   
60 '**************************************************************************
70 '
80 GOSUB 10000                                                    'show header
90 '
100 PI = 3.1415926#                                        'define constant pi
110 '
120 DEF FNL (X) = (1 - 2 * D / TUBL) * X + (2 * D - .5 * TUBL)
130                                                'eq. for front part of tube
140 GOSUB 1000                   'introduce the problem & get input parameters
150 GOSUB 3000                                                'to draw the CRT
160 GOSUB 2000                                  'find position of the particle
170 '
180 LOCATE 22, 1
190 IF INKEY$ = "" THEN 190
200 '
210 CLS : LOCATE 12, 15: BEEP: PRINT " Thank you !"
220 END
230 '
1000 '************************************************************************
1010 'subroutine to introduce the problem & to get input variables
1020 'input variables:  none
1030 'output variables: CH, MASS, VI, L, D, POT, TUBL, H, XI, YI, TH
1040 'global variables: none
1050 'local variables:  none
1060 'test case: CH=1, MASS=1, VI=1, L=10, D=5, POT =0.1, TUBL=50
1070 '************************************************************************
1080 '
1090 LOCATE 1, 10: PRINT "This program describes motion of an electron"
1100 LOCATE 2, 15: PRINT "under Electric field in a Cathode Ray Tube"
1110 LOCATE 3, 10: PRINT "***************************************************"
1120 LOCATE 4, 10: PRINT "Give the following parameters:"
1130 '
1140 CH = -1: MASS = 1  'charge and mass of the electron fixed in some units
1150 '
1160 LOCATE 6, 5: INPUT "Initial velocity of the electron "; VI
1170   IF VI > 0 THEN 1190
1180   BEEP: LOCATE 6, 5: PRINT SPACE$(72): GOTO 1160
1190 LOCATE 8, 5: INPUT "Length of the plates < 20"; L
1200   IF L > 0 AND L <= 20 THEN 1220
1210   BEEP: LOCATE 8, 5: PRINT SPACE$(72): GOTO 1190
1220 LOCATE 10, 5: INPUT "Distance between the plates < 10 "; D
1230   IF D > 0 AND D <= 10 THEN 1250
1240   BEEP: LOCATE 10, 5: PRINT SPACE$(72): GOTO 1220
1250 LOCATE 12, 5: INPUT "Potential difference between plates "; POT
1260 '
1280 LOCATE 14, 5: INPUT "Give length of tube  < 100 "; TUBL
1290   IF TUBL > 0 AND TUBL <= 100 THEN 1310
1300   BEEP: LOCATE 14, 5: PRINT SPACE$(72): GOTO 1280
1310 LOCATE 16, 5: INPUT "Give time step "; H
1320   IF H <> 0 THEN 1340
1330   BEEP: LOCATE 16, 5: PRINT SPACE$(72): GOTO 1310
1340 '
1350 TH = 0                                 'initial angle of projection fixed
1360 XI = 0: YI = 0                        'position of electron emitter fixed
1370 '
1380 RETURN
1390 '
2000 '************************************************************************
2010 'subroutine to find position of particle using Euler method
2020 'input variables: POT, D, TH, PI, VI, CH, MASS, TUBL, XI, YI, H
2030 'output variables: T, X, Y, KE
2040 'global variables: none
2050 'local variables: VX, VY, EF,  AX, AY
2060 '************************************************************************
2070 '
2080 EF = POT / D                                         'find electric field
2090 '
2100 TH = TH * PI / 180                              'convert degree to radian
2110 VX = VI * COS(TH): VY = VI * SIN(TH)            'find velocity components
2120  X = XI: Y = YI                                           'initialization
2130 '
2140   AX = 0                                  'no acceleration in x-direction
2150   AY = CH * EF / MASS                        'acceleration in y-direction
2160   T = 0                                                  'initialize time
2170 '
2180 WHILE X <= TUBL
2190   KE = .5 * MASS * (VX * VX + VY * VY)               'find kinetic energy
2200   LOCATE 22, 1
2210  PRINT USING "t=###.##, KE= ####.###, X= +##.###, Y=+##.###"; T; KE; X; Y
2220 '
2230   IF X <= L AND ABS(Y) > .5 * D THEN 2410
2240                                            'see if particle strikes plate
2250   IF X <= .5 * TUBL AND ABS(Y) > D THEN 2430
2260                                          'see if it touches CRT rear side
2270   IF X >= .5 * TUBL AND ABS(Y) > FNL(X) THEN 2450
2280                                             'if it touches CRT front side
2290   GOSUB 4000                                             'draw trajectory
2300   IF X > L THEN AY = 0                   'if particle comes out of plates
2310  '
2320   VX = VX + AX * H                                      'velocity formula
2330   VY = VY + AY * H
2340    X = X + VX * H                                       'distance formula
2350    Y = Y + VY * H
2360   T = T + H                                               'increment time
2370 WEND
2380 '
2390 BEEP: LOCATE 20, 10
2400  PRINT "particle has touched the CRT screen": GOTO 2480
2410 BEEP: LOCATE 20, 10
2420  PRINT "particle has collided with plates": GOTO 2480
2430 BEEP: LOCATE 20, 10
2440  PRINT "particle has touched the CRT rear side": GOTO 2480
2450 BEEP: LOCATE 20, 10
2460  PRINT "particle has touched the CRT front side": GOTO 2480
2470 '
2480 RETURN
2490 '
3000 '************************************************************************
3010 'subroutine to draw CRT along with its electrode system
3020 'input variables: D,  TUBL, XI
3030 'output variables: K1, K2, K3, K4,
3040 'global variables: none
3050 'local variables: PXMIN, PXMAX, PYMIN, PYMAX, TXL%, TYL%, TXM%, TYM%
3060 '        XMIN, XMAX, YMIN, YMAX, PXL%, PYL%, PXM%, PYM%
3070 '        GXL%, GYL%, GXM%, GYM%, PMID%, PYL%, PMID%, PYLD%, PYM%, PYMD%
3080 '************************************************************************
3090 '
3100 CLS : KEY OFF                                               'clear screen
3110 LOCATE 2, 10: PRINT "Particle Trajectory in Electric Field"
3120 '
3130 PXMIN = 100: PXMAX = 500: PYMIN = 50: PYMAX = 150   'screen region chosen
3140 '
3150 XMAX = TUBL: XMIN = XI                          'max & min pts. on x-axis
3160 YMAX = .5 * TUBL: YMIN = -YMAX                  'max & min pts. on y axis
3170                               'find coefficients for scale transformation
3180 K1 = (PXMAX - PXMIN) / (XMAX - XMIN)
3190 K2 = (PXMIN * XMAX - PXMAX * XMIN) / (XMAX - XMIN)
3200 K3 = (PYMIN - PYMAX) / (YMAX - YMIN)
3210 K4 = (PYMAX * YMAX - PYMIN * YMIN) / (YMAX - YMIN)
3220                                         'scale transformations for plates
3230  PXL% = K1 * 0 + K2
3240  PYL% = K3 * (-.5 * D) + K4
3250  PXM% = K1 * L + K2
3260  PYM% = K3 * (.5 * D) + K4
3270 LINE (PXL%, PYL%)-(PXM%, PYL%)                          'draw lower plate
3280 LINE (PXL%, PYM%)-(PXM%, PYM%)                          'draw upper plate
3290 LINE (PMID%, PYL%)-(PMID%, PYL%)                        'draw lower plate
3300 '
3310  PYMD% = K3 * (2 * D) + K4: PMID% = .5 * (PXL% + PXM%)  'electrodes lines
3320  PYLD% = K3 * (-2 * D) + K4
3330 LINE (PMID%, PYL%)-(PMID%, PYLD%)                   'draw lower electrode
3340 LINE (PMID%, PYM%)-(PMID%, PYMD%)                   'draw upper electrode
3350                                           'scale transformations for tube
3360  TXL% = K1 * (-.5 * D) + K2                                 'rear of tube
3370  TYL% = K3 * (-D) + K4
3380  TXM% = K1 * (.5 * TUBL) + K2
3390  TYM% = K3 * (D) + K4
3400 LINE (TXL%, TYL%)-(TXL%, TYM%)                     'draw back end of tube
3410 LINE (TXL%, TYL%)-(TXM%, TYL%)                   'draw lower line of tube
3420 LINE (TXL%, TYM%)-(TXM%, TYM%)                   'draw upper line of tube
3430 '
3440  TXL% = K1 * (-.5 * D) + K2                                'front of tube
3450  FYL% = K3 * (YMIN) + K4
3460  FXM% = K1 * (TUBL) + K2
3470  FYM% = K3 * (YMAX) + K4
3480 LINE (TXM%, TYL%)-(FXM%, FYL%)             'draw lower front line of tube
3490 LINE (TXM%, TYM%)-(FXM%, FYM%)             'draw upper front line of tube
3500 LINE (FXM%, FYL%)-(FXM%, FYM%)                     'draw back end of tube
3510 '
3520  GXL% = K1 * (-.5 * D) + K2                     'location of electron gun
3530  GYL% = K3 * (-.2 * D) + K4
3540  GXM% = K1 * 0 + K2
3550  GYM% = K3 * (.2 * D) + K4
3560 '
3570 LINE (GXL%, GYL%)-(GXM%, GYM%), 1, BF                  'draw electron gun
3580 '
3590 RETURN
3600 '
4000 '************************************************************************
4010 'subroutine to draw trajectory of charged particle
4020 'input variables:  K1, K2, K3, K4, X, Y, XI
4030 'output variables: none
4040 'global variables: none
4050 'local variables: PX%, PY%
4060 '************************************************************************
4070                       'perform scale transformation for coordinates x & y
4080  PX% = K1 * X + K2
4090  PY% = K3 * Y + K4
4100  IF X > XI THEN LINE -(PX%, PY%) ELSE PSET (PX%, PY%)    'draw trajectory
4110 '
4120 RETURN
4130 '
10000 '***********************************************************************
10010 'subroutine to display header screen
10020 'input variables:  none
10030 'output variables: none
10040 'global variables: none
10050 'local variables:  none
10060 '***********************************************************************
10070 CLS : KEY OFF                                           'clear screen
10080 SCREEN 2, 0, 0: WIDTH 80                               'change screen
10090  '
10100 LINE (1, 1)-(638, 40), , B                                'draw boxes
10110 LINE (5, 3)-(633, 37), , B
10120 LINE (70, 50)-(570, 140), , B
10130 LINE (75, 53)-(565, 137), , B
10140                                           'print title, authors, etc.
10150 LOCATE 3, 7
10160 PRINT " C O M P U T A T I O N A L   P H Y S I C S :";
10170 PRINT " (An  Introduction)"
10180 LOCATE 9, 40
10190 PRINT "by";
10200 LOCATE 11, 20: PRINT "R.C. Verma, P.K. Ahluwalia & K.C. Sharma"
10210 LOCATE 13, 34
10220 PRINT "   * * * *    "
10230 '
10240 BEEP
10250 LOCATE 15, 35: PRINT "* * * * * *"
10260 LOCATE 22, 25: PRINT "press any key to continue"
10270 LOCATE 23, 25: PRINT "-------------------------"
10280 '
10290 IF INKEY$ = "" THEN 10290                            'hold the screen
10300 CLS                                                     'clear screen
10310 RETURN
10320 '

