10 '**************************************************************************
20 'Chapter 7, Example 2: Motion of charged particle in uniform magnetic field
30 '                      (using Half-step method)
40 'COMPUTATIONAL PHYSICS: An Introduction
50 'by R. C. Verma, P. K. Ahluwalia and K. C. Sharma
60 '   * * * *   
70 '**************************************************************************
80 '
90 GOSUB 10000                                                    'show header
100 '
110 PI = 3.1415926#                                        'define constant pi
120 '
130 GOSUB 1000                   'introduce the problem & get input parameters
140 GOSUB 3000                                           'draw magnetic region
150 GOSUB 2000                                           'find & draw position
160 '
170 LOCATE 22, 1
180 IF INKEY$ = "" THEN 180
190 '
200 CLS : LOCATE 12, 15: BEEP: PRINT " Thank you !"
210 END
220 '
1000 '************************************************************************
1010 'subroutine to introduce the problem & to get input variables
1020 'input variables:  none
1030 'output variables: CH,MASS, VI,TH, XI,YI, MAG, TMAX, H, LSPEED, RS, XE,YE
1040 'global variables: none
1050 'local variables:  none
1060 'test cases:CH=1,MASS=1,RS=50, XI=10,YI=40, VI=4 , TH=0  H=0.1, TMAX= 100
1070 '************************************************************************
1080 '
1090 LOCATE 1, 10: PRINT "This program describes motion of a charged particle"
1100 LOCATE 2, 15: PRINT "inside a magnetic field "
1110 LOCATE 3, 10: PRINT "---------------------------------------------------"
1120 LOCATE 4, 5: PRINT "Give the following parameters:"
1130 '
1140 LOCATE 6, 5: INPUT "Charge & mass of the particle "; CH, MASS: PRINT
1150  IF MASS > 0 THEN 1170
1160  BEEP: LOCATE 6, 5: PRINT SPACE$(72): GOTO 1140
1170 LOCATE 8, 5: INPUT "Initial velocity of the particle "; VI
1180  IF VI <> 0 THEN 1200
1190  BEEP: LOCATE 8, 5: PRINT SPACE$(72): GOTO 1170
1200 LOCATE 10, 5: INPUT "Angle of shooting  of the particle "; TH
1210  IF TH >= -360 AND TH <= 360 THEN 1230
1220  BEEP: LOCATE 10, 5: PRINT SPACE$(72): GOTO 1200
1230 LOCATE 12, 5: INPUT "Initial position x,y (-100 to 100)"; XI, YI
1240  IF ABS(XI) <= 100 AND ABS(YI) <= 100 THEN 1260
1250  BEEP: LOCATE 12, 5: PRINT SPACE$(72): GOTO 1230
1260 LOCATE 14, 5: INPUT "Strength of the magnetic field (0.5)"; MAG
1270  IF MAG <> 0 THEN 1290
1280  BEEP: LOCATE 14, 5: PRINT SPACE$(72): GOTO 1260
1290 LOCATE 16, 5: INPUT "Maximum time "; TMAX
1300  IF TMAX > 0 THEN 1320
1310  BEEP: LOCATE 16, 5: PRINT SPACE$(72): GOTO 1290
1320 LOCATE 18, 5: INPUT "Time step size "; H
1330  IF H <> 0 THEN 1350
1340  BEEP: LOCATE 18, 5: PRINT SPACE$(72): GOTO 1320
1350 '
1360                            'set parameters of the field etc. for graphics
1370 RS = 50                                  'radius of magnetic region fixed
1380 LSPEED = 1                                            'speed of light = 1
1390 '
1400 XE = 120: YE = 100                      'position of magnetic field fixed
1410 '
1420 RETURN
1430 '
2000 '************************************************************************
2010 'subroutine to find position of the particle using half-step method
2020 'input variables: CH, MAG, MASS, LSPEED, TH, PI, VI, XI, YI, TMAX, H, RS
2030 'output variables: T, KE, X, Y, X0, Y0
2040 'global variables: none
2050 'local variables: MF, UX, UY, VX, VY, VXH, VYH, AX, AY, R
2060 '************************************************************************
2070 '
2080 MF = CH * MAG / (MASS * LSPEED)                            'useful factor
2090 '
2100 TH = TH * PI / 180                              'convert degree to radian
2110 UX = VI * COS(TH): UY = VI * SIN(TH)            'find velocity components
2120 '
2130 VX = UX: VY = UY: X = XI: Y = YI                          'initialization
2140 X0 = XI: Y0 = YI
2150 '
2160   AX = MF * VY                                    'acceleration decrement
2170   AY = -MF * VX
2180   VXH = UX + .5 * AX * H                   'velocity formula at half-step
2190   VYH = UY + .5 * AY * H
2200   T = 0                                                  'initialize time
2210 '
2220 WHILE (T < TMAX AND ABS(X) < 200 AND ABS(Y) < 200)
2230   IF (X * X + Y * Y) > RS * RS THEN 2280     'particle lies outside field
2240   AX = MF * VY                                    'acceleration decrement
2250   AY = -MF * VX
2260   VX = VXH + AX * H                                     'velocity formula
2270   VY = VYH + AY * H
2280   X = X0 + VX * H                                       'distance formula
2290   Y = Y0 + VY * H
2300   GOSUB 4000                                             'draw trajectory
2310 '
2320   R = SQR(X ^ 2 + Y ^ 2)
2330   KE = .5 * MASS * (VX * VX + VY * VY)               'find kinetic energy
2340 '
2350   LOCATE 20, 1
2360   PRINT USING "time = ###.##, K.E.= ####.###,"; T; KE: PRINT
2370   PRINT USING "distance from centre of field = ###.###"; R
2380 '
2390   X0 = X: Y0 = Y                                           'roll position
2400   IF R <= RS THEN VXH = VX: VYH = VY                       'roll velocity
2410   T = T + H                                               'increment time
2420 WEND
2430 '
2440 RETURN
2450 '
3000 '************************************************************************
3010 'subroutine to draw draw the magnetic field
3020 'input variables: RS, XE, YE,
3030 'output variables: X, Y
3040 'global variables: none
3050 'local variables: HC, XP, YP, ZP, XP0, YP0, ZP0
3060 '************************************************************************
3070 CLS
3080 LOCATE 1, 5
3090 PRINT "Simulation of Trajectory of a charged Particle"
3100 PRINT "          inside a magnetic field"
3110  HC = 1                       'step size to draw circular magnetic region
3120 '
3130 FOR I% = 0 TO 2 * RS                        'loop to draw magnetic region
3140   XP = -RS + I% * HC + XE
3150   YP = YE + SQR(RS * RS - (XP - XE) ^ 2)
3160   ZP = YE - SQR(RS * RS - (XP - XE) ^ 2)
3170   XP = XP * 2.5                             'multiply by aspect ratio 2.5
3180   IF I% = 0 THEN 3210                         'normalize the x & y scales
3190   LINE (XP0, YP0)-(XP, YP)
3200   LINE (XP0, ZP0)-(XP, ZP)
3210   XP0 = XP: YP0 = YP: ZP0 = ZP                          'roll coordinates
3220 NEXT I%
3230 '
3240 RETURN
3250 '
4000 '************************************************************************
4010 'subroutine to display trajectory
4020 'input variables: XE, YE, X0, Y0, X, Y
4030 'output variables: none
4040 'global variables: none
4050 'local variables: X1, Y1, X2, Y2
4060 '************************************************************************
4070 '
4080 Y1 = YE - Y0                            'scale transformations last point
4090 X1 = XE + X0: X1 = X1 * 2.5                         '2.5 for aspect ratio
4100 Y2 = YE - Y                                         'for present position
4110 X2 = XE + X: X2 = X2 * 2.5
4120 LINE (X1, Y1)-(X2, Y2)                                   'draw trajectory
4130 '
4140 RETURN
4150 '
10000 '***********************************************************************
10010 'subroutine to display header screen
10020 'input variables:  none
10030 'output variables: none
10040 'global variables: none
10050 'local variables:  none
10060 '***********************************************************************
10070 CLS : KEY OFF                                           'clear screen
10080 SCREEN 2, 0, 0: WIDTH 80                               'change screen
10090  '
10100 LINE (1, 1)-(638, 40), , B                                'draw boxes
10110 LINE (5, 3)-(633, 37), , B
10120 LINE (70, 50)-(570, 140), , B
10130 LINE (75, 53)-(565, 137), , B
10140                                           'print title, authors, etc.
10150 LOCATE 3, 7
10160 PRINT " C O M P U T A T I O N A L   P H Y S I C S :";
10170 PRINT " (An  Introduction)"
10180 LOCATE 9, 40
10190 PRINT "by";
10200 LOCATE 11, 20: PRINT "R.C. Verma, P.K. Ahluwalia & K.C. Sharma"
10210 LOCATE 13, 34
10220 PRINT "   * * * *    "
10230 '
10240 BEEP
10250 LOCATE 15, 35: PRINT "* * * * * *"
10260 LOCATE 22, 25: PRINT "press any key to continue"
10270 LOCATE 23, 25: PRINT "-------------------------"
10280 '
10290 IF INKEY$ = "" THEN 10290                            'hold the screen
10300 CLS                                                     'clear screen
10310 RETURN
10320 '

