10 '**************************************************************************
20 'Chapter 7, Example 2(a): Charged particle in uniform magnetic field
30 '                         (using Predictor-Corrector method)
40 'COMPUTATIONAL PHYSICS: An Introduction
50 'by R. C. Verma, P. K. Ahluwalia and K. C. Sharma
60 '   * * * *   
70 '**************************************************************************
80 '
90  GOSUB 10000                                                   'show header
100 '
110 PI = 3.1415926#                                        'define constant pi
120 '
130 DEF FNX (V) = MAGFAC * V                     'define acceleration function
140 DEF FNY (V) = -MAGFAC * V
150 '
160 GOSUB 1000                   'introduce the problem & get input parameters
170 GOSUB 3000                                           'draw magnetic region
180 GOSUB 2000                                           'perform Euler method
190 '
200 LOCATE 22, 1
210 IF INKEY$ = "" THEN 210
220 '
230 CLS : LOCATE 12, 15: BEEP: PRINT " Thank you !"
240 END
250 '
1000 '************************************************************************
1010 'subroutine to introduce the problem & to get input variables
1020 'input variables:  none
1030 'output variables: CH,MASS, VI,TH, XI,YI, MAG, TMAX, H, LSPEED, RS, XE,YE
1040 'global variables: none
1050 'local variables:  none
1060 'test case: RS=50, XI=10,YI=40, VI=4 , TH=0  H=0.1, TMAX= 100, MAG= 0.5
1065 '           CH=1, MASS=1
1070 '************************************************************************
1080 '
1090 LOCATE 1, 10: PRINT "This program describes motion of a charged particle"
1100 LOCATE 2, 15: PRINT "inside a magnetic field "
1110 LOCATE 3, 10: PRINT "---------------------------------------------------"
1120 LOCATE 4, 5: PRINT "Give the following parameters:"
1130 '
1140 LOCATE 6, 5: INPUT "Charge & mass of the particle "; CH, MASS: PRINT
1150  IF CH <> 0 AND MASS > 0 THEN 1170
1160  BEEP: LOCATE 6, 5: PRINT SPACE$(72): GOTO 1140
1170 LOCATE 8, 5: INPUT "Initial velocity of the particle (4)"; VI
1180  IF CH <> 0 AND MASS > 0 THEN 1200
1190  BEEP: LOCATE 8, 5: PRINT SPACE$(72): GOTO 1170
1200 LOCATE 10, 5: INPUT "Angle of shooting  of the particle (0)"; TH
1210  IF TH > -360 AND TH < 360 THEN 1230
1220  BEEP: LOCATE 10, 5: PRINT SPACE$(72): GOTO 1200
1230 LOCATE 12, 5: INPUT "Initial position x,y (-100 to 100)"; XI, YI
1240  IF ABS(XI) < 100 AND ABS(YI) < 100 THEN 1260
1250  BEEP: LOCATE 12, 5: PRINT SPACE$(72): GOTO 1230
1260 LOCATE 14, 5: INPUT "Strength of the magnetic field (0.5)"; MAG
1270  IF MAG > 0 THEN 1290
1280  BEEP: LOCATE 14, 5: PRINT SPACE$(72): GOTO 1260
1290 LOCATE 16, 5: INPUT "Maximum time "; TMAX
1300  IF TMAX > 0 THEN 1320
1310  BEEP: LOCATE 16, 5: PRINT SPACE$(72): GOTO 1290
1320 LOCATE 18, 5: INPUT "Time step size "; H
1330  IF H > 0 AND H <= 1 THEN 1350
1340  BEEP: LOCATE 18, 5: PRINT SPACE$(72): GOTO 1320
1350 '
1360                            'set parameters of the field etc. for graphics
1370 RS = 50                                  'radius of magnetic region fixed
1380 LSPEED = 1                                            'speed of light = 1
1390 '
1400 XE = 120: YE = 100                      'position of magnetic field fixed
1410 '
1420 RETURN
1430 '
2000 '************************************************************************
2010 'subroutine to find position of the particle using half-step method
2020 'input variables: CH, MAG, MASS, LSPEED, TH, PI, VI, XI, YI, TMAX, H, RS
2030 'output variables: T, KE, X, Y
2040 'global variables: none
2050 'local variables: PVX, PVY, CVX, CVY, VXH, VYH, AX, AY, R,
2060 '                 COUNT, MAGFAC
2070 '************************************************************************
2080 '
2090  MAGFAC = CH * MAG / (MASS * LSPEED)                       'useful factor
2100 '
2110 TH = TH * PI / 180                              'convert degree to radian
2120 CVX = VI * COS(TH): CVY = VI * SIN(TH)          'find velocity components
2130 '
2140 X = XI: Y = YI                                            'initialization
2150  T = 0
2160 COUNT = 0
2170 '
2180 WHILE T < TMAX
2190   VXH = CVX: VYH = CVY                          'roll velocity components
2200    R = SQR(X * X + Y * Y)                      'find distance from centre
2210   KE = .5 * MASS * (CVX * CVX + CVY * CVY)           'find Kinetic energy
2220    LOCATE 22, 1                        'print time & distance from centre
2230   PRINT USING "t = ###.##  x = +###.###, y = +###.### "; T; X; Y;
2240   PRINT USING "K.E. = ###.### "; KE
2250   PRINT USING "distance from centre of field = ###.###"; R
2260    GOSUB 4000                                            'draw trajectory
2270                                             'gain energy only between D's
2280   IF R > RS THEN 2330                        'particle lies outside field
2290   PVX = VXH + FNX(VYH) * H                             'predictor formula
2300   PVY = VYH + FNY(VXH) * H
2310    CVX = VXH + .5 * (FNX(VYH) + FNX(PVY)) * H          'corrector formula
2320    CVY = VYH + .5 * (FNY(VXH) + FNY(PVX)) * H
2330   X = X + .5 * (CVX + VXH) * H                          'distance formula
2340   Y = Y + .5 * (CVY + VYH) * H
2350    T = T + H                                              'increment time
2360  COUNT = COUNT + 1
2370 WEND
2380 '
2390 RETURN
2400 '
3000 '************************************************************************
3010 'subroutine to draw draw the magnetic field
3020 'input variables: RS, XE, YE,
3030 'output variables: X, Y
3040 'global variables: none
3050 'local variables: HC, XP, YP, ZP, XP0, YP0, ZP0
3060 '************************************************************************
3070 CLS
3080 '
3090 LOCATE 1, 5
3100 PRINT "Simulation of Trajectory of a charged Particle"
3110 PRINT "          inside a magnetic field"
3120  HC = 1                       'step size to draw circular magnetic region
3130 '
3140 FOR I = 0 TO 2 * RS                         'loop to draw magnetic region
3150   XP = -RS + I * HC + XE
3160   YP = YE + SQR(RS * RS - (XP - XE) ^ 2)
3170   ZP = YE - SQR(RS * RS - (XP - XE) ^ 2)
3180   XP = XP * 2.5                             'multiply by aspect ratio 2.5
3190   IF I = 0 THEN 3220                          'normalize the x & y scales
3200   LINE (XP0, YP0)-(XP, YP)
3210   LINE (XP0, ZP0)-(XP, ZP)
3220   XP0 = XP: YP0 = YP: ZP0 = ZP
3230 NEXT I
3240 '
3250 RETURN
3260 '
4000 '************************************************************************
4010 'subroutine to display trajectory
4020 'input variables: XE, YE, X, Y
4030 'output variables: none
4040 'global variables: none
4050 'local variables: XG, YG
4060 '************************************************************************
4070                                            'subroutine to draw trajectory
4080 YG = YE - Y                                         'for present position
4090 XG = XE + X: XG = XG * 2.5
4100 IF T > 0 THEN LINE -(XG, YG) ELSE PSET (XG, YG)          'draw trajectory
4110 '
4120 RETURN
4130 '
10000 '***********************************************************************
10010 'subroutine to display header screen
10020 'input variables:  none
10030 'output variables: none
10040 'global variables: none
10050 'local variables:  none
10060 '***********************************************************************
10070 CLS : KEY OFF                                           'clear screen
10080 SCREEN 2, 0, 0: WIDTH 80                               'change screen
10090  '
10100 LINE (1, 1)-(638, 40), , B                                'draw boxes
10110 LINE (5, 3)-(633, 37), , B
10120 LINE (70, 50)-(570, 140), , B
10130 LINE (75, 53)-(565, 137), , B
10140                                           'print title, authors, etc.
10150 LOCATE 3, 7
10160 PRINT " C O M P U T A T I O N A L   P H Y S I C S :";
10170 PRINT " (An  Introduction)"
10180 LOCATE 9, 40
10190 PRINT "by";
10200 LOCATE 11, 20: PRINT "R.C. Verma, P.K. Ahluwalia & K.C. Sharma"
10210 LOCATE 13, 34
10220 PRINT "   * * * *    "
10230 '
10240 BEEP
10250 LOCATE 15, 35: PRINT "* * * * * *"
10260 LOCATE 22, 25: PRINT "press any key to continue"
10270 LOCATE 23, 25: PRINT "-------------------------"
10280 '
10290 IF INKEY$ = "" THEN 10290                            'hold the screen
10300 CLS                                                     'clear screen
10310 RETURN
10320 '

