10 '**************************************************************************
20 'Chapter 7, Example 3: Simulated Motion of a charged particle in cyclotron
30 'COMPUTATIONAL PHYSICS: An Introduction
40 'by R. C. Verma, P. K. Ahluwalia and K. C. Sharma
50 '   * * * *   
60 '**************************************************************************
70 '
80 GOSUB 10000                                                    'show header
90 '
100 PI = 3.1415926#                                        'define constant pi
110                              'define functions for acceleration components
120 DEF FNX (V, T) = MAGFAC * V + AMP * SIN(WF * T)
130 DEF FNY (V, T) = -MAGFAC * V
140 FLAG = 0                                                         'set flag
150 '
160 GOSUB 1000                'introduce the problem & to get input parameters
170 GOSUB 3000                                  'draw the D's of the cyclotron
180 GOSUB 5000                                                  'draw the axes
190 GOSUB 2000                         'find position of particle in cyclotron
200 '
210 LOCATE 22, 1
220 IF INKEY$ = "" THEN 220
230 '
240 CLS : LOCATE 12, 15: BEEP: PRINT " Thank you !"
250 END
260 '
1000 '************************************************************************
1010 'subroutine to introduce the problem & to get input variables
1020 'input variables:  none
1030 'output variables: CH, MASS, MAG, TH, VI, XI, YI, EL, WF, EYMIN, EYMAX,
1040 '                  TMIN, TMAX, RS, WID, LSPEED, XE, YE, VMAX, H
1050 'global variables: none
1060 'local variables:  none
1070 '
1080 'test case : RS=50, XI=0,YI=0, VI=0 , TH=0  H=0.05, TMAX= 100
1090 '            EL. FIELD =0.25, MAG FIELD = 0.5 : W =0.5
1100 'test case : RS=50, XI=0,YI=0, VI=5 , TH=0  H=0.01, TMAX= 100
1110 '            EL. FIELD =0, MAG FIELD = 1.0 : W =1.0
1120 'test case : RS=50, XI=0,YI=0, VI=0 , TH=0  H=0.5, TMAX= 500
1130 '            EL. FIELD =0.30, MAG FIELD = 0.5 : W =0.5
1140 'test case : RS=50, XI=0,YI=0, VI=0 , TH=0  H=0.2, TMAX= 300
1150 '            CH=1, MASS=1, EL. FIELD =0.30, MAG FIELD = 0.3 : W =0.3     
1160 '************************************************************************
1170 '
1180 LOCATE 1, 10: PRINT "This program describes motion of a charged particle"
1190 LOCATE 2, 15: PRINT "within a cyclotron"
1200 LOCATE 3, 10: PRINT "-------------------------------------------------"
1210 LOCATE 4, 5: PRINT "Give the following parameters:"
1220 '
1230 LOCATE 6, 5: INPUT "Charge & mass of the particle"; CH, MASS: PRINT
1240  IF CH <> 0 AND MASS > 0 THEN 1260
1250  BEEP: LOCATE 6, 5: PRINT SPACE$(72): GOTO 1230
1260 LOCATE 8, 5: INPUT "Initial velocity of the particle "; VI
1270  IF VI < 10 THEN 1290
1280  BEEP: LOCATE 8, 5: PRINT SPACE$(72): GOTO 1260
1290 LOCATE 10, 5: INPUT "Angle of shooting  of the particle "; TH
1300  IF TH > -360 AND TH < 360 THEN 1320
1310  BEEP: LOCATE 10, 5: PRINT SPACE$(72): GOTO 1290
1320 LOCATE 14, 5: INPUT "Strength of the magnetic field "; MAG
1330  IF MAG > 0 THEN 1350
1340  BEEP: LOCATE 14, 5: PRINT SPACE$(72): GOTO 1320
1350 LOCATE 16, 8: INPUT "Strength & freq. of the Electric Field"; EL, WF
1355  PRINT
1360  IF EL > 0 AND WF > 0 THEN 1380
1370  BEEP: LOCATE 16, 5: PRINT SPACE$(72): GOTO 1350
1380 LOCATE 18, 5: INPUT "Maximum time "; TMAX
1390  IF TMAX > 0 THEN 1410
1400  BEEP: LOCATE 18, 5: PRINT SPACE$(72): GOTO 1380
1410 LOCATE 20, 5: INPUT "Time step size "; H
1420  IF H > 0 AND H <= 3 THEN 1440
1430  BEEP: LOCATE 20, 5: PRINT SPACE$(72): GOTO 1410
1440 '
1450                         'set parameters of cyclotron & initial conditions
1460                                             'for calculation and graphics
1470 TMIN = 0                                                    'initial time
1480 XI = 0: YI = 0                                        'position of source
1490 LSPEED = 1                                            'speed of light = 1
1500 '
1510 RS = 50                                                    'radius of D's
1520 WID = 8                                             'distance between D's
1530 XE = 60: YE = 100                              'centre of cyclotron fixed
1540 '
1550 EYMIN = .5 * MASS * VI * VI
1560 VMAX = CH * MAG * RS / (LSPEED * MASS): EYMAX = .5 * MASS * VMAX * VMAX
1570                                           'set energy limits for graphics
1580 RETURN
1590 '
2000 '************************************************************************
2010 'subroutine to do numerical work
2020 'input variables: CH, MAG, MASS, LSPEED, TH, PI, VI, XI, YI, EYMIN,  FLAG
2030 'output variables: X, Y, T, X0, Y0, KE,
2040 'global variables: none
2050 'local variables: MAGFAC, PVX, PVY, CVX, CVY, VXH, VYH, AX, AY, R
2060 '                 UX, UY, VX, VY,
2070 '************************************************************************
2080 '
2090 MAGFAC = CH * MAG / (MASS * LSPEED): EF = CH * EL / MASS
2100 '                                         'useful factors for calculation
2110 TH = TH * PI / 180                              'convert degree to radian
2120 UX = VI * COS(TH): UY = VI * SIN(TH)            'find velocity components
2130 VX = UX: VY = UY                                     'initialize velocity
2140  CVX = UX: CVY = UY
2150                                                      'initialize position
2160 X = XI: Y = YI
2170 X0 = X: Y0 = Y
2180  T = 0                                                   'initialize time
2190 KE = EYMIN                                               'initialize K.E.
2200                                                       'motion starts here
2210 WHILE (T < TMAX AND Y > -2 * RS)
2220 '
2230   GOSUB 4000                                             'draw trajectory
2240   GOSUB 6000                                           'draw energy graph
2250 '
2260   VXH = CVX: VYH = CVY                          'roll velocity components
2270   X0 = X: Y0 = Y                                           'roll position
2280 '
2290   R = SQR(X * X + Y * Y)                       'find distance from centre
2300   KE = .5 * MASS * (CVX * CVX + CVY * CVY)                     'find K.E.
2310                                        'print time & distance from centre
2320   LOCATE 22, 1
2330   PRINT USING "t = ####.##  r = ###.##,  K.E. = ####.### "; T; R; KE
2340                                                       'check for ejection
2350   IF X < (RS - 4) THEN 2410
2360   IF FLAG = 1 THEN 2470                                   'beep only once
2370   BEEP: LOCATE 20, 60: PRINT "ejection starts"
2380   CVX = 0: FLAG = 1                     'fix x-components for getting out
2390   GOTO 2470                                           'avoid acceleration
2400                                             'gain energy only between D's
2410    IF X <= .5 * WID AND X >= -.5 * WID THEN AMP = EF ELSE AMP = 0
2420   PVX = VXH + FNX(VYH, T) * H                          'predictor formula
2430   PVY = VYH + FNY(VXH, T) * H
2440    CVX = VXH + .5 * (FNX(VYH, T) + FNX(PVY, T)) * H    'corrector formula
2450    CVY = VYH + .5 * (FNY(VXH, T) + FNY(PVX, T)) * H
2460   X = X0 + .5 * (CVX + VXH) * H                         'distance formula
2470   Y = Y0 + .5 * (CVY + VYH) * H
2480    T = T + H                                              'increment time
2490 WEND
2500 '
2510 RETURN
2520 '
3000 '************************************************************************
3010 'subroutine to draw cyclotron D's
3020 'input variables: RS, XE, YE, WID
3030 'output variables: none
3040 'global variables: none
3050 'local variables: HC, XP, YP, ZP, XP0, YP0, ZP0,.
3060 '************************************************************************
3070 CLS
3080 LOCATE 1, 20: PRINT "Simulation of Trajectory of a charged Particle"
3090  HC = 1                                       'step for drawing D's fixed
3100 '
3110 FOR I% = 0 TO 2 * RS                                'loop for drawing D's
3120   XP = -RS + I% * HC + XE
3130   YP = YE + SQR(RS * RS - (XP - XE) ^ 2)
3140   ZP = YE - SQR(RS * RS - (XP - XE) ^ 2)
3150    XP = XP * 2.5                            'multiply by aspect ratio 2.5
3160    IF I% = 0 THEN 3190                        'normalize the x & y scales
3170   LINE (XP0, YP0)-(XP, YP)
3180   LINE (XP0, ZP0)-(XP, ZP)
3190    XP0 = XP: YP0 = YP: ZP0 = ZP
3200 NEXT I%
3210 '
3220 LINE ((XE - .5 * WID) * 2.5, YE + RS)-((XE + .5 * WID) * 2.5, YE - RS), , B
3230 LINE ((XE - .5 * WID + 1) * 2.5, YE + RS + 5)-((XE + .5 * WID - 1) * 2.5, YE - RS - 5), 0, BF
3240 LINE ((XE + RS) * 2.5, YE)-((XE + RS) * 2.5, YE + RS)
3250 LINE ((XE + RS - 5) * 2.5, YE + 20)-((XE + RS - 5) * 2.5, YE + RS)
3260 LINE ((XE + RS - 5) * 2.5 + 1, YE)-((XE + RS) * 2.5 - 1, YE + RS), 0, BF
3270 '
3280 RETURN
3290 '
4000 '************************************************************************
4010 'subroutine to display graph
4020 'input variables: XE, YE, X0, Y0
4030 'output variables: none
4040 'global variables: none
4050 'local variables: X1, Y1, X2, Y2
4060 '************************************************************************
4070 '
4080 Y1 = YE - Y0
4090 X1 = XE + X0: X1 = X1 * 2.5                          '2.5 is aspect ratio
4100 Y2 = YE - Y
4110 X2 = XE + X: X2 = X2 * 2.5
4120 LINE (X1, Y1)-(X2, Y2)                                   'draw trajectory
4130 '
4140 RETURN
4150 '
5000 '************************************************************************
5010 'subroutine to draw axes of the graph
5020 'input variables: XMAX.XMIN,YMIN,YMAX, TMIN, TMAX, EYMIN, EYMAX
5030 'output variable: XSCALE, YSCALE, XBEG1%, YBEG1%, XI1%, YI1%
5040 'local variables: XPT, YPT, XL1%. YL1%, XM1%, YM1%,
5050 '                 XH1%. YH1%, XD1%, YD1%,
5060 'global variable: none
5070 '************************************************************************
5080 '
5090 XSCALE = 80 / (TMAX - TMIN)                       'scales for x- & y-axes
5100 YSCALE = 80 / (EYMAX - EYMIN)
5110 XSCALE = XSCALE * 2.5                                '2.5 is aspect ratio
5120 '
5130 LOCATE 2, 40: PRINT "Kinetic Energy vs.  cycle"
5140 XBEG1% = 364: YBEG1% = 16
5150 XL1% = (TMIN - TMIN) * XSCALE + XBEG1%
5160 YM1% = (EYMAX - EYMIN) * YSCALE + YBEG1%
5170 XM1% = (TMAX - TMIN) * XSCALE + XBEG1%
5180 YL1% = (EYMAX - EYMAX) * YSCALE + YBEG1%
5190 '
5200 LINE (XL1%, YL1%)-(XM1%, YM1%), , B                        'draw the axes
5210 XH1% = .5 * (XL1% + XM1%): XD1% = (XM1% - XL1%) / 4
5220 YH1% = .5 * (YL1% + YM1%): YD1% = (YM1% - YL1%) / 4
5230 '
5240 FOR J% = 0 TO 4                                              'mark y-axis
5250   YPT = J% * YD1% + YL1%
5260   LINE (XL1% - 4, YPT)-(XL1%, YPT)
5270   IF J% / 2 <> INT(J% / 2) THEN 5300
5280   K% = YPT / 8 + 1                                    '8 for pixels ratio
5290   LOCATE K%, 41
5295   PRINT USING "###.# "; (4 - J%) * (EYMAX - EYMIN) / 4 + EYMIN
5300 NEXT J%
5310 '
5320 LOCATE YH1% / 8 - 1, 38: PRINT "^"
5330 LOCATE YH1% / 8, 38: PRINT "|"                              'label y-axis
5340 LOCATE YH1% / 8 + 1, 38: PRINT "KE"
5350 '
5360 FOR J% = 0 TO 4                                              'mark x-axis
5370   XPT = J% * XD1% + XL1%
5380   LINE (XPT, YM1%)-(XPT, YM1% + 4)
5390   IF J% / 2 <> INT(J% / 2) THEN 5420
5400   K% = XPT / 8
5410   LOCATE YM1% / 8 + 2, K% - 3
5415   PRINT USING "#####"; J% * (TMAX - TMIN) / 4 + TMIN
5420 NEXT J%
5430 LOCATE YM1% / 8 + 3, XH1% / 8: PRINT "t -> "                'label x-axis
5440 '
5450 XI1% = XH1%: YI1% = YH1%
5460 '
5470 RETURN
5480 '
6000 '************************************************************************
6010 'subroutine to graph K.E. vs. time
6020 'input variables: T, TMIN, EYMAX, KE, XBEG1%, YBEG1%, XSCALE, YSCALE
6030 'output variable: XI%, YI%
6040 'global variables: none
6050 'loCal variable: X%, Y%
6060 '************************************************************************
6070                                     'scale transformation for x- & y-axes
6080 X% = (T - TMIN) * XSCALE + XBEG1%
6090 Y% = (EYMAX - KE) * YSCALE + YBEG1%
6100 '
6110 IF T > 0 THEN LINE (XI%, YI%)-(X%, Y%)              'join with last point
6120 XI% = X%: YI% = Y%
6130 '
6140 RETURN
6150 '
10000 '***********************************************************************
10010 'subroutine to display header screen
10020 'input variables:  none
10030 'output variables: none
10040 'global variables: none
10050 'local variables:  none
10060 '***********************************************************************
10070 CLS : KEY OFF                                           'clear screen
10080 SCREEN 2, 0, 0: WIDTH 80                               'change screen
10090  '
10100 LINE (1, 1)-(638, 40), , B                                'draw boxes
10110 LINE (5, 3)-(633, 37), , B
10120 LINE (70, 50)-(570, 140), , B
10130 LINE (75, 53)-(565, 137), , B
10140                                           'print title, authors, etc.
10150 LOCATE 3, 7
10160 PRINT " C O M P U T A T I O N A L   P H Y S I C S :";
10170 PRINT " (An  Introduction)"
10180 LOCATE 9, 40
10190 PRINT "by";
10200 LOCATE 11, 20: PRINT "R.C. Verma, P.K. Ahluwalia & K.C. Sharma"
10210 LOCATE 13, 34
10220 PRINT "   * * * *    "
10230 '
10240 BEEP
10250 LOCATE 15, 35: PRINT "* * * * * *"
10260 LOCATE 22, 25: PRINT "press any key to continue"
10270 LOCATE 23, 25: PRINT "-------------------------"
10280 '
10290 IF INKEY$ = "" THEN 10290                            'hold the screen
10300 CLS                                                     'clear screen
10310 RETURN
10320 '

