10 '**************************************************************************
20 'Chapter 8, Example 2: Circuit Analysis using Kirchoff's laws
30 'COMPUTATIONAL PHYSICS: An Introduction
40 'by R. C. Verma, P. K. Ahluwalia and K. C. Sharma
50 '   * * * *   
60 '**************************************************************************
70 '
80 GOSUB 10000                                                    'show header
90 '
100 GOSUB 1000                   'introduce the problem & get input parameters
110 GOSUB 2000                                              'solve eigenmatrix
120 '
130 LOCATE 22, 1: PRINT "press any key to exit."
140 IF INKEY$ = "" THEN 140
150 '
160 CLS : LOCATE 12, 15: BEEP: PRINT " Thank you !"
170 END
180 '
1000 '************************************************************************
1010 'subroutine to introduce the problem & to get input variables
1020 'input variables:  none
1030 'output variables: DIMMAT%, ORMAT(I,J), RHS(I)
1040 'global variables: none
1050 'local variables: ROW%, COL%
1060 '************************************************************************
1070 '
1080 LOCATE 1, 10: PRINT "This program finds current elements in a circuit"
1090 LOCATE 2, 15: PRINT "using Kirchoff law"
1100 LOCATE 3, 10: PRINT "-------------------------------------------------"
1110 '
1120 LOCATE 5, 10: PRINT SPACE$(70)
1125 LOCATE 5, 10: INPUT "Give no. of current elements = "; DIMMAT%: PRINT
1130  IF DIMMAT% > 0 THEN 1140
1135  BEEP: GOTO 1120
1140 DIM ORMAT(DIMMAT%, DIMMAT%), INMAT(DIMMAT%, DIMMAT%)
1145 DIM C(DIMMAT%), RHS(DIMMAT%)
1150                                    'set dimensions of the array variables
1160 PRINT "Give coefficients in the equations ": PRINT
1170 '
1180 FOR ROW% = 1 TO DIMMAT%                                'loop for each row
1190   PRINT "Equation no. = "; ROW%: PRINT
1200   FOR COL% = 1 TO DIMMAT%                               'loop over column
1210     PRINT "Give coefficients ("; ROW%; ","; COL%; ")"
1220     INPUT ORMAT(ROW%, COL%)
1230   NEXT COL%
1240   PRINT
1250 NEXT ROW%
1260 '
1270  PRINT "Give r.h.s. values in the equations ": PRINT
1280 FOR ROW% = 1 TO DIMMAT%
1290    PRINT "Equation no. = "; ROW%: PRINT
1300    INPUT RHS(ROW%)
1310 NEXT ROW%
1320 '
1330 CLS
1340 LOCATE 2, 10: BEEP
1350 PRINT "Please check that equations given are right!": PRINT
1360 '
1370 FOR ROW% = 1 TO DIMMAT%
1380   FOR COL% = 1 TO DIMMAT%
1390     IF ORMAT(ROW%, COL%) < 0 THEN 1410
1400     PRINT "+"; ORMAT(ROW%, COL%); "i("; ROW%; ")"; SPC(3); : GOTO 1420
1410     PRINT ORMAT(ROW%, COL%); "i("; ROW%; ")"; SPC(1);
1420   NEXT COL%
1430   PRINT " = "; RHS(ROW%)
1440   PRINT
1450 NEXT ROW%
1460 '
1470 RETURN
1480 '
2000 '************************************************************************
2010 'subroutine to solve simultaneous equations
2020 'input variables: DIMMAT%, ORMAT(I%,J%), RHS(I%)
2030 'output variables: INMAT( , )
2040 'global variables: none
2050 'local variables: ROW%, COL%, S,
2060 '************************************************************************
2070                                         'initialize with identity matrix
2080 FOR ROW% = 1 TO DIMMAT%
2090  INMAT(ROW%, ROW%) = 1
2100 NEXT ROW%
2110 '
2120  FOR S% = 1 TO DIMMAT%
2130   GOSUB 3000
2140 NEXT S%
2150 '
2160 PRINT
2170 '
2180   PRINT
2190 FOR ROW% = 1 TO DIMMAT%
2200   SUM = 0                                                 'initialize sum
2210   FOR COL% = 1 TO DIMMAT%
2220     SUM = SUM + INMAT(ROW%, COL%) * RHS(COL%)
2230   NEXT COL%
2240   PRINT USING "current element i(##) = ###.###"; ROW%; SUM: PRINT
2250 NEXT ROW%
2260 '
2270 RETURN
2280 '
3000 '************************************************************************
3010 'subroutine to find inverse of L.H.S. matrix
3020 'input variables: DIMMAT%, ORMAT(I%, J%)
3030 'output variables: INMAT(I%,J%)
3040 'global variables: none
3050 'local variables: J%, K%, S%, C(J%), D, M, X, Y
3060 '************************************************************************
3070 '
3080 FOR J% = S% TO DIMMAT%
3090  C(J%) = ABS(ORMAT(J%, S%))                   'identify vector components
3100 NEXT J%
3110 '
3120 J% = S%
3130 FOR K% = J% TO DIMMAT%
3140  IF C(J%) < C(K%) THEN 3190
3150 NEXT K%
3160 '
3170 GOTO 3220
3180 '
3190 J% = K%
3200 GOTO 3130
3210 '
3220 IF C(J%) = 0 THEN 3470                         'if inverse does not exist
3230 '
3240 FOR K% = 1 TO DIMMAT%
3250  X = ORMAT(S%, K%): Y = INMAT(S%, K%)
3260  ORMAT(S%, K%) = ORMAT(J%, K%): INMAT(S%, K%) = INMAT(J%, K%)
3270  ORMAT(J%, K%) = X: INMAT(J%, K%) = Y
3280 NEXT K%
3290 '
3300 D = ORMAT(S%, S%)
3310 '
3320 FOR J% = 1 TO DIMMAT%
3330  ORMAT(S%, J%) = ORMAT(S%, J%) / D
3340  INMAT(S%, J%) = INMAT(S%, J%) / D
3350 NEXT J%
3360 '
3370 FOR J% = 1 TO DIMMAT%: IF J% = S% THEN 3430
3380   M = ORMAT(J%, S%)
3390   FOR K% = 1 TO DIMMAT%
3400     ORMAT(J%, K%) = ORMAT(J%, K%) - M * ORMAT(S%, K%)
3410     INMAT(J%, K%) = INMAT(J%, K%) - M * INMAT(S%, K%)
3420   NEXT K%
3430 NEXT J%
3440 '
3450 GOTO 3490
3460 '
3470 LOCATE 22, 10: BEEP: PRINT "Inverse matrix does not exist!": END
3480 '
3490 RETURN
3500 '
10000 '***********************************************************************
10010 'subroutine to display header screen
10020 'input variables:  none
10030 'output variables: none
10040 'global variables: none
10050 'local variables:  none
10060 '***********************************************************************
10070 CLS : KEY OFF                                           'clear screen
10080 SCREEN 2, 0, 0: WIDTH 80                               'change screen
10090  '
10100 LINE (1, 1)-(638, 40), , B                                'draw boxes
10110 LINE (5, 3)-(633, 37), , B
10120 LINE (70, 50)-(570, 140), , B
10130 LINE (75, 53)-(565, 137), , B
10140                                           'print title, authors, etc.
10150 LOCATE 3, 7
10160 PRINT " C O M P U T A T I O N A L   P H Y S I C S :";
10170 PRINT " (An  Introduction)"
10180 LOCATE 9, 40
10190 PRINT "by";
10200 LOCATE 11, 20: PRINT "R.C. Verma, P.K. Ahluwalia & K.C. Sharma"
10210 LOCATE 13, 34
10220 PRINT "   * * * *    "
10230 '
10240 BEEP
10250 LOCATE 15, 35: PRINT "* * * * * *"
10260 LOCATE 22, 25: PRINT "press any key to continue"
10270 LOCATE 23, 25: PRINT "-------------------------"
10280 '
10290 IF INKEY$ = "" THEN 10290                            'hold the screen
10300 CLS                                                     'clear screen
10310 RETURN
10320 '

