10 '**************************************************************************
20 'Chapter 8, Example 3: Voltage Divider Circuit
30 'COMPUTATIONAL PHYSICS: An Introduction
40 'by R. C. Verma, P. K. Ahluwalia and K. C. Sharma
50 '   * * * *    
60 '**************************************************************************
70 '
80 GOSUB 10000                                                    'show header
90 '
100 PI = 3.1415926#                                        'define constant pi
110                                                          'define functions
120 DEF FNI (W, L, C) = W * L - 1 / (W * C)
130 DEF FNRE (R1, R2, L1, L2) = (R2 * (R1 + R2) + L2 * (L1 + L2)) / ((R1 + R2) ^ 2 + (L1 + L2) ^ 2)
140    'real part of total impedance
150 DEF FNIM (R1, R2, L1, L2) = (R1 * L2 - R2 * L1) / ((R1 + R2) ^ 2 + (L1 + L2) ^ 2)
160    'imaginary part of total impedance
170 '
180 GOSUB 1000                   'introduce the problem & get input parameters
190 GOSUB 2000                         'find output voltage & Phase difference
200 '
210 LOCATE 22, 1: PRINT "press any key to exit."
220 IF INKEY$ = "" THEN 220
230 '
240 CLS : LOCATE 12, 15: BEEP: PRINT " Thank you !"
250 END
260 '
1000 '************************************************************************
1010 'subroutine to introduce the problem & to get input variables
1020 'input variables:  none
1030 'output variables: L1, C1, R1, L2, C2, R2, VIN, FREQ
1040 'global variables: none
1050 'local variables:  none
1060 'test cases:L1=1, C1=1, R1=1, L2=1.5, C2=2, R2=3,  V=10, freq.=50
1070 '************************************************************************
1080 '
1090 LOCATE 2, 10: PRINT "This program studies a Voltage Divider Circuit"
1100 LOCATE 3, 10: PRINT "              involving 2 impedances"
1110 LOCATE 4, 10: PRINT "----------------------------------------------"
1120 '
1125 LOCATE 6, 10: PRINT SPACE$(70)
1130 LOCATE 6, 10: INPUT "Give R1, L1, C1 "; R1, L1, C1
1135  IF R1 >= 0 AND L1 >= 0 AND C1 >= 0 AND (R1 + L1 + C1) <> 0 THEN 1145
1140  BEEP: GOTO 1125
1145 LOCATE 8, 10: PRINT SPACE$(70)
1150 LOCATE 8, 10: INPUT "Give R2, L2, C2 "; R2, L2, C2
1155  IF R2 >= 0 AND L2 >= 0 AND C2 >= 0 AND (R2 + L2 + C2) <> 0 THEN 1165
1160  BEEP: GOTO 1145
1165 LOCATE 10, 10: PRINT SPACE$(70)
1170 LOCATE 10, 10: INPUT "Give amplitude & frequency of voltage"; VIN, FREQ
1180  IF VIN <> 0 AND FREQ > 0 THEN 1200
1190  BEEP: LOCATE 10, 1: PRINT SPACE$(75): GOTO 1165
1200 '
1210 RETURN
1220 '
2000 '************************************************************************
2010 'subroutine to do numerical work
2020 'input variables: L1, C1, R1, L2, C2, R2, VIN, FREQ
2030 'output variables: PHASE, VOUT,  REACT1, REACT2,
2040 'global variables: PI
2050 'local variables: QI, QR, WF
2060 '************************************************************************
2070 '
2080 WF = 2 * PI * FREQ                          'convert to angular frequency
2090
2100 IF C1 > 0 THEN REACT1 = FNI(WF, L1, C1) ELSE REACT1 = WF * L1
2105                                               'reactance in 1st impedance
2110 IF C2 > 0 THEN REACT2 = FNI(WF, L2, C2) ELSE REACT2 = WF * L2
2125                                               'reactance in 1st impedance
2130 QR = FNRE(R1, R2, L1, L2)                   'real part of total impedance
2140 QI = FNIM(R1, R2, L1, L2)              'imaginary part of total impedance
2150 '
2160 VOUT = VIN * (QR * QR + QL * QL) ^ .5                'find output voltage
2170                     'find phase difference between output & input voltage
2175 IF QR = 0 THEN PHASE = 90: GOTO 2200             'if denominator vanishes
2180 PHASE = ATN(QI / QR)
2190 PHASE = PHASE * 180 / PI                              'convert to degrees
2200 '
2210 BEEP
2220 LOCATE 12, 10
2230  PRINT USING "Reactance of 1st impedance +###.###"; REACT1
2240 LOCATE 14, 10
2250  PRINT USING "Reactance of 2nd impedance +###.###"; REACT2
2260 LOCATE 16, 10
2270  PRINT USING "Output Voltage = +###.###"; VOUT
2280 LOCATE 18, 10
2290  PRINT USING "Phase difference = +###.### degree"; PHASE
2300 '
2310 RETURN
2320 '
10000 '***********************************************************************
10010 'subroutine to display header screen
10020 'input variables:  none
10030 'output variables: none
10040 'global variables: none
10050 'local variables:  none
10060 '***********************************************************************
10070 CLS : KEY OFF                                           'clear screen
10080 SCREEN 2, 0, 0: WIDTH 80                               'change screen
10090  '
10100 LINE (1, 1)-(638, 40), , B                                'draw boxes
10110 LINE (5, 3)-(633, 37), , B
10120 LINE (70, 50)-(570, 140), , B
10130 LINE (75, 53)-(565, 137), , B
10140                                           'print title, authors, etc.
10150 LOCATE 3, 7
10160 PRINT " C O M P U T A T I O N A L   P H Y S I C S :";
10170 PRINT " (An  Introduction)"
10180 LOCATE 9, 40
10190 PRINT "by";
10200 LOCATE 11, 20: PRINT "R.C. Verma, P.K. Ahluwalia & K.C. Sharma"
10210 LOCATE 13, 34
10220 PRINT "   * * * *    "
10230 '
10240 BEEP
10250 LOCATE 15, 35: PRINT "* * * * * *"
10260 LOCATE 22, 25: PRINT "press any key to continue"
10270 LOCATE 23, 25: PRINT "-------------------------"
10280 '
10290 IF INKEY$ = "" THEN 10290                            'hold the screen
10300 CLS                                                     'clear screen
10310 RETURN
10320 '

