10 '**************************************************************************
20 'Chapter 8, Example 4: Nonlinearity in Electrical Circuits
25 '                      (Rectifier's Output)
30 'COMPUTATIONAL PHYSICS: An Introduction
40 'by R. C. Verma, P. K. Ahluwalia and K. C. Sharma
50 '   * * * *   
60 '**************************************************************************
70 '
80 GOSUB 10000                                                    'show header
90 '
100 PI = 3.1415926#                                        'define constant pi
110 '
120 DEF FNF (X, T) = AMPD * SIN(KVEC * X - WFRQ * T) 'define function for wave
130 DEF FNNON (X) = (X + EPSI * X ^ POW) / (1 + EPSI)        'and nonlinearity
140 '
150 GOSUB 1000                   'introduce the problem & get input parameters
160 GOSUB 3000                                                    'find scales
170 GOSUB 2000              'find and draw amplitudes for input & output waves
180 '
190 LOCATE 22, 1
200 IF INKEY$ = "" THEN 200
210 '
220 CLS : LOCATE 12, 15: BEEP: PRINT " Thank you !"
230 '
240 END
250 '
1000 '************************************************************************
1010 'subroutine to introduce the problem & to get input variables
1020 'input variables:  none
1030 'output variables: NWAVE%, EPSI, POW, AMP(I%), KVE(I%), WFR(I%), T, H, HT
1040 '                  TMIN, TMAX, XMIN, XMAX, YMIN, YMAX, ZMIN, ZMAX,
1050 'global variables: none
1060 'local variables:  none
1070 'test cases: epsilon = 0.1, p=2, no. of wanes=2,
1080 '            a1=1, k1=1, f=1, a2=1.5, k2=1.1, f2=1.2
1090 '************************************************************************
1100 '
1110 LOCATE 1, 10
1115  PRINT "This program demonstrate output of a nonlinear system"
1120 LOCATE 2, 10
1125  PRINT "------------------------------------------------------"
1130 LOCATE 4, 5: PRINT "Please give the following parameters:"
1140 '
1150 LOCATE 6, 5: INPUT "No. of waves (even no.) = "; NWAVE%
1160  IF NWAVE% > 0 AND (NWAVE% MOD 2 = 0) THEN 1180
1170  BEEP: LOCATE 6, 5: PRINT SPACE$(72): GOTO 1150
1180 LOCATE 8, 5: INPUT "Nonlinearity Parameters: epsilon, power"; EPSI, POW
1190  IF EPSI <> 0 AND POW <> 0 THEN 1200
1195  BEEP: LOCATE 8, 5: PRINT SPACE$(72): GOTO 1180
1200 '
1205 XMIN = 0: T = 0                            'initial position & time fixed
1210 LOCATE 10, 5: PRINT "initial position & time are set: Xmin = 0, Tmin = 0"
1215 LOCATE 12, 5: INPUT "Give Xmax, Tmax "; XMAX, TMAX
1220   IF XMAX > 0 AND TMAX > 0 THEN 1230
1225   BEEP: LOCATE 12, 5: PRINT SPACE$(72): GOTO 1215
1230 '
1235 LOCATE 14, 5: PRINT "Give amplitude, wave vector, & angular frequency "
1240 '
1245 FOR I% = 1 TO NWAVE%
1250   LOCATE 15, 5: PRINT SPACE$(72)
1260   LOCATE 15, 5: PRINT " for wave no. "; I%
1265   LOCATE 16, 5: PRINT SPACE$(72)
1270   LOCATE 16, 5: INPUT AMP(I%), KVE(I%), WFR(I%)
1280    IF AMP(I%) > 0 AND KVE(I%) > 0 AND WFR(I%) > 0 THEN 1300
1290    BEEP: GOTO 1265
1300 NEXT I%
1310 '
1330 H = .1: HT = .5                                         'step sizes fixed
1340 '
1350   YMAX = 0: ZMAX = 0
1360 FOR I% = 1 TO NWAVE%                             'set limits for graphics
1370   YMAX = YMAX + AMP(I%): ZMAX = ZMAX + FNNON(AMP(I%))
1380 NEXT I%
1390 '
1400  YMIN = -YMAX: ZMIN = -ZMAX
1410 '
1420 RETURN
1430 '
2000 '************************************************************************
2010 'subroutine to do find output wave
2020 'input variables: NWAVE%, EPSI, POW, AMP(I), KVE(I%), WFR(I%)
2030 '                 TMIN, TMAX, XMIN, XMAX, H, HT
2040 'output variables: none
2050 'global variables: none
2060 'local variables: T, X, SUMY, SUMZ, AMPIN, AMPOUT,
2070 '************************************************************************
2080 T = TMIN                                                 'initialize time
2090 '
2100 WHILE T < TMAX                                            'loop over time
2110   LOCATE 4, 70: PRINT USING "t = ##.##"; T            'print current time
2120   X = XMIN                                           'initialize position
2130    GOSUB 4070                                                'clear boxes
2140   WHILE X < XMAX                                  'loop over space points
2150     SUMY = 0: SUMZ = 0                                   'initialize sums
2160     FOR I% = 1 TO NWAVE%                                 'loop over waves
2170       AMPD = AMP(I%): KVEC = KVE(I%): WFRQ = WFR(I%) 'get wave parameters
2180       AMPIN = FNF(X, T)                      'find amplitude of each wave
2190       SUMY = SUMY + AMPIN                           'update amplitude sum
2200       AMPOUT = FNNON(AMPIN)                             'add nonlinearity
2210       SUMZ = SUMZ + AMPOUT              'update the sum with nonlinearity
2220     NEXT I%
2230     Z = SUMZ                                  'assign values for graphics
2240     Y = SUMY
2250      GOSUB 5070                                'draw input & output waves
2260     X = X + H                                          'next space points
2270  WEND
2280  T = T + HT                                               'increment time
2290 WEND
2300 '
2310 RETURN
2320 '
3000 '************************************************************************
3010 'subroutine to set scales
3020 'input variables: AMP(I), XMIN, XMAX, YMIN, YMAX, ZMIN ZMAX
3030 'output variables: XSCALE, YSCALE, ZSCALE
3040 'global variables: none
3050 'local variables: none
3060 '************************************************************************
3070 '
3080 'subroutine to draw axes of the graph
3090 '
3100 XSCALE = 480 / (XMAX - XMIN)               'find scales for x, y & z-axes
3110 YSCALE = 80 / (YMAX - YMIN)
3120 ZSCALE = 80 / (ZMAX - ZMIN)
3130 CLS
3140 LOCATE 1, 10: PRINT "Nonlinear effects on incoming wave"
3150 LINE (64, 56)-(544, 56)                             'draw upper base line
3160 LINE (64, 144)-(544, 144)                           'draw lower base line
3170 '
3180 RETURN
3190 '
4000 '************************************************************************
4010 'subroutine to clear the lower box
4020 'input variables:  none
4030 'output variables: none
4040 'global variables: none
4050 'local variables: none
4060 '************************************************************************
4070 '
4080 LINE (64, 16)-(544, 55), 0, BF
4090 LINE (64, 57)-(544, 143), 0, BF
4100 LINE (64, 145)-(544, 184), 0, BF
4110 '
4120 RETURN
4130 '
5000 '************************************************************************
5010 'subroutine to draw incoming and outgoing waves
5020 'input variables:  X, Y, Z, XMIN, YMAX, ZMAX, XSCALE, YSCALE, ZSCALE,
5030 'output variables: none
5040 'global variables: none
5050 'local variables:  X1%, Y1%, Z1%, X2%, Y2%, Z2%
5060 '************************************************************************
5070                                                            'incoming wave
5080 X2% = X * XSCALE + 64                       'perform scale transformation
5090 Y2% = (YMAX - Y) * YSCALE + 16
5100 IF X > XMIN THEN LINE (X1%, Y1%)-(X2%, Y2%)
5110                                            'output wave with nonlinearity
5120 Z2% = (ZMAX - Z) * ZSCALE + 104             'perform scale transformation
5130 IF X > XMIN THEN LINE (X1%, Z1%)-(X2%, Z2%)
5140 '
5150 X1% = X2%: Z1% = Z2%: Y1% = Y2%                          'roll the values
5160 '
5170 RETURN
5180 '
10000 '***********************************************************************
10010 'subroutine to display header screen
10020 'input variables:  none
10030 'output variables: none
10040 'global variables: none
10050 'local variables:  none
10060 '***********************************************************************
10070 CLS : KEY OFF                                           'clear screen
10080 SCREEN 2, 0, 0: WIDTH 80                               'change screen
10090  '
10100 LINE (1, 1)-(638, 40), , B                                'draw boxes
10110 LINE (5, 3)-(633, 37), , B
10120 LINE (70, 50)-(570, 140), , B
10130 LINE (75, 53)-(565, 137), , B
10140                                           'print title, authors, etc.
10150 LOCATE 3, 7
10160 PRINT " C O M P U T A T I O N A L   P H Y S I C S :";
10170 PRINT " (An  Introduction)"
10180 LOCATE 9, 40
10190 PRINT "by";
10200 LOCATE 11, 20: PRINT "R.C. Verma, P.K. Ahluwalia & K.C. Sharma"
10210 LOCATE 13, 34
10220 PRINT "   * * * *    "
10230 '
10240 BEEP
10250 LOCATE 15, 35: PRINT "* * * * * *"
10260 LOCATE 22, 25: PRINT "press any key to continue"
10270 LOCATE 23, 25: PRINT "-------------------------"
10280 '
10290 IF INKEY$ = "" THEN 10290                            'hold the screen
10300 CLS                                                     'clear screen
10310 RETURN
10320 '

