10 '**************************************************************************
20 'Chapter 9, Example 1: Charging & Discharging in RC-Circuit
30 'COMPUTATIONAL PHYSICS: An Introduction
40 'by R. C. Verma, P. K. Ahluwalia and K. C. Sharma
50 '   * * * *    
60 '**************************************************************************
70 '
80  GOSUB 10000                                                   'show header
90 '
100 GOSUB 1000                   'introduce the problem & get input parameters
110 GOSUB 3000                                     'draw boxes and find scales
120 GOSUB 2000                             'perform Euler method & draw graphs
130 '
140 LOCATE 22, 1
150 IF INKEY$ = "" THEN 150
160 '
170 CLS : LOCATE 12, 15: BEEP: PRINT " Thank you !"
180 END
190 '
1000 '************************************************************************
1010 'subroutine to introduce the problem & to get input variables
1020 'input variables:  none
1030 'output variables: none
1040 'global variables: none
1050 'local variables:  CH%
1060 '************************************************************************
1070 '
1080 LOCATE 1, 10
1090 PRINT "This Program describes Charging/Discharging  of a Capacitor"
1100 LOCATE 2, 15: PRINT "in a RC-Circuit"
1110 LOCATE 3, 10: PRINT "---------------------------------------------------"
1120 '
1130 LOCATE 5, 10: PRINT "Please make your choice .... "
1140 LOCATE 7, 10: PRINT "1. Charging of the capacitor ": PRINT
1150 LOCATE 9, 10: PRINT "2. Discharging of the capacitor ": PRINT
1160 LOCATE 12, 10: INPUT "press 1 or 2 "; CH%
1170  IF CH% = 1 OR CH% = 2 THEN 1200
1180  PRINT "try again !": BEEP
1190  LOCATE 12, 10: PRINT SPACE$(65): GOTO 1160
1200 '
1210 IF CH% = 1 THEN GOSUB 5000 ELSE GOSUB 6000
1220 '
1230 RETURN
1240 '
2000 '************************************************************************
2010 'subroutine to do numerical work
2020 'input variables: II, QI, TMIN, TMAX, H, V, C, R
2030 'output variables: T, I, Q
2040 'global variables: none
2050 'local variables:  none
2060 '************************************************************************
2070 '
2080 I = II: Q = QI: T = TMIN                            'initialize variables
2090 '
2100 WHILE T <= TMAX
2110    GOSUB 4000                                                'draw graphs
2120   T = T + H                                               'increment time
2130   I = (V - Q / C) / R                        'find current in the circuit
2140   Q = Q + I * H                             'find charge on the capacitor
2150 WEND
2160 '
2170 RETURN
2180 '
3000 '************************************************************************
3010 'subroutine to draw x- & y- axis , to choose scales
3020 'input variables: TMIN, QMIN, IMIN, TMAX, QMAX, IMAX, CH%
3030 'output variables: TITLE$, SUBTITLE$, XTITLE$, YTITLE$
3040 '                  KT1, KT2, KQ3, KQ4, KI3, KI4
3050 'global variables: none
3060 '************************************************************************
3070 CLS
3080 '
3090 IF CH% = 1 THEN TITLE$ = "Charging of a Capacitor :(RC - circuit)"
3100 IF CH% = 2 THEN TITLE$ = "Discharging of a Capacitor :(RC - circuit)"
3110                                                                '1st graph
3120 PXMIN% = 136: PXMAX% = 392: PYMIN% = 24: PYMAX% = 88
3130                                                     'choose screen region
3140 SUBTITLE$ = "Charge vs. time"
3150 XTITLE$ = "t": YTITLE$ = "q"
3160 XMIN = TMIN: XMAX = TMAX           'assign limits for graphics subroutine
3170 YMIN = QMIN: YMAX = QMAX
3180 '
3190  GOSUB 7000                                    'find scales and draw axes
3200 '
3210 KT1 = KCOEFF1: KT2 = KCOEFF2            'preserve scales for time display
3220 KQ3 = KCOEFF3: KQ4 = KCOEFF4          'preserve scales for charge display
3230                                                                '2nd graph
3240 PXMIN% = 136: PXMAX% = 392: PYMIN% = 112: PYMAX% = 168
3250                                                     'choose screen region
3260 YMIN = IMIN: YMAX = IMAX           'assign limits for graphics subroutine
3270 SUBTITLE$ = "Current vs. time"
3280 XTITLE$ = "t": YTITLE$ = "i"
3290 '
3300  GOSUB 7000                                    'find scales and draw axes
3310 '
3320 KI3 = KCOEFF3: KI4 = KCOEFF4         'preserve scales for current display
3330 '
3340 RETURN
3350 '
4000 '************************************************************************
4010 'subroutine to display graph
4020 'input variables: T, Q, I, KT1, KT2, KQ3, KQ4, KI3, KI4
4030 'output variables: none
4040 'global variables: none
4050 'local variables:T1, T2, I1, I2, Q1, Q2
4060 '************************************************************************
4070 '
4080  Q2 = KQ3 * Q + KQ4                        'perform scale transformations
4090  I2 = KI3 * I + KI4
4100  T2 = KT1 * T + KT2
4110 '
4120 IF T = TMIN THEN 4160                                 'jump if first time
4130 '
4140 LINE (T1, Q1)-(T2, Q2)                              'draw charge vs. time
4150 LINE (T1, I1)-(T2, I2)                             'draw current vs. time
4160 '
4170 T1 = T2: Q1 = Q2: I1 = I2                    'roll variables for graphics
4180 '
4190 RETURN
4200 '
5000 '************************************************************************
5010 'subroutine for getting inputs for charging of a capacitor
5020 'input variables:  none
5030 'output variables: R,C, Q0, H, TMIN,TMAX, IMIN,IMAX, QMIN,QMAX, QI,II
5040 'global variables: none
5050 'local variables:  none
5060 '************************************************************************
5070 CLS
5080 LOCATE 1, 10: PRINT "Please give circuit parameters"
5090 LOCATE 2, 15: PRINT "for charging the capacitor"
5100 PRINT
5110 LOCATE 4, 10: INPUT "Value of Resistance R = "; R
5120  IF R > 0 THEN 5140
5130  BEEP: LOCATE 4, 10: PRINT SPACE$(65): GOTO 5110
5140 LOCATE 6, 10: INPUT "Value of Capacitance C = "; C
5150  IF C > 0 THEN 5170
5160  BEEP: LOCATE 6, 10: PRINT SPACE$(65): GOTO 5140
5170 LOCATE 8, 10: INPUT "Voltage applied = "; V
5180  IF V <> 0 THEN 5200
5190  BEEP: LOCATE 8, 10: PRINT SPACE$(65): GOTO 5170
5200 LOCATE 10, 10: INPUT "Maximum time = "; TMAX
5210  IF TMAX > 0 THEN 5240
5220  BEEP: LOCATE 10, 10: PRINT SPACE$(65): GOTO 5200
5230 '
5240 LOCATE 12, 10: INPUT "time step h = "; H: PRINT
5250  IF H > 0 THEN 5270
5260  BEEP: LOCATE 12, 10: PRINT SPACE$(65): GOTO 5240
5270 '
5280 QMAX = C * V: QMIN = 0: TMIN = 0: IMAX = V / R: IMIN = 0
5290                                         'set limits for scaling the graph
5300 QI = 0: II = V / R                'initial conditions fixed for this case
5310 '
5320 RETURN
5330 '
6000 '************************************************************************
6010 'subroutine to for getting inputs for discharging of a capacitor
6020 'input variables:  none
6030 'output variables: R, C, Q0, H, TC, TMIN, TMAX, IMIN, IMAX, QI, II
6040 'global variables: none
6050 'local variables:  none
6060 '************************************************************************
6070 '
6080 CLS
6090 LOCATE 1, 10: PRINT "Please give circuit parameters"
6100 LOCATE 2, 15: PRINT "for discharging the capacitor"
6110 '
6120 LOCATE 4, 10: INPUT "Value of Resistance R = "; R: PRINT
6130  IF R > 0 THEN 6150
6140  BEEP: LOCATE 4, 10: PRINT SPACE$(65): GOTO 6120
6150 LOCATE 6, 10: INPUT "Value of Capacitance C = "; C: PRINT
6160  IF C > 0 THEN 6180
6170  BEEP: LOCATE 6, 10: PRINT SPACE$(65): GOTO 6150
6180 LOCATE 8, 10: INPUT "Charge on the capacitor Q ="; QI: PRINT
6190  IF QI > 0 THEN 6210
6200  BEEP: LOCATE 8, 10: PRINT SPACE$(65): GOTO 6180
6210 LOCATE 10, 10: INPUT "Maximum time = "; TMAX
6220  IF TMAX > 0 THEN 6240
6230  BEEP: LOCATE 10, 10: PRINT SPACE$(65): GOTO 6210
6240 '
6250 LOCATE 12, 10: INPUT "time step h = "; H: PRINT
6260  IF H > 0 THEN 6280
6270  BEEP: LOCATE 12, 10: PRINT SPACE$(65): GOTO 6250
6280 '
6290 QMAX = QI: QMIN = 0: TMIN = 0: IMIN = -QI / (C * R): IMAX = 0
6300 '                                       'set limits for scaling the graph
6310 II = -QI / (C * R)                'initial conditions fixed for this case
6320 '
6330 RETURN
6340 '
7000 '************************************************************************
7010 'subroutine to display graph between x(i) & y(i) variables
7020 'input variables: XTITLE$, YTITLE$, TITLE$, SUBTITLE$
7030 '                 PXMIN%, PXMAX%, PYMIN%, PYMAX%
7040 'output variables: KCOEFF1, KCOEFF2, KCOEFF3 ,KCOEFF4,
7050 'global variables: none
7060 'local variables: XMIN, XMAX, YMIN, YMAX, ROWTIT%, COLTIT%
7070 '                 DIV%, XSCALE, YSCALE, PXSCALE, PYSCALE
7080 '                 PMX%(J%), PMY%(J%), ROWX%, COLX%, ROWY%, COLY%
7090 '************************************************************************
7100 LOCATE 1, 15: PRINT TITLE$
7110  DIV% = 4                    'set number of divisions (<5) on x- & y-axes
7120                                                  'find scale coefficients
7130 KCOEFF1 = (PXMAX% - PXMIN%) / (XMAX - XMIN)
7140 KCOEFF2 = (PXMAX% * XMIN - PXMIN% * XMAX) / (XMIN - XMAX)
7150 KCOEFF3 = (PYMAX% - PYMIN%) / (YMIN - YMAX)
7160 KCOEFF4 = (PYMAX% * YMAX - PYMIN% * YMIN) / (YMAX - YMIN)
7170 '
7180 ROWTIT% = PYMIN% / 8 + 1: COLTIT% = PXMAX% / 8 + 2
7190 LOCATE ROWTIT%, COLTIT%: PRINT SUBTITLE$               'print graph-title
7200 '
7210 LINE (PXMIN%, PYMIN%)-(PXMAX%, PYMAX%), , B           'draw box for graph
7220 LINE (PXMIN%, KCOEFF4)-(PXMAX%, KCOEFF4)                     'draw x-axis
7230 LINE (KCOEFF2, PYMIN%)-(KCOEFF2, PYMAX%)                     'draw y-axis
7240 '
7250 XSCALE = (XMAX - XMIN) / DIV%                       'find scale for marks
7260 YSCALE = (YMAX - YMIN) / DIV%
7270 '
7280 PXSCALE = (PXMAX% - PXMIN%) / DIV%            'find pixel scale for marks
7290 PYSCALE = (PYMAX% - PYMIN%) / DIV%
7300 '
7310 ROWX% = PYMAX% / 8 + 2              'find row for marks along x-direction
7320 COLY% = PXMIN% / 8 - 10          'find column for marks along y-direction
7330 '
7340 FOR J% = 0 TO DIV%                          'start loop to mark divisions
7350   PMX(J%) = PXMIN% + PXSCALE * J%                    'find places to mark
7360   PMY(J%) = PYMIN% + PYSCALE * J%
7370    LINE (PMX(J%), PYMAX%)-(PMX(J%), PYMAX% - 5)               'draw marks
7380    LINE (PXMIN%, PMY(J%))-(PXMIN% + 5, PMY(J%))
7390 '
7400   IF J% / 2 <> INT(J% / 2) THEN 7640            'jump for alternate marks
7410 '
7420   COLX% = PMX(J%) / 8 - 3         'find column for each mark along x-axis
7430   ROWY% = PMY(J%) / 8 + 1            'find row for each mark along y-axis
7440 '
7450   LOCATE ROWX%, COLX%
7460    IF XMAX > 999.99 OR XMIN < -999.99 THEN 7500
7470    IF XMAX < .01 AND XMIN > -.01 THEN 7500              'format selection
7480   PRINT USING "+###.##"; XMIN + XSCALE * J%      'print x-scale divisions
7490   GOTO 7520
7500   PRINT USING "+#.##^^^^"; XMIN + XSCALE * J%    'print x-scale divisions
7510 '
7520   LOCATE ROWY%, COLY%
7530    IF YMAX > 999.99 OR YMIN < -999.99 THEN 7570
7540    IF YMAX < .01 AND YMIN > -.01 THEN 7570              'format selection
7550   PRINT USING "+###.##"; YMAX - YSCALE * J%      'print y-scale divisions
7560   GOTO 7590
7570   PRINT USING "+#.##^^^^"; YMAX - YSCALE * J%    'print y-scale divisions
7580 '
7590   IF J% <> 2 THEN 7640              'print names of variables along x & y
7600    LOCATE ROWX%, COLTIT% + 3: PRINT XTITLE$; " ->"
7610    LOCATE ROWY% - 1, COLY% - 1: PRINT "^"
7620    LOCATE ROWY%, COLY% - 1: PRINT "|"
7630    LOCATE ROWY% + 1, COLY% - 1: PRINT YTITLE$
7640 NEXT J%
7650 '
7660 RETURN
7670 '
10000 '***********************************************************************
10010 'subroutine to display header screen
10020 'input variables:  none
10030 'output variables: none
10040 'global variables: none
10050 'local variables:  none
10060 '***********************************************************************
10070 CLS : KEY OFF                                           'clear screen
10080 SCREEN 2, 0, 0: WIDTH 80                               'change screen
10090  '
10100 LINE (1, 1)-(638, 40), , B                                'draw boxes
10110 LINE (5, 3)-(633, 37), , B
10120 LINE (70, 50)-(570, 140), , B
10130 LINE (75, 53)-(565, 137), , B
10140                                           'print title, authors, etc.
10150 LOCATE 3, 7
10160 PRINT " C O M P U T A T I O N A L   P H Y S I C S :";
10170 PRINT " (An  Introduction)"
10180 LOCATE 9, 40
10190 PRINT "by";
10200 LOCATE 11, 20: PRINT "R.C. Verma, P.K. Ahluwalia & K.C. Sharma"
10210 LOCATE 13, 34
10220 PRINT "   * * * *    "
10230 '
10240 BEEP
10250 LOCATE 15, 35: PRINT "* * * * * *"
10260 LOCATE 22, 25: PRINT "press any key to continue"
10270 LOCATE 23, 25: PRINT "-------------------------"
10280 '
10290 IF INKEY$ = "" THEN 10290                            'hold the screen
10300 CLS                                                     'clear screen
10310 RETURN
10320 '
