10 '**************************************************************************
20 'Chapter 9, Example 3: Power analysis in a RL-circuit
30 'COMPUTATIONAL PHYSICS: An Introduction
40 'by R. C. Verma, P. K. Ahluwalia and K. C. Sharma
50 '   * * * *    
60 '**************************************************************************
70 '
80 GOSUB 10000                                                    'show header
90 '
100 GOSUB 1000                   'introduce the problem & get input parameters
110 GOSUB 3000                                        'draw boxes & set scales
120 GOSUB 2000                                            'find and draw power
130 '
140 LOCATE 22, 1
150 IF INKEY$ = "" THEN 150
160 '
170 CLS : LOCATE 12, 15: BEEP: PRINT " Thank you !"
180 END
190 '
1000 '************************************************************************
1010 'subroutine for getting inputs for growth of current in LR circuit
1020 'input variables:  none
1030 'output variables: R, L, V, H, TMIN, TMAX, IMIN, IMAX, II, POWMIN, POWMAX
1040 'global variables: none
1050 'local variables:  none
1060 '************************************************************************
1070 CLS
1080 LOCATE 2, 5: PRINT "This Program describes Power Analysis in"
1090 LOCATE 3, 15: PRINT "in a RL - Circuit ) "
1100 LOCATE 4, 5: PRINT "--------------------------------------------"
1110 LOCATE 8, 1
1120 PRINT "Please give circuit parameters to study growth of the current"
1130 '
1140 LOCATE 10, 10: INPUT "Value of Resistance R = "; R
1150  IF R > 0 THEN 1170
1160  BEEP: LOCATE 10, 10: PRINT SPACE$(65): GOTO 1140
1170 LOCATE 12, 10: INPUT "Value of Inductance L = "; L
1180  IF L > 0 THEN 1210
1190  BEEP: LOCATE 12, 10: PRINT SPACE$(65): GOTO 1170
1200 '
1210  II = 0                                    'set initial current condition
1220 LOCATE 14, 10: INPUT "Voltage applied (at t = 0) "; V
1230  IF V <> 0 THEN 1260
1240  BEEP: LOCATE 14, 10: PRINT SPACE$(65): GOTO 1220
1250 '
1260 TMIN = 0                                                    'initial time
1270 LOCATE 16, 10: INPUT "Maximum time = "; TMAX
1280   IF TMAX > 0 THEN 1300
1290   BEEP: LOCATE 16, 10: PRINT SPACE$(65): GOTO 1260
1300 LOCATE 18, 10: INPUT "Give time step h  = "; H
1310   IF H > 0 THEN 1330
1320   BEEP: LOCATE 18, 10: PRINT SPACE$(65): GOTO 1300
1330 '                                         
1340 POWMAX = V * V / R: POWMIN = 0                   'set limits for graphics
1350 '
1360 RETURN
1370 '
2000 '************************************************************************
2010 'subroutine to find current and power in different elements
2020 'input variables: R, L, H, V, II, TMIN, TMAX
2030 'output variables: T, PL, PR, PV, WL, WR, WT
2040 'global variables: none
2050 'local variables: DI
2060 '************************************************************************
2070 '
2080 I = II: T = TMIN
2090 PR = 0: PL = 0: PV = 0: WR = 0: WL = 0: WT = 0
2100                                              'initialization of variables
2110 WHILE T <= TMAX
2120   DI = (V - I * R) / L                                        'find di/dt
2130 '
2140   PL = L * I * DI                                    'find power across L
2150   PR = I * I * R                                   'find power consumed R
2160   PV = V * I                               'find power supplied by source
2170 '
2180   WL = .5 * L * I * I                            'find energy stored in L
2190   WR = WR + R * I * I * H                            'find thermal energy
2200   WT = WT + V * I * H                     'find energy supplied by source
2210 '
2220   GOSUB 5000                                       'draw power/s vs. time
2230 '
2240   T = T + H                                               'increment time
2250   I = I + DI * H                                         'current formula
2260 WEND
2270 '
2280 RETURN
2290 '
3000 '************************************************************************
3010 'subroutine to x- & y- axis , to choose scales
3020 'input variables: TMIN, TMAX, POWMIN, POWMAX
3030 'output variables: XMIN, XMAX, YMIN, YMAX, PXMIN%, PXMAX%, PYMIN%, PYMAX%
3040 '                  SUBTITLE$, XTITLE$, YTITLE$,  KL1, KL2, KL3, KL4,
3050 '                  KV1, KV2, KV3, KV4, KR1, KR2, KR3, KR4
3060 'global variables: none
3070 'local variables: none
3080 '************************************************************************
3090 CLS                                                         'clear screen
3100 LOCATE 1, 15: PRINT " -:-:- Power Analysis in RL - Circuit -:-:-"
3110                                                              'print title
3120 XMIN = TMIN: XMAX = TMAX           'assign limits for graphics subroutine
3130 YMIN = POWMIN: YMAX = POWMAX
3140 XTITLE$ = "t": YTITLE$ = "P"
3150                                       'choose screen region for 1st graph
3160  PXMIN% = 120: PXMAX% = 280: PYMIN% = 24: PYMAX% = 72
3170 SUBTITLE$ = "Power in L"
3180   GOSUB 4000                                   'find scales and draw axes
3190 KL1 = KCOEFF1: KL2 = KCOEFF2: KL3 = KCOEFF3: KL4 = KCOEFF4
3200                                'preserve scale coefficients for later use
3210                                       'choose screen region for 2nd graph
3220  PXMIN% = 440: PXMAX% = 600: PYMIN% = 24: PYMAX% = 72
3230 SUBTITLE$ = "Power in R"
3240   GOSUB 4000                                   'find scales and draw axes
3250 KR1 = KCOEFF1: KR2 = KCOEFF2: KR3 = KCOEFF3: KR4 = KCOEFF4
3260                                'preserve scale coefficients for later use
3270                                       'choose screen region for 3rd graph
3280 PXMIN% = 120: PXMAX% = 280: PYMIN% = 112: PYMAX% = 160
3290 SUBTITLE$ = "Power supplied"
3300   GOSUB 4000                                   'find scales and draw axes
3310 KV1 = KCOEFF1: KV2 = KCOEFF2: KV3 = KCOEFF3: KV4 = KCOEFF4
3320                                'preserve scale coefficients for later use
3330 LOCATE 15, 48: PRINT "  Energy analysis"
3340 LOCATE 16, 48: PRINT "-------------------"
3350 '
3360 RETURN
3370 '
4000 '************************************************************************
4010 'subroutine to draw box
4020 'input variables: XTITLE$, YTITLE$, SUBTITLE$
4030 '                 XMIN, XMAX, YMIN, YMAX, PXMIN%, PXMAX%, PYMIN%, PYMAX%
4040 'output variables: KCOEFF1, KCOEFF2, KCOEFF3 ,KCOEFF4
4050 'global variables: none
4060 'local variables: PXMIN%, PXMAX%, PYMIN%, PYMAX%
4070 '                 DIV%, XSCALE, YSCALE, PXSCALE, PYSCALE
4080 '                 PMX%(J%), PMY%(J%), ROWX%, COLX%, ROWY%, COLY%
4090 '************************************************************************
4100                                                  'find scale coefficients
4110 KCOEFF1 = (PXMAX% - PXMIN%) / (XMAX - XMIN)
4120 KCOEFF2 = (PXMAX% * XMIN - PXMIN% * XMAX) / (XMIN - XMAX)
4130 KCOEFF3 = (PYMAX% - PYMIN%) / (YMIN - YMAX)
4140 KCOEFF4 = (PYMAX% * YMAX - PYMIN% * YMIN) / (YMAX - YMIN)
4150 '
4160 ROWTIT% = PYMIN% / 8 - 1: COLTIT% = PXMIN% / 8 + 1
4170 LOCATE ROWTIT%, COLTIT%: PRINT SUBTITLE$               'print graph-title
4180 '
4190 LINE (PXMIN%, PYMIN%)-(PXMAX%, PYMAX%), , B           'draw box for graph
4200 LINE (PXMIN%, KCOEFF4)-(PXMAX%, KCOEFF4)                     'draw x-axis
4210 LINE (KCOEFF2, PYMIN%)-(KCOEFF2, PYMAX%)                     'draw y-axis
4220 '
4230 DIV% = 4                                   'no. of divisions on x, y axes
4240 XSCALE = (XMAX - XMIN) / DIV%                       'find scale for marks
4250 YSCALE = (YMAX - YMIN) / DIV%
4260 '
4270 PXSCALE = (PXMAX% - PXMIN%) / DIV%            'find pixel scale for marks
4280 PYSCALE = (PYMAX% - PYMIN%) / DIV%
4290 '
4300 ROWX% = PYMAX% / 8 + 2              'find row for marks along x-direction
4310 COLY% = PXMIN% / 8 - 10          'find column for marks along y-direction
4320 '
4330 FOR J% = 0 TO DIV%                          'start loop to mark divisions
4340   PMX(J%) = PXMIN% + PXSCALE * J%                    'find places to mark
4350   PMY(J%) = PYMIN% + PYSCALE * J%
4360    LINE (PMX(J%), PYMAX%)-(PMX(J%), PYMAX% - 5)               'draw marks
4370    LINE (PXMIN%, PMY(J%))-(PXMIN% + 5, PMY(J%))
4380 '
4390   IF J% / 2 <> INT(J% / 2) THEN 4630            'jump for alternate marks
4400 '
4410   COLX% = PMX(J%) / 8 - 3         'find column for each mark along x-axis
4420   ROWY% = PMY(J%) / 8 + 1            'find row for each mark along y-axis
4430 '
4440   LOCATE ROWX%, COLX%
4450    IF XMAX > 999.99 OR XMIN < -999.99 THEN 4490
4460    IF XMAX < .1 AND XMIN > -.1 THEN 4490                'format selection
4470   PRINT USING "+###.##"; XMIN + XSCALE * J%      'print x-scale divisions
4480   GOTO 4510
4490   PRINT USING "+#.##^^^^"; XMIN + XSCALE * J%    'print x-scale divisions
4500 '
4510   LOCATE ROWY%, COLY%
4520    IF YMAX > 999.99 OR YMIN < -999.99 THEN 4560
4530    IF YMAX < .1 AND YMIN > -.1 THEN 4560                'format selection
4540   PRINT USING "+###.##"; YMAX - YSCALE * J%      'print y-scale divisions
4550   GOTO 4580
4560   PRINT USING "+#.##^^^^"; YMAX - YSCALE * J%    'print y-scale divisions
4570 '
4580   IF J% <> 2 THEN 4630              'print names of variables along x & y
4590    LOCATE ROWX% - 2, PXMAX% / 8 + 3: PRINT XTITLE$; "->"
4600    LOCATE ROWY% - 1, COLY% - 1: PRINT "^"
4610    LOCATE ROWY%, COLY% - 1: PRINT "|"
4620    LOCATE ROWY% + 1, COLY% - 1: PRINT YTITLE$
4630 NEXT J%
4640 '
4650 RETURN
4660 '
5000 '************************************************************************
5010 'subroutine to display graph
5020 'input variables:  T, PL, PR, PV, WL, WR, WT, KL1, KL2, KL3, KL4
5030 '                  KV1, KV2, KV3, KV4, KR1, KR2, KR3, KR4
5040 'output variables: none
5050 'global variables: none
5060 'local variables: TL1,TL2,PL1,PL2, TV1,TV2,PV1,PV2, TR1,TR2,PR1,PR2
5070 '************************************************************************
5080                                        'perform scale transformations for
5090 TL2 = KL1 * T + KL2: PL2 = KL3 * PL + KL4                      '1st graph
5100 TV2 = KV1 * T + KV2: PV2 = KV3 * PV + KV4                      '2nd graph
5110 TR2 = KR1 * T + KR2: PR2 = KR3 * PR + KR4                      '3rd graph
5120 '
5130 IF T = TMIN THEN 5180                                'jump if first time
5140 '
5150 LINE (TL1, PL1)-(TL2, PL2)
5160 LINE (TV1, PV1)-(TV2, PV2)
5170 LINE (TR1, PR1)-(TR2, PR2)
5180                                                       'roll variables for
5190 TL1 = TL2: PL1 = PL2                                           '1st graph
5200 TV1 = TV2: PV1 = PV2                                           '2nd graph
5210 TR1 = TR2: PR1 = PR2                                           '3rd graph
5220 '
5230   LOCATE 17, 48: PRINT "stored in L = ";
5240 IF ABS(WL) < 1000 THEN PRINT USING "###.###"; WL ELSE PRINT USING "#.###^^^^"; WL
5250   LOCATE 19, 48: PRINT "consumed in R = ";
5260 IF ABS(WR) < 1000 THEN PRINT USING "###.###"; WR ELSE PRINT USING "#.###^^^^"; WR
5270   LOCATE 21, 48: PRINT "total supply  = ";
5280 IF ABS(WT) < 1000 THEN PRINT USING "###.###"; WT ELSE PRINT USING "#.###^^^^"; WT
5290                                                 'print energy consumption
5300 RETURN
5310 '
10000 '***********************************************************************
10010 'subroutine to display header screen
10020 'input variables:  none
10030 'output variables: none
10040 'global variables: none
10050 'local variables:  none
10060 '***********************************************************************
10070 CLS : KEY OFF                                           'clear screen
10080 SCREEN 2, 0, 0: WIDTH 80                               'change screen
10090  '
10100 LINE (1, 1)-(638, 40), , B                                'draw boxes
10110 LINE (5, 3)-(633, 37), , B
10120 LINE (70, 50)-(570, 140), , B
10130 LINE (75, 53)-(565, 137), , B
10140                                           'print title, authors, etc.
10150 LOCATE 3, 7
10160 PRINT " C O M P U T A T I O N A L   P H Y S I C S :";
10170 PRINT " (An  Introduction)"
10180 LOCATE 9, 40
10190 PRINT "by";
10200 LOCATE 11, 20: PRINT "R.C. Verma, P.K. Ahluwalia & K.C. Sharma"
10210 LOCATE 13, 34
10220 PRINT "   * * * *    "
10230 '
10240 BEEP
10250 LOCATE 15, 35: PRINT "* * * * * *"
10260 LOCATE 22, 25: PRINT "press any key to continue"
10270 LOCATE 23, 25: PRINT "-------------------------"
10280 '
10290 IF INKEY$ = "" THEN 10290                            'hold the screen
10300 CLS                                                     'clear screen
10310 RETURN
10320 '
