10 '**************************************************************************
20 'Chapter 9, Example 4:  EM Oscillations in a LCR - Circuit
30 'COMPUTATIONAL PHYSICS: An Introduction
40 'by R. C. Verma, P. K. Ahluwalia and K. C. Sharma
50 '   * * * *   
60 '**************************************************************************
70 '
80 GOSUB 10000                                                    'show header
90 '
100 DEF FNI (Q, I) = -(Q / C + R * I) / L    'define function for LCR equation
110 '
120 GOSUB 1000                   'introduce the problem & get input parameters
130 '
140 GOSUB 3000                                       'draw boxes & find scales
150 GOSUB 2000                                     'perform Runge-Kutta method
160 '
170 LOCATE 22, 1
180 IF INKEY$ = "" THEN 180
190 '
200 CLS : LOCATE 12, 15: BEEP: PRINT " Thank you !"
210 END
220 '
1000 '************************************************************************
1010 'subroutine to get L, C, R and other variables
1020 'input variables:  none
1030 'output variables: L, C, R, QI, II, H,
1040 '                  TMIN, TMAX, QMIN, QMAX, IMAX, IMIN, VMIN, VMAX
1050 'global variables: none
1060 'local variables:  none
1070 'Test case : C=0.25, L=1, R=1, I=0, QI=10, tmax=15, H=0.1  
1080 'Test case : C=0.25, L=1.5, R=0.5, I=0, QI=10, tmax=15, H=0.1
1090 'For small C, small L keep time interval small or large resistance
1100 '************************************************************************
1110 '
1120 LOCATE 2, 1: PRINT "This program analyzes an LCR circuit  "
1130 LOCATE 3, 1: PRINT "--------------------------------------"
1140 LOCATE 5, 10: PRINT "Please give the following quantities:"
1150 '
1160 LOCATE 8, 10: INPUT "Value of Resistance R = "; R
1170  IF R >= 0 THEN 1190
1180  BEEP: LOCATE 8, 10: PRINT SPACE$(65): GOTO 1160
1190 LOCATE 10, 10: INPUT "Value of Capacitance = "; C
1200  IF C > 0 THEN 1220
1210  BEEP: LOCATE 10, 10: PRINT SPACE$(65): GOTO 1190
1220 LOCATE 12, 10: INPUT "Value of Inductance L = "; L
1230  IF L > 0 THEN 1250
1240  BEEP: LOCATE 12, 10: PRINT SPACE$(65): GOTO 1220
1250 LOCATE 14, 10: INPUT "Initial charge on the Capacitor "; QI
1260  IF QI >= 0 THEN 1280
1270  BEEP: LOCATE 14, 10: PRINT SPACE$(65): GOTO 1250
1280 LOCATE 16, 10: INPUT "Initial current in the circuit"; II
1290  IF II >= 0 THEN 1310
1300  BEEP: LOCATE 16, 10: PRINT SPACE$(65): GOTO 1280
1310 '
1320 IF QI = 0 AND II = 0 THEN 1330 ELSE 1370
1330 BEEP: LOCATE 18, 1:
1340 PRINT "No EM oscillations would occur since q = 0 & i = 0 at t = 0!"
1350 LOCATE 20, 5: PRINT "so program terminate now": END
1360 '
1370 TMIN = 0                                                    'initial time
1380 LOCATE 18, 10: INPUT "Maximum time = "; TMAX
1390   IF TMAX > 0 THEN 1410
1400   BEEP: LOCATE 18, 10: PRINT SPACE$(65): GOTO 1370
1410 LOCATE 20, 10: INPUT "Give time step h  = "; H
1420   IF H > 0 THEN 1440
1430   BEEP: LOCATE 20, 10: PRINT SPACE$(65): GOTO 1410
1440 '
1450  W = 1 / SQR(L * C)                 'find expected frequency for graphics
1460                                      'set min. & max. limits for graphics
1470 QMAX = SQR(QI ^ 2 + (II / W) ^ 2): QMIN = -QMAX
1480 IMAX = QMAX * W:  IMIN = -IMAX
1490 VMAX = IMAX * W * L: VMIN = -VMAX
1500 '
1510 RETURN
1520 '
2000 '************************************************************************
2010 'subroutine to find charge, current & Voltage across inductor
2020 'input variables: TMIN, TMAX, L, C, R, QI, II, FNI(), H
2030 'output variables: T, I, Q, VL
2040 'global variables: none
2050 'local variables: K1, K2, K3, K4, P1, P2, P3, P4, DI
2060 '************************************************************************
2070 '
2080 I = II: T = TMIN: Q = QI                              'initial conditions
2090 '
2100 WHILE T <= TMAX
2110  GOSUB 5000                                'draw current, charge vs. time
2120  T = T + H                                                'increment time
2130   K1 = H * I                               'find Runge-Kutta coefficients
2140    P1 = H * FNI(Q, I)
2150   K2 = H * (I + P1 / 2)
2160    P2 = H * FNI(Q + K1 / 2, I + P1 / 2)
2170   K3 = H * (I + P2 / 2)
2180    P3 = H * FNI(Q + K2 / 2, I + P2 / 2)
2190   K4 = H * (I + P3)
2200    P4 = H * FNI(Q + K3, I + P3)
2210   Q = Q + (K1 + 2 * K2 + 2 * K3 + K4) / 6           'apply charge formula
2220   I = I + (P1 + 2 * P2 + 2 * P3 + P4) / 6          'apply current formula
2230    DI = -(1 / C * Q + R * I) / L                              'find di/dt
2240    VL = -L * DI                                    'find voltage across L
2250 WEND
2260 '
2270 RETURN
2280 '
3000 '************************************************************************
3010 'subroutine to x- & y- axis , to choose scales
3020 'input variables: TMIN, TMAX, QMIN, QMAX, IMIN, IMAX, VMIN, VMAX
3030 'output variables: XMIN, XMAX, YMIN, YMAX, PXMIN%, PXMAX%, PYMIN%, PYMAX%
3040 '                  SUBTITLE$, XTITLE$, YTITLE$,  KQ1, KQ2, KQ3, KQ4,
3050 '                  KV1, KV2, KV3, KV4, KI1, KI2, KI3, KI4
3060 'global variables: none
3070 'local variables: none
3080 '************************************************************************
3090 CLS                                                         'clear screen
3100 LOCATE 1, 15: PRINT "* * * Analysis of LCR - Circuit * * *"
3110                                                              'print title
3120 XMIN = TMIN: XMAX = TMAX           'assign limits for graphics subroutine
3130 XTITLE$ = "t"
3140                                       'choose screen region for 1st graph
3150  PXMIN% = 120: PXMAX% = 280: PYMIN% = 24: PYMAX% = 72
3160 YMIN = QMIN: YMAX = QMAX
3170  YTITLE$ = "q": SUBTITLE$ = "Charge on C vs. t"
3180   GOSUB 4000                                   'find scales and draw axes
3190 KQ1 = KCOEFF1: KQ2 = KCOEFF2: KQ3 = KCOEFF3: KQ4 = KCOEFF4
3200                                'preserve scale coefficients for later use
3210                                       'choose screen region for 2nd graph
3220  PXMIN% = 440: PXMAX% = 600: PYMIN% = 24: PYMAX% = 72
3230 YMIN = IMIN: YMAX = IMAX
3240 YTITLE$ = "i": SUBTITLE$ = "Current vs. t"
3250   GOSUB 4000                                   'find scales and draw axes
3260 KI1 = KCOEFF1: KI2 = KCOEFF2: KI3 = KCOEFF3: KI4 = KCOEFF4
3270                                'preserve scale coefficients for later use
3280                                       'choose screen region for 3rd graph
3290 PXMIN% = 120: PXMAX% = 280: PYMIN% = 112: PYMAX% = 160
3300 YMIN = VMIN: YMAX = VMAX
3310  YTITLE$ = "VL": SUBTITLE$ = "Voltage at L vs. t"
3320   GOSUB 4000                                   'find scales and draw axes
3330 KV1 = KCOEFF1: KV2 = KCOEFF2: KV3 = KCOEFF3: KV4 = KCOEFF4
3340                                'preserve scale coefficients for later use
3350 LOCATE 15, 45: PRINT "* Instantaneous Values *"
3360 '
3370 RETURN
3380 '
4000 '************************************************************************
4010 'subroutine to draw box
4020 'input variables: XTITLE$, YTITLE$, SUBTITLE$
4030 '                 XMIN, XMAX, YMIN, YMAX, PXMIN%, PXMAX%, PYMIN%, PYMAX%
4040 'output variables: KCOEFF1, KCOEFF2, KCOEFF3 ,KCOEFF4
4050 'global variables: none
4060 'local variables: PXMIN%, PXMAX%, PYMIN%, PYMAX%
4070 '                 DIV%, XSCALE, YSCALE, PXSCALE, PYSCALE
4080 '                 PMX%(J%), PMY%(J%), ROWX%, COLX%, ROWY%, COLY%
4090 '************************************************************************
4100                                                  'find scale coefficients
4110 KCOEFF1 = (PXMAX% - PXMIN%) / (XMAX - XMIN)
4120 KCOEFF2 = (PXMAX% * XMIN - PXMIN% * XMAX) / (XMIN - XMAX)
4130 KCOEFF3 = (PYMAX% - PYMIN%) / (YMIN - YMAX)
4140 KCOEFF4 = (PYMAX% * YMAX - PYMIN% * YMIN) / (YMAX - YMIN)
4150 '
4160 ROWTIT% = PYMIN% / 8 - 1: COLTIT% = PXMIN% / 8 + 1
4170 LOCATE ROWTIT%, COLTIT%: PRINT SUBTITLE$               'print graph-title
4180 '
4190 LINE (PXMIN%, PYMIN%)-(PXMAX%, PYMAX%), , B           'draw box for graph
4200 LINE (PXMIN%, KCOEFF4)-(PXMAX%, KCOEFF4)                     'draw x-axis
4210 LINE (KCOEFF2, PYMIN%)-(KCOEFF2, PYMAX%)                     'draw y-axis
4220 '
4230 DIV% = 4                                   'no. of divisions on x, y axes
4240 XSCALE = (XMAX - XMIN) / DIV%                       'find scale for marks
4250 YSCALE = (YMAX - YMIN) / DIV%
4260 '
4270 PXSCALE = (PXMAX% - PXMIN%) / DIV%            'find pixel scale for marks
4280 PYSCALE = (PYMAX% - PYMIN%) / DIV%
4290 '
4300 ROWX% = PYMAX% / 8 + 2              'find row for marks along x-direction
4310 COLY% = PXMIN% / 8 - 10          'find column for marks along y-direction
4320 '
4330 FOR J% = 0 TO DIV%                          'start loop to mark divisions
4340   PMX(J%) = PXMIN% + PXSCALE * J%                    'find places to mark
4350   PMY(J%) = PYMIN% + PYSCALE * J%
4360    LINE (PMX(J%), PYMAX%)-(PMX(J%), PYMAX% - 5)               'draw marks
4370    LINE (PXMIN%, PMY(J%))-(PXMIN% + 5, PMY(J%))
4380 '
4390   IF J% / 2 <> INT(J% / 2) THEN 4630            'jump for alternate marks
4400 '
4410   COLX% = PMX(J%) / 8 - 3         'find column for each mark along x-axis
4420   ROWY% = PMY(J%) / 8 + 1            'find row for each mark along y-axis
4430 '
4440   LOCATE ROWX%, COLX%
4450    IF XMAX > 999.99 OR XMIN < -999.99 THEN 4490
4460    IF XMAX < .1 AND XMIN > -.1 THEN 4490                'format selection
4470   PRINT USING "+###.##"; XMIN + XSCALE * J%      'print x-scale divisions
4480   GOTO 4510
4490   PRINT USING "+#.##^^^^"; XMIN + XSCALE * J%    'print x-scale divisions
4500 '
4510   LOCATE ROWY%, COLY%
4520    IF YMAX > 999.99 OR YMIN < -999.99 THEN 4560
4530    IF YMAX < .1 AND YMIN > -.1 THEN 4560                'format selection
4540   PRINT USING "+###.##"; YMAX - YSCALE * J%      'print y-scale divisions
4550   GOTO 4580
4560   PRINT USING "+#.##^^^^"; YMAX - YSCALE * J%    'print y-scale divisions
4570 '
4580   IF J% <> 2 THEN 4630              'print names of variables along x & y
4590    LOCATE ROWX% - 2, PXMAX% / 8 + 3: PRINT XTITLE$; "->"
4600    LOCATE ROWY% - 1, COLY% - 1: PRINT "^"
4610    LOCATE ROWY%, COLY% - 1: PRINT "|"
4620    LOCATE ROWY% + 1, COLY% - 1: PRINT YTITLE$
4630 NEXT J%
4640 '
4650 RETURN
4660 '
5000 '************************************************************************
5010 'subroutine to display graph
5020 'input variables:  T, Q, I, VL, DI, KQ1, KQ2, KQ3, KQ4
5030 '                  KV1, KV2, KV3, KV4, KI1, KI2, KI3, KI4
5040 'output variables: none
5050 'global variables: none
5060 'local variables: TL1,TL2,PL1,PL2, TV1,TV2,PV1,PV2, TR1,TR2,PR1,PR2
5070 '************************************************************************
5080                                        'perform scale transformations for
5090 TQ2 = KQ1 * T + KQ2: Q2 = KQ3 * Q + KQ4                        '1st graph
5100 TI2 = KI1 * T + KI2: I2 = KI3 * I + KI4                        '2nd graph
5110 TV2 = KV1 * T + KV2: VL2 = KV3 * VL + KV4                      '3rd graph
5120 '
5130 IF T = TMIN THEN 5180                                 'jump if first time
5140 '
5150 LINE (TQ1, Q1)-(TQ2, Q2)
5160 LINE (TI1, I1)-(TI2, I2)
5170 LINE (TV1, VL1)-(TV2, VL2)
5180                                                       'roll variables for
5190 TQ1 = TQ2: Q1 = Q2                                             '1st graph
5200 TI1 = TI2: I1 = I2                                             '2nd graph
5210 TV1 = TV2: VL1 = VL2                                           '3rd graph
5220 '
5225 LOCATE 16, 48: PRINT USING "  t = ####.###"; T
5230   LOCATE 18, 48: PRINT "charge on C = ";
5240 IF ABS(Q) < 1000 THEN PRINT USING "###.###"; Q ELSE PRINT USING "#.###^^^^"; Q
5250   LOCATE 20, 48: PRINT "Current = ";
5260 IF ABS(I) < 1000 THEN PRINT USING "###.###"; I ELSE PRINT USING "#.###^^^^"; I
5270   LOCATE 22, 48: PRINT "Voltage at L  = ";
5280 IF ABS(VL) < 1000 THEN PRINT USING "###.###"; VL ELSE PRINT USING "#.###^^^^"; VL
5290                                                     'print instant values
5300 RETURN
5310 '
10000 '***********************************************************************
10010 'subroutine to display header screen
10020 'input variables:  none
10030 'output variables: none
10040 'global variables: none
10050 'local variables:  none
10060 '***********************************************************************
10070 CLS : KEY OFF                                           'clear screen
10080 SCREEN 2, 0, 0: WIDTH 80                               'change screen
10090  '
10100 LINE (1, 1)-(638, 40), , B                                'draw boxes
10110 LINE (5, 3)-(633, 37), , B
10120 LINE (70, 50)-(570, 140), , B
10130 LINE (75, 53)-(565, 137), , B
10140                                           'print title, authors, etc.
10150 LOCATE 3, 7
10160 PRINT " C O M P U T A T I O N A L   P H Y S I C S :";
10170 PRINT " (An  Introduction)"
10180 LOCATE 9, 40
10190 PRINT "by";
10200 LOCATE 11, 20: PRINT "R.C. Verma, P.K. Ahluwalia & K.C. Sharma"
10210 LOCATE 13, 34
10220 PRINT "   * * * *    "
10230 '
10240 BEEP
10250 LOCATE 15, 35: PRINT "* * * * * *"
10260 LOCATE 22, 25: PRINT "press any key to continue"
10270 LOCATE 23, 25: PRINT "-------------------------"
10280 '
10290 IF INKEY$ = "" THEN 10290                            'hold the screen
10300 CLS                                                     'clear screen
10310 RETURN
10320 '

