10 '**************************************************************************
20 'Chapter 9, Example 4:  EM Oscillations in a LCR - Circuit
30 'COMPUTATIONAL PHYSICS: An Introduction
40 'by R. C. Verma, P. K. Ahluwalia and K. C. Sharma
50 ' * * * *
60 '**************************************************************************
70 '
80 GOSUB 10000                                                    'show header
90 '
100  DEF FNV (T) = VI * SIN(W * T)                    'define voltage function
110  DEF FNI (Q, I, T) = -(Q / C + R * I - FNV(T)) / L    'define LCR equation
120 '
130 GOSUB 1000                   'introduce the problem & get input parameters
140 '
150  N% = (TMAX - TMIN) / H   'find number of steps to cover the time interval
160 DIM T(N% + 5), DI(N% + 5), CUR(N% + 5), Q(N% + 5), VL(N% + 5)
170                                     'set dimensions of the array variables
180 GOSUB 2000                                     'perform Runge-Kutta method
190 GOSUB 3000                               'draw the x,y-axis and set scales
200 GOSUB 5000               'draw charge, current, Voltage across L  vs. time
210 '
220 LOCATE 22, 1
230 IF INKEY$ = "" THEN 230
240 '
250 CLS : LOCATE 12, 15: BEEP: PRINT " Thank you !"
260 END
270 '
1000 '************************************************************************
1010 'subroutine to get L, C, R and other variables
1020 'input variables:  none
1030 'output variables: L, C, R, QI, II, H, VI, W, TMIN, TMAX
1040 'global variables: none
1050 'local variables:  none
1060 'Test case : C=0.25, L=1, R=1, I=0, QI=10, tmax=15, H=0.1
1070 'Test case : C=0.25, L=1.5, R=0.5, I=0, QI=10, tmax=15, H=0.1
1080 'For small C, small L keep time interval small or large resistance
1090 '************************************************************************
1100 '
1110 LOCATE 2, 1: PRINT "This program studies a driven LCR circuit  "
1120 LOCATE 3, 1: PRINT "--------------------------------------"
1130 LOCATE 5, 10: PRINT "Please give the following quantities:"
1140 '
1150 LOCATE 8, 10: INPUT "Value of Resistance R = "; R
1160  IF R > 0 THEN 1180
1170  BEEP: LOCATE 8, 10: PRINT SPACE$(65): GOTO 1150
1180 LOCATE 10, 10: INPUT "Value of Capacitance = "; C
1190  IF C > 0 THEN 1210
1200  BEEP: LOCATE 10, 10: PRINT SPACE$(65): GOTO 1180
1210 LOCATE 12, 10: INPUT "Value of Inductance L = "; L
1220  IF L > 0 THEN 1240
1230  BEEP: LOCATE 12, 10: PRINT SPACE$(65): GOTO 1210
1235 '
1240 LOCATE 14, 10: PRINT SPACE$(65)
1250 LOCATE 14, 10: INPUT "Initial charge on the Capacitor "; QI
1260 LOCATE 16, 10: PRINT SPACE$(65)
1270 LOCATE 16, 10: INPUT "Initial current in the circuit"; II
1300 '
1310 LOCATE 18, 1: INPUT "Give amplitude & frequency of driver = "; VI, W
1315  IF (ABS(QI) + ABS(II) + ABS(VI)) <> 0 THEN 1330   'if circuit oscillates
1320     CLS : BEEP: LOCATE 12, 10
1325     PRINT "Circuit does not oscillate!": END               'program ends
1330     IF W >= 0 THEN 1340
1335  BEEP: LOCATE 18, 1: PRINT SPACE$(75): GOTO 1310
1340 '
1350 TMIN = 0                                                    'initial time
1360 LOCATE 20, 10: INPUT "Maximum time = "; TMAX
1370   IF TMAX > 0 THEN 1390
1380   BEEP: LOCATE 20, 10: PRINT SPACE$(65): GOTO 1350
1390 LOCATE 22, 10: INPUT "Give time step h  = "; H
1400   IF H > 0 THEN 1420
1410   BEEP: LOCATE 22, 10: PRINT SPACE$(65): GOTO 1390
1420 '
1430 RETURN
1440 '
2000 '************************************************************************
2010 'subroutine to find charge, current & voltage across inductor
2020 'input variables: TMIN, QI, I0, N%, FNI(), H
2030 'output variables: T(I%), CUR(I%), Q(I%), VL(I%), DI(I%)
2040 'global variables: none
2050 'local variables: K1, K2, K3, K4, P1, P2, P3, P4
2060 '************************************************************************
2070 '
2080 CUR(0) = II: T(0) = TMIN: Q(0) = QI                   'initial conditions
2090 '
2100 TERM% = INT(N% / 20)              'defined for selective printing results
2110 '
2120 PRINT "         Analysis of LCR circuit"
2130 PRINT "------------------------------------------"         'prepare table
2140 PRINT "  t     Q at C        I           V at L"
2150 PRINT "------------------------------------------"
2160 '
2170 FOR I% = 0 TO N%
2180   T(I%) = TMIN + H * I%                                   'time increment
2190 '
2200   K1 = H * CUR(I%)                         'find Runge-Kutta coefficients
2210    P1 = H * FNI(Q(I%), CUR(I%), T(I%))
2220   K2 = H * (CUR(I%) + P1 / 2)
2230    P2 = H * FNI(Q(I%) + K1 / 2, CUR(I%) + P1 / 2, T(I%) + H / 2)
2240   K3 = H * (CUR(I%) + P2 / 2)
2250    P3 = H * FNI(Q(I%) + K2 / 2, CUR(I%) + P2 / 2, T(I%) + H / 2)
2260   K4 = H * (CUR(I%) + P3)
2270    P4 = H * FNI(Q(I%) + K3, CUR(I%) + P3, T(I%) + H)
2280 '
2290   Q(I% + 1) = Q(I%) + (K1 + 2 * K2 + 2 * K3 + K4) / 6     'charge formula
2300   CUR(I% + 1) = CUR(I%) + (P1 + 2 * P2 + 2 * P3 + P4) / 6
2310                                                          'current formula
2320   DI(I%) = -(1 / C * Q(I%) + R * CUR(I%)) / L                      'di/dt
2330   VL(I%) = -L * DI(I%)                                  'voltage across L
2340 '
2350   IF (I% MOD TERM% <> 0) THEN 2370                     'print selectively
2360   PRINT USING "#.###^^^^ +#.###^^^^  +#.###^^^^  +#.###^^^^"; T(I%); Q(I%); CUR(I%); VL(I%)
2370 NEXT I%
2380 '
2390 PRINT "--------------------------------------------"
2400 '
2410  IMAX = 0: VLMAX = 0: QMAX = 0                      'initialize variables
2420  IMIN = 0: VLMIN = 0: QMIN = 0
2430                                         'find maxima & minima for scaling
2440 FOR I% = 0 TO N%
2450   IF CUR(I%) > IMAX THEN IMAX = CUR(I%)
2460    IF CUR(I%) < IMIN THEN IMIN = CUR(I%)
2470   IF VL(I%) > VLMAX THEN VLMAX = VL(I%)
2480    IF VL(I%) < VLMIN THEN VLMIN = VL(I%)
2490   IF Q(I%) < QMIN THEN QMIN = Q(I%)
2500    IF Q(I%) > QMAX THEN QMAX = Q(I%)
2510 NEXT I%
2520 '
2530 RETURN
2540 '
3000 '************************************************************************
3010 'subroutine to x- & y- axis , to choose scales
3020 'input variables: TMIN, TMAX, QMIN, QMAX, IMIN, IMAX, VMIN, VMAX
3030 'output variables: XMIN, XMAX, YMIN, YMAX, PXMIN%, PXMAX%, PYMIN%, PYMAX%
3040 '                  SUBTITLE$, XTITLE$, YTITLE$,  KQ1, KQ2, KQ3, KQ4,
3050 '                  KV1, KV2, KV3, KV4, KI1, KI2, KI3, KI4
3060 'global variables: none
3070 'local variables: none
3080 '************************************************************************
3090 CLS                                                         'clear screen
3100 LOCATE 1, 15: PRINT "* * * Analysis of LCR - Circuit * * *"
3110                                                              'print title
3120 XMIN = TMIN: XMAX = TMAX           'assign limits for graphics subroutine
3130 XTITLE$ = "t"
3140                                       'choose screen region for 1st graph
3150  PXMIN% = 120: PXMAX% = 280: PYMIN% = 24: PYMAX% = 72
3160 YMIN = QMIN: YMAX = QMAX
3170  YTITLE$ = "q": SUBTITLE$ = "Charge on C vs. t"
3180   GOSUB 4000                                   'find scales and draw axes
3190 KQ1 = KCOEFF1: KQ2 = KCOEFF2: KQ3 = KCOEFF3: KQ4 = KCOEFF4
3200                                'preserve scale coefficients for later use
3210                                       'choose screen region for 2nd graph
3220  PXMIN% = 440: PXMAX% = 600: PYMIN% = 24: PYMAX% = 72
3230 YMIN = IMIN: YMAX = IMAX
3240 YTITLE$ = "i": SUBTITLE$ = "Current vs. t"
3250   GOSUB 4000                                   'find scales and draw axes
3260 KI1 = KCOEFF1: KI2 = KCOEFF2: KI3 = KCOEFF3: KI4 = KCOEFF4
3270                                'preserve scale coefficients for later use
3280                                       'choose screen region for 3rd graph
3290 PXMIN% = 120: PXMAX% = 280: PYMIN% = 112: PYMAX% = 160
3300 YMIN = VLMIN: YMAX = VLMAX
3305 '  YMIN = VMIN: YMAX = VMAX
3310  YTITLE$ = "VL": SUBTITLE$ = "Voltage at L vs. t"
3320   GOSUB 4000                                   'find scales and draw axes
3330 KV1 = KCOEFF1: KV2 = KCOEFF2: KV3 = KCOEFF3: KV4 = KCOEFF4
3340                                'preserve scale coefficients for later use
3350 LOCATE 15, 45: PRINT "* Instantaneous Values *"
3360 '
3370 RETURN
3380 '
4000 '************************************************************************
4010 'subroutine to draw box
4020 'input variables: XTITLE$, YTITLE$, SUBTITLE$
4030 '                 XMIN, XMAX, YMIN, YMAX, PXMIN%, PXMAX%, PYMIN%, PYMAX%
4040 'output variables: KCOEFF1, KCOEFF2, KCOEFF3 ,KCOEFF4
4050 'global variables: none
4060 'local variables: PXMIN%, PXMAX%, PYMIN%, PYMAX%
4070 '                 DIV%, XSCALE, YSCALE, PXSCALE, PYSCALE
4080 '                 PMX%(J%), PMY%(J%), ROWX%, COLX%, ROWY%, COLY%
4090 '************************************************************************
4100                                                  'find scale coefficients
4110 KCOEFF1 = (PXMAX% - PXMIN%) / (XMAX - XMIN)
4120 KCOEFF2 = (PXMAX% * XMIN - PXMIN% * XMAX) / (XMIN - XMAX)
4130 KCOEFF3 = (PYMAX% - PYMIN%) / (YMIN - YMAX)
4140 KCOEFF4 = (PYMAX% * YMAX - PYMIN% * YMIN) / (YMAX - YMIN)
4150 '
4160 ROWTIT% = PYMIN% / 8 - 1: COLTIT% = PXMIN% / 8 + 1
4170 LOCATE ROWTIT%, COLTIT%: PRINT SUBTITLE$               'print graph-title
4180 '
4190 LINE (PXMIN%, PYMIN%)-(PXMAX%, PYMAX%), , B           'draw box for graph
4200 LINE (PXMIN%, KCOEFF4)-(PXMAX%, KCOEFF4)                     'draw x-axis
4210 LINE (KCOEFF2, PYMIN%)-(KCOEFF2, PYMAX%)                     'draw y-axis
4220 '
4230 DIV% = 4                                   'no. of divisions on x, y axes
4240 XSCALE = (XMAX - XMIN) / DIV%                       'find scale for marks
4250 YSCALE = (YMAX - YMIN) / DIV%
4260 '
4270 PXSCALE = (PXMAX% - PXMIN%) / DIV%            'find pixel scale for marks
4280 PYSCALE = (PYMAX% - PYMIN%) / DIV%
4290 '
4300 ROWX% = PYMAX% / 8 + 2              'find row for marks along x-direction
4310 COLY% = PXMIN% / 8 - 10          'find column for marks along y-direction
4320 '
4330 FOR J% = 0 TO DIV%                          'start loop to mark divisions
4340   PMX(J%) = PXMIN% + PXSCALE * J%                    'find places to mark
4350   PMY(J%) = PYMIN% + PYSCALE * J%
4360    LINE (PMX(J%), PYMAX%)-(PMX(J%), PYMAX% - 5)               'draw marks
4370    LINE (PXMIN%, PMY(J%))-(PXMIN% + 5, PMY(J%))
4380 '
4390   IF J% / 2 <> INT(J% / 2) THEN 4630            'jump for alternate marks
4400 '
4410   COLX% = PMX(J%) / 8 - 3         'find column for each mark along x-axis
4420   ROWY% = PMY(J%) / 8 + 1            'find row for each mark along y-axis
4430 '
4440   LOCATE ROWX%, COLX%
4450    IF XMAX > 999.99 OR XMIN < -999.99 THEN 4490
4460    IF XMAX < .1 AND XMIN > -.1 THEN 4490                'format selection
4470   PRINT USING "+###.##"; XMIN + XSCALE * J%      'print x-scale divisions
4480   GOTO 4510
4490   PRINT USING "+#.##^^^^"; XMIN + XSCALE * J%    'print x-scale divisions
4500 '
4510   LOCATE ROWY%, COLY%
4520    IF YMAX > 999.99 OR YMIN < -999.99 THEN 4560
4530    IF YMAX < .1 AND YMIN > -.1 THEN 4560                'format selection
4540   PRINT USING "+###.##"; YMAX - YSCALE * J%      'print y-scale divisions
4550   GOTO 4580
4560   PRINT USING "+#.##^^^^"; YMAX - YSCALE * J%    'print y-scale divisions
4570 '
4580   IF J% <> 2 THEN 4630              'print names of variables along x & y
4590    LOCATE ROWX% - 2, PXMAX% / 8 + 3: PRINT XTITLE$; "->"
4600    LOCATE ROWY% - 1, COLY% - 1: PRINT "^"
4610    LOCATE ROWY%, COLY% - 1: PRINT "|"
4620    LOCATE ROWY% + 1, COLY% - 1: PRINT YTITLE$
4630 NEXT J%
4640 '
4650 RETURN
4660 '
5000 '************************************************************************
5005 'subroutine to display graph
5010 'input variables:  T, Q, I, VL, DI, KQ1, KQ2, KQ3, KQ4
5015 '                  KV1, KV2, KV3, KV4, KI1, KI2, KI3, KI4
5020 'output variables: none
5030 'global variables: none
5040 'local variables: TL1,TL2,PL1,PL2, TV1,TV2,PV1,PV2, TR1,TR2,PR1,PR2
5050 '************************************************************************
5060 '
5070 FOR I% = 0 TO N%
5080                                        'perform scale transformations for
5090 TQ2 = KQ1 * T(I%) + KQ2: Q2 = KQ3 * Q(I%) + KQ4                        '1st graph
5100 TI2 = KI1 * T(I%) + KI2: I2 = KI3 * CUR(I%) + KI4                       '2nd graph
5110 TV2 = KV1 * T(I%) + KV2: VL2 = KV3 * VL(I%) + KV4                      '3rd graph
5120 '
5130 IF I% = 0 THEN 5180                               'jump if first time
5140 '
5150 LINE (TQ1, Q1)-(TQ2, Q2)
5160 LINE (TI1, I1)-(TI2, I2)
5170 LINE (TV1, VL1)-(TV2, VL2)
5180                                                       'roll variables for
5190 TQ1 = TQ2: Q1 = Q2                                             '1st graph
5200 TI1 = TI2: I1 = I2                                             '2nd graph
5210 TV1 = TV2: VL1 = VL2                                           '3rd graph
5220 '
5225 LOCATE 16, 48: PRINT USING "  t = ####.###"; T(I%)
5230   LOCATE 18, 48: PRINT "charge on C = ";
5240 IF ABS(Q(I%)) < 1000 THEN PRINT USING "###.###"; Q(I%) ELSE PRINT USING "#.###^^^^"; Q(I%)
5250   LOCATE 20, 48: PRINT "Current = ";
5260 IF ABS(CUR(I%)) < 1000 THEN PRINT USING "###.###"; CUR(I%) ELSE PRINT USING "#.###^^^^"; CUR(I%)
5270   LOCATE 22, 48: PRINT "Voltage at L  = ";
5280 IF ABS(VL(I%)) < 1000 THEN PRINT USING "###.###"; VL(I%) ELSE PRINT USING "#.###^^^^"; VL(I%)
5290                                                     'print instant values
5300 NEXT I%
5310 '
5320 RETURN
5330 '
10000 '***********************************************************************
10010 'subroutine to display header screen
10020 'input variables:  none
10030 'output variables: none
10040 'global variables: none
10050 'local variables:  none
10060 '***********************************************************************
10070 CLS : KEY OFF                                           'clear screen
10080 SCREEN 2, 0, 0: WIDTH 80                               'change screen
10090  '
10100 LINE (1, 1)-(638, 40), , B                                'draw boxes
10110 LINE (5, 3)-(633, 37), , B
10120 LINE (70, 50)-(570, 140), , B
10130 LINE (75, 53)-(565, 137), , B
10140                                           'print title, authors, etc.
10150 LOCATE 3, 7
10160 PRINT " C O M P U T A T I O N A L   P H Y S I C S :";
10170 PRINT " (An  Introduction)"
10180 LOCATE 9, 40
10190 PRINT "by";
10200 LOCATE 11, 20: PRINT "R.C. Verma, P.K. Ahluwalia & K.C. Sharma"
10210 LOCATE 13, 34
10220 PRINT "Copyright 1995"
10230 '
10240 BEEP
10250 LOCATE 15, 35: PRINT "* * * * * *"
10260 LOCATE 22, 25: PRINT "press any key to continue"
10270 LOCATE 23, 25: PRINT "-------------------------"
10280 '
10290 IF INKEY$ = "" THEN 10290                            'hold the screen
10300 CLS                                                     'clear screen
10310 RETURN
10320 '

