10 CLS: KEY OFF                     'program begins in text mode
20 'Solution of radial Schrodinger equation  (for 3 dim H.O.)
30 ' using Numerov method
40 PI=3.1415927#
50 ESCALE=1
60 NPOINTS%=200                                     'mesh points
70                                               'h.O. potential
80 DEF FNV(X)=CONST2*(X/ALFA)^POW2
90 DIM V(NPOINTS%), PSI(NPOINTS%)       'dimension the arrays
100 TOLF=.00005                                  'matching tolerance
110 '
120 GOSUB 1000                                   'get inputs
130 GOSUB 2000                                   'input potential
140 GOSUB 4000                                  'graph potential
150 '
160 FOR N%=1 TO 10                            'loop over radial q. no.
170 GOSUB 5000                                  'draw axes for psi
180   LOCATE 1,50:PRINT "n = ";N%
190   SECAN%=0:  BEGIN%=-1:  F=TOLF            'set values for E search
200   WHILE ABS(F)>= TOLF
210     GOSUB 3000                             'solve Schrodinger eq.
220     GOSUB 6000                             'plot wavefunction
225      LOCATE 17,10:PRINT USING "energy = +###.### ";E
230     IF BEGIN% THEN FBEGIN=F               'Save first F value
240     IF F*FBEGIN<0 THEN SECAN%=-1          'then when F switches sign
250     IF F=FOLD GOTO 270                     'use the secant search
260     IF SECAN% THEN DE=-F*(E-EOLD)/(F-FOLD) 'scale energy step
270     EOLD=E:  FOLD=F                        'store old values
280     E=E+DE                                 'increment Energy
290     BEGIN%=0                               'reset the variable
300   WEND
310 '
320   LOCATE 17,10
330   PRINT USING  "energy = +###.### ";E
340   LOCATE 23,15:  BEEP
350   PRINT  "Bound state searched.  Press any key to continue"
360   IF INKEY$=""  THEN 360                    'hold screen
370   DE=DESAVE                                 'recall energy step
380   EIGENE=EOLD                               'save eigenvalue
390   E=EOLD+DE                                 'new trial energy
400 NEXT N%                                      ' next search
410 '
420 END
430 '
1000 'subroutine to get parameters of problem
1010  CONST2=.5:POW2=2               'H.O. potential
1020 INPUT "r max = ";XMAX2
1030 INPUT "reduced mass  = ";MASS
1040 INPUT " alfa parameter = ";ALFA
1050 INPUT "guess energy = ";E
1060 INPUT "energy step size DE = ";DE
1070  DESAVE=DE                             'store the energy step
1080  EIGENE=-1                             'minimum value for E
1090 RETURN
1100 '
2000 'subroutine to calculate potential value
2010  XMIN2=0
2020   DX2=(XMAX2-XMIN2)/NPOINTS%
2030   XMIN=XMIN2:  DX=DX2
2040 '
2050  FOR I%=0 TO NPOINTS%
2060   V(I%)=FNV(XMIN+DX*I%)            'find potential at all lattice points
2070  NEXT I%
2080 '
2090 RETURN
2100 '
3000 'subroutine to solve Schroedinger equation
3005 C=(DX*DX/12)*GAMMA*GAMMA                   'constant factor
3010 MATCHP%=0                                  'clear matching point
3015 '
3020 PSI(0)=0                               'initial conditions for l.h.s.
3025 PSI(1)=9.999999E-10
3030 '
3035 KIM1=C*(E-V(0))                            'initial K*K values
3040 KI=C*(E-V(1))
3045 '
3050 FOR I%=1 TO NPOINTS%-1                        'Numerov algorithm
3055   KIP1=C*(E-V(I%+1))
3060   IF (KI*KIP1<=0 AND KI>0) THEN MATCHP%=I%: GOTO 3105   'matching point
3065   PSI(I%+1)=(PSI(I%)*(2-10*KI)-PSI(I%-1)*(1+KIM1))/(1+KIP1)
3070   IF ABS(PSI(I%+1))<(1E+10) GOTO 3090     'if PSI grows too big
3075   FOR K%=1 TO I%+1                        ' rescale all previous points
3080     PSI(K%)=PSI(K%)*9.999999E-06
3085   NEXT K%
3090   KIM1=KI:  KI=KIP1                       'roll values of K*K
3095 NEXT I%
3100 '
3105 IF MATCHP%=0 THEN MATCHP%=NPOINTS%-10      'if there is no turning point
3110                                            ' match wave functions near rhs
3115 PSIMMTCH=PSI(MATCHP%)                      'save value for normalization
3120 '
3125 PSI(NPOINTS%)=0                            'initial conditions for r.h.s.
3130 PSI(NPOINTS%-1)=9.999999E-10
3135 '
3140 KIP1=C*(E-V(NPOINTS%))                        'initial K*K value
3145 KI=C*(E-V(NPOINTS%-1))
3150 '
3155 FOR I%=NPOINTS%-1 TO MATCHP%+1 STEP -1        'Numerov algorithm
3160   KIM1=C*(E-V(I%-1))
3165   PSI(I%-1)=(PSI(I%)*(2-10*KI)-PSI(I%+1)*(1+KIP1))/(1+KIM1)
3170   IF ABS(PSI(I%-1))<(1E+10) GOTO 3195     'if PSI grows too big
3175   FOR K%=NPOINTS%-1 TO I%-1 STEP -1        'rescale all points
3180     PSI(K%)=PSI(K%)*9.999999E-06
3185   NEXT K%
3190 '
3195   KIP1=KI:  KI=KIM1                       'roll values of K*K
3200 NEXT I%
3205 '
3210 KIM1=C*(E-V(MATCHP%-1))    'finds value needed for log derivative
3215 PSIPMTCH=(PSI(MATCHP%)*(2-10*KI)-PSI(MATCHP%+1)*(1+KIP1))/(1+KIM1)
3220 '
3225 NORM=PSI(MATCHP%)/PSIMMTCH          'norm PSI left to PSI right
3230 '
3235 FOR I%=1 TO MATCHP%-1               'keep signs unchanged
3240    PSI(I%)=PSI(I%)*ABS(NORM)
3245 NEXT I%
3250 '
3255 IF NORM>0 THEN GOTO 3285                   'flip sign of PSI right if
3260   FOR I%=MATCHP% TO NPOINTS%                  'NORM is negative
3265      PSI(I%)=-PSI(I%)
3270   NEXT I%
3275   PSIPMTCH=-PSIPMTCH
3280 '
3285 PSIMAX=ABS(PSI(1))                         'find maximum PSI value
3290 FOR I%=1 TO NPOINTS%
3295   IF ABS(PSI(I%))>PSIMAX THEN PSIMAX=ABS(PSI(I%))
3300 NEXT I%
3305 PSIMAX= 1.1*PSIMAX : PSIMIN=-PSIMAX       'limits for graphics
3310 '
3315 F=(PSI(MATCHP%-1)-PSIPMTCH)/PSIMAX         'evaluate matching condition
3320 '
3325 RETURN
3330 '
4000 'subroutine to draw potential function
4010 SCREEN 2,0,0,0: WIDTH 80 :CLS
4020 XMIN=0 :XMAX=XMAX2 :VMIN=0: VMAX = FNV(XMAX2) 'max, min limits for graphics
4030  GAMMA2=2*MASS/ALFA^2:GAMMA=SQR(GAMMA2)  'useful factor
4040 XSCALE=500/(XMAX-XMIN)          'scales for x- & y-axes
4050 YSCALE=150/(VMAX-VMIN)
4060 LINE (40,8)- (540,158), ,B         'draw the axes
4070 LOCATE 1,10:PRINT "Potential V(r) vs. r"
4080 '                               draw potential
4090  FOR I% = 0 TO NPOINTS%
4100   X2%=(XMIN+I%*DX)*XSCALE+40
4110   Y2%=(VMAX-V(I%))*YSCALE+8
4120   IF I%> 0 THEN LINE -(X2%,Y2%) ELSE PSET(X2%,Y2%)  'draw curve
4130 NEXT I%
4140 '
4150 LOCATE 23,1:PRINT "press any key to continue" 'hold screen
4160 IF INKEY$="" THEN 4160
4170 RETURN
4180 '
5000 'subroutine to draw axes of the wavefunction
5010 CLS
5020 XSCALE=500/(XMAX-XMIN)          'scales for x- & y-axes
5030 LINE (40,8)- (540,158), ,B         'draw the axes
5040 LINE (40,83) - (540,83)        'draw axes
5050 LOCATE 1,10:PRINT "Wavefunction u(r) vs. r"
5060 RETURN
5070 '
6000 'subroutine to draw the wavefunction
6010 LINE (41,9)-(539,82), 0, BF        'clear boxes
6020 LINE (41,84)-(539,157), 0, BF
6030 YSCALE=150/(PSIMAX-PSIMIN)         'find scale
6040 '                            draw wavefn from l.h.s.
6050  FOR I% = 0 TO MATCHP%
6060   X2%=(XMIN+I%*DX)*XSCALE+40
6070   Y2%=(PSIMAX-PSI(I%))*YSCALE+8
6080   IF I%> 0 THEN LINE -(X2%,Y2%) ELSE PSET(X2%,Y2%)  'draw psi
6090 NEXT I%
6100 '                             draw wavefn from r.h.s.
6110 FOR I% = NPOINTS% TO MATCHP% STEP -1
6120   X2%=(XMIN+I%*DX)*XSCALE+40
6130   Y2%=(PSIMAX-PSI(I%))*YSCALE+8
6140   IF I%< NPOINTS% THEN LINE -(X2%,Y2%) ELSE PSET(X2%,Y2%) 'draw psi
6150  NEXT I%
6160 '
6170 RETURN
