10 CLS:KEY OFF
20 'This program analyzes a driven LCR circuit
30  DEF FNV(T)=VI*SIN(W*T)       'define voltage function
40  DEF FNI(Q,I,T)=-(Q/C +R*I-FNV(T))/L 'define LCR equation
50 DIM T(1505),I(1505),Q(1505)
60 PI=3.1415926#
70 '                                    'give circuit parameters
80  INPUT "Capacitance & Inductance of the circuit ";C,L
90  INPUT "Resistance in the circuit ";R  :PRINT
100 INPUT " initial charge on the Capacitor ";QI
110 INPUT " initial current ";I0 :PRINT
120 INPUT "amplitude & frequency of driver = ";VI,W :PRINT
130 INPUT "time step  0 < t < 0.1  = ",H
140 INPUT "maximum time = ";TMAX :PRINT
150 '
160 N%=TMAX/H                         'find number of time steps
170 I(0)=I0 :T(0)=0 :Q(0)=QI          'initial conditions
180 IMAX=0:IMIN=0:QMAX=0:QMIN=0:TMIN=0 'intialize limits for graphics
190 '
200 FOR  J = 0 TO  N%                 'loop over time steps
210    T(J)=H*J                       'find current time
220   PRINT USING "t = ###.###  q = +###.####  i = +###.####";T(J),Q(J),I(J)
230   K1 = H*I(J)               'Runge-Kutta coefficients
240    P1 = H*FNI(Q(J),I(J),T(J))
250   K2 = H*(I(J)+P1/2)
260    P2 = H*FNI(Q(J)+K1/2,I(J)+P1/2,T(J)+H/2)
270   K3 = H*(I(J)+P2/2)
280    P3 = H*FNI(Q(J)+K2/2,I(J)+P2/2,T(J)+H/2)
290   K4 = H*(I(J)+P3)
300    P4 = H*FNI(Q(J)+K3,I(J)+P3,T(J)+H)
310   Q(J+1)=Q(J)+(K1+2*K2+2*K3+K4)/6   'charge   formula
320   I(J+1)=I(J)+(P1+2*P2+2*P3+P4)/6   'current  formula
330   '
340   IF I(J)> IMAX THEN IMAX=I(J)    'find limits for graphics
350   IF I(J)< IMIN THEN IMIN=I(J)
360   IF Q(J)< QMIN THEN QMIN=Q(J)
370   IF Q(J)> QMAX THEN QMAX=Q(J)
380 NEXT J
390 '
400 PRINT :PRINT "press any key to continue"  :PRINT
410 IF INKEY$="" THEN 410             'hold screen
420 GOSUB 1000                         'draw curves for q & i
430 END                               'main program ends here
440 '
1000 'subroutine to graph
1010 SCREEN 2,0,0:WIDTH 80:CLS   'change screen
1020 LOCATE 1,2 : PRINT "charge vs. time"
1030 LOCATE 11,2: PRINT "current vs. time"
1040 LOCATE 22,50: PRINT "Charge & current in a driven LCR circuit"
1050 '
1060 ZSCALE=75/(IMAX-IMIN)             'find scales
1070 YSCALE=75/(QMAX-QMIN)
1080 XSCALE=500/(TMAX-TMIN)
1090 '
1100 SC1=QMAX*YSCALE: VC1=IMAX*ZSCALE   'draw axes for graphs
1110 LINE (70,SC1+5)-(570,SC1+5)
1120 LINE (70,90+VC1)-(570,90+VC1)
1130 '
1140 FOR I%=0 TO N%       'scale transformations & draw curves
1150    I(I%)=(IMAX-I(I%))*ZSCALE+90
1160    Q(I%)=(QMAX-Q(I%))*YSCALE+5
1170    T(I%)=(T(I%)-TMIN)*XSCALE+70
1180   IF I%=0 THEN 1210
1190    LINE (T(I%-1),Q(I%-1))-(T(I%),Q(I%)) 'curve for q vs t
1200    LINE (T(I%-1),I(I%-1))-(T(I%),I(I%))  'curve f or i vs t
1210 NEXT I%
1220 '
1230 RETURN
